      subroutine bioegg(filnam, jdisk, ioro, inout, text, ical, rcal,
     &   spec, ninout, ierr)
*---Purpose:
*     EG&G MCS Plus binary format.
*     This routine reads or writes one or more spectra in an 
*     unformatted, direct-access file, though it currently accesses
*     this file only sequentially.  Each spectrum consists of a
*     single 
*     256 byte header and as many following blocks each
*     containing 2048 bytes of data as the length of the spectrum
*     requires. 
*--- Contact- OLD: WEB Research Co. /// NEW: WMOSS.ORG
*---orig: tk 950706
*--- mru: tk 971226
*-----------------------------------------------------------------------
*---common blocks and parameters
      include 'limits.inc'
      include 'errcodes.inc'
*---externals
      character*(MFLLEN) filnam
      character text(MCOMMT)*72
      integer ioro, irec, ierr, jdisk, inout, ninout
      integer ical(MCAL)
      real rcal(MCAL), spec(LSPEC)
*-----------------------------------------------------------------------
*
*     filnam : [In/Out] Array of names; first is filename to access;
*     jdisk  : [Input]  FORTRAN unit number to use;
*     ioro   : [Input]  Read/write toggle: 0 to read, 1 to write;
*     inout  : [Input]  Reading: Maximum number of spectra to read;
*            :          Writing: Number of spectra to write;
*     text   : [In/Out] Array of six-line comments to read/write;
*     ical   : [In/Out] Array of integer calibration constants to read/write;
*     rcal   : [In/Out] Array of real calibration constants to read/write;
*     spec   : [In/Out] Array of spectra to read/write;
*     ninout : [Output] Number of spectra read or written;
*     ierr   : [Output] Error code.
*--------------------------------------------------------------------------
*---internals 
c     ;; All records have a length of 256 bytes
      integer LREC
      parameter (LREC = 256)
      real tmpspc(2*lspec)
      integer*4 i4arr(LREC/4)
      integer i, j, ierr1
      character  pad10*10, pad4*4,time8*8,date8*8,pad212*212,mcsid8*8
      integer*2 nchan2
      integer*4 nswp4
      integer nblocks, ndrec
*---number of channels in unfolded and folded data
      integer nuf,nf

*--------------------------------------------------------------------------
*---executables

*---EG&G MCA plus files have only one spectrum per file
      if(inout.lt.1)then
         ierr=-2
         call errmsg('In routine BIOEGG: inout .lt. 1')
         return
      endif

*---Attempt to open file
      if ((ioro.eq.0).and.(inout.gt.0)) then
c        ;;;;;;;;;;;;;;;;;;
c        ;; Read spectra ;;
c        ;;;;;;;;;;;;;;;;;;
         open (jdisk, file=filnam, access='DIRECT',
     &      form='UNFORMATTED', recl=lrec, status='OLD', err=1100,
     &      iostat=ierr)
*---read header
         irec = 1
            read (jdisk, rec=irec, iostat=ierr) 
     &         pad10,
     &         nchan2,
     &         nswp4,
     &         pad4,time8,date8,
     &         pad212,mcsid8
            if ( ierr.ne.0) then
               call errmsg('Error reading header in routine BIOEGG.')
               goto 1000
            elseif(nchan2.lt.0 .or. nchan2.gt.2*LSPEC) then
                  call errmsg('~WARNING~:  EG&G binary file too long.')
                  ierr = ErrBadFileFormat
               goto 1000
            elseif( mcsid8.ne.'0923-000') then
                  call errmsg('This file is not a EG&G MCS+ file.')
                  ierr = ErrBadFileFormat
               goto 1000
            endif
*---read data
*---EG&G data stored as 2048 byte blocks (512 *4 byte integers)
*---Because of 256 byte header length, our record length is 256 bytes
*---(256 bytes = 64 *4 byte integers)
            nblocks = 1 + (nchan2-1)/512
c            ndrec = 8 * nblocks
            ndrec = 1 +(nchan2-1)/64
            do 100 i = 0, ndrec-1
               irec = irec + 1
               read(jdisk, rec=irec, iostat=ierr) i4arr
               do 90 j=1,64
                  tmpspc(j + i*64) = i4arr(j)
   90          continue
100   continue
*patch tk 950707; tk 971226
*---force 1st channel to be equal to 2nd channel
      tmpspc(1)=tmpspc(2)
*---transfer number of channels to 4 byte integer
      nuf = nchan2
*----fold spectrum
      CALL FALT(filnam, nuf, tmpspc, nf, spec, rcal(5), rcal(6),
     &   rcal(7), ierr)
      if (ierr.eq.0) then
*---define cal header info.
         ical(1) = nf
         ical(2) = 1
         do 200 i=3, 10
            ical(i) = 0
  200    continue
         do 210 i = 1,4
            rcal(i) = 0.0
  210    continue
         do 220 i = 8,10
            rcal(i) = 0.0
  220    continue
      text(2)= date8 // ' ' // time8
*---set file type (see readsp.for)
       ical(5)=10
      else
         call errmsg('BIOEGG: FALT: error!')
         goto 1000
      endif

      elseif ((ioro.eq.1).and.(inout.gt.0)) then
c        ;;;;;;;;;;;;;;;;;;;
c        ;; Write spectra ;;
c        ;;;;;;;;;;;;;;;;;;;
c         open (jdisk, file=filnam, access='DIRECT',
c     &      form='UNFORMATTED', recl=lrec, status='UNKNOWN', err=1100,
c     &      iostat=ierr)
c
      endif

 1000 close (jdisk, iostat=ierr1)
      if (ierr.eq.NoError) then
       ninout = 1
       ierr = ierr1
      else
        ninout =0
      endif
 1100 return
      end
