*-----------------------------------------------------------------------
      subroutine calbas(nsppts,dspec,lospc,adjdat,subsp,subvec,rhs,bl)
*-----------------------------------------------------------------------
*--- Contact- OLD: WEB Research Co. /// NEW: WMOSS.ORG
*     mru 20Jul93 cs
*     Sets up the basis functions--baselines and subspectra--for the
*     linear fitting routine LINFIT.
*     MOSS/MAG/EPR Version: Assumes 1 area fraction/subsite
*                         : and 1 baseline value per spectrum.
*
*     nsppts : [Input]  list of the number of points in each spectrum;
*     dspec  : [Input]  Data the vectors should approximate;
*     lospc  : [Input]  Array of subspectra bases;
*     adjdat : [Output] Final data vector to fit TO
*            :          (DSPEC after removing constant bases);
*     subsp  : [Output] Final bases to fit WITH [floating bases];
*     subvec : [Output] Floating points from subsp [basis vectors];
*     rhs    : [Output] constant bases to be added back after the 
*            :          linear fit. (rhs+adjdat=dspec)
*     bl     : [Output] List of flags, one for each basis vector;
*            :          true if the vector is a baseline.
*-----------------------------------------------------------------------
      include 'limits.inc'
      include 'parcom.inc'
      include 'varcom.inc'
      real baseln(maxpts,lt1)
      common /bascom/ baseln

      integer nsppts(LT1)
      logical bl(MBASIS)
      real rhs(LSPEC,LT1),dspec(LSPEC,LT1)
      real lospc(LSPEC,LT1,MLOC),subsp(LSPEC,LT1,MBASIS)
      real adjdat(MAXPTS), subvec(MAXPTS,MBASIS)

*-----------------------------------------------------------------------
      integer i, j, k, isub, ispec, ivec, nbases
      real y(LSPEC,LT1)
*-----------------------------------------------------------------------
      integer ij2kon
      ij2kon(i,j) = i + 100 * j
*-----------------------------------------------------------------------

*---Initialize bl
      do 50 i = 1, mbasis
         bl(i) = .false.
   50 continue

*---Clear basis vectors
      do 60 k = 1, MBASIS
         do 70 j = 1, nspec
            do 80 i = 1, nsppts(j)
               subsp(i,j,k) = 0.
   80       continue
   70    continue
   60 continue

*---Clear RHS and initialize Y
      do 100 j = 1, nspec
         do 90 i = 1, nsppts(j)
            rhs(i,j) = 0.
            y(i,j) = dspec(i,j)
   90    continue
  100 continue

*---Copy DSPEC into Y, subtracting fixed baselines
      do 200 j = 1, nspec
         if (konbl(1,j).eq.0) then
            do 120 i = 1, nsppts(j)
               rhs(i,j) = rhs(i,j) + pbl0(1,j)
               y(i,j) = y(i,j) - pbl0(1,j)
  120       continue
         elseif (indbl(1,j).lt.0) then
            do 125 i = 1, nsppts(j)
               rhs(i,j) = rhs(i,j) + pbl0(1,-indbl(1,j))
               y(i,j) = y(i,j) - pbl0(1,-indbl(1,j))
  125       continue
         endif
  200 continue
 
*---ADJUST Y FOR CONSTANT SPECTRUM COEFFICIENTS
      do 300 isub = 1, nsubs
         if (konln(1,isub).eq.0) then
            do 210 j = 1, nspec
               do 220 i = 1, nsppts(j)
                  rhs(i,j) = rhs(i,j) + pln0(1,isub)*lospc(i,j,isub)
                  y(i,j) = y(i,j) - pln0(1,isub)*lospc(i,j,isub)
  220          continue
  210       continue
         elseif (indln(1,isub).lt.0) then
            do 225 i = 1, nsppts(j)
               rhs(i,j) = rhs(i,j) + pbl0(1,-indbl(1,j))
               y(i,j) = y(i,j) - pbl0(1,-indbl(1,j))
  225       continue
         endif
  300 continue
 
*---ADD FLOATING BASELINES
      nbases = 0
      do 400 ispec = 1, nspec
         if (indbl(1,ispec).gt.0) then
            if (konbl(1,ispec).eq.ij2kon(1,ispec)) then
               do 310 j = 1, nspec
                  do 305 i = 1, nsppts(j)
                     subsp(i,j , nlnfr + indbl(1,ispec)) =
     &                  subsp(i, j, nlnfr + indbl(1,ispec)) +
     &                  baseln(i + (j-1) * lspec, ispec)
  305             continue
  310          continue
               bl(nlnfr + indbl(1,ispec)) = .true.
               nbases = nbases + 1
            else
               do 320 j = 1, nspec
                  do 315 i = 1, nsppts(j)
                     subsp(i, j, nlnfr + indbl(1,ispec)) =
     &                  subsp(i,j,nlnfr + indbl(1,ispec)) +
     &                  baseln(i+(j-1)*lspec,ispec) * weibl(1,ispec)
  315             continue
  320          continue
            endif
         endif
  400 continue
 
*---ADD FLOATING SPECTRA
      do 500 isub = 1, nsubs
         if (indln(1,isub).gt.0) then
            if (konln(1,isub).eq.ij2kon(1,isub)) then
               do 410 j = 1, nspec
                  do 405 i = 1, nsppts(j)
                     subsp(i,j,indln(1,isub)) =
     &                  subsp(i,j,indln(1,isub)) + lospc(i,j,isub)
  405             continue
  410          continue
               nbases = nbases + 1
            else
               do 420 j = 1, nspec
                  do 415 i = 1, nsppts(j)
                     subsp(i,j,indln(1,isub)) = 
     &                  subsp(i,j,indln(1,isub)) +
     &                  lospc(i,j,isub) * weiln(1,isub)
  415             continue
  420          continue
            endif
         endif
  500 continue

*---Apply mask to theory
      do 600 ivec = 1, nbases
         call mskapp(nsppts, subsp(1,1,ivec), smask(1,1),
     &      subvec(1,ivec), nfrpt)
600   continue

*---Apply mask to Y data
      call mskapp(nsppts, y, smask(1,1), adjdat, nfrpt)
 
      return
      end
