      SUBROUTINE CALFUN(SUBFUN,NFREE,VFREE1,VLB,VUB,IBOUND,IGRAD,NPOINT,
     &                  CHI2,GRAD,FVEC,IERR)
*-----------------------------------------------------------------------
*--- Contact- OLD: WEB Research Co. /// NEW: WMOSS.ORG
*---orig: 23May94 CS
*---mru:  951226 tk
*-----------------------------------------------------------------------
*     Main CAL?? shell gradient calculation routine for all levels.
*
*     SUBFUN : [input]  Subroutine to call to do actual calculations;
*     nfree  : [input]  Number of independent global variables;
*     vfree1 : [input]  Values for global variables;
*     ibound : [input]  Penalty function
*            :          = 0 => Ignore Bounds
*            :          < 0 => Return Large chi^2, do not do simulation,
*            :                   DO NOT USE THIS AND ALSO
*            :                   CALC. GRADIENT OR USE FVEC!
*            :          > 0 => penalty function :
*            :                   add SIGMA ((v-bound)/(vub-vlb))**ibound
*            :                   to the chi^2 of the closest in-bound point.
*     igrad  : [input]  Gradient flag
*            :            0 => Do not calculate gradient
*            :            1 => Calculate gradient (forward difference)
*            :            2 => Calculate gradient (central difference);
*     npoint : [input]  Number of unmasked data points;
*     vlb    : [input]  Free variable lower bounds;
*     vub    : [input]  Free variable upper bounds;
*     chi2   : [output] Chi-squared at given point;
*     grad   : [output] Gradient of chi-squared at given point;
*     fvec   : [output] Residuals: (Data-Theory)/Noise;
*     ierr   : [output] Result code
*            :            0 => No error
*            :          <>0 => Abort global optimization;
*
*     Notes:  1) If changed, INFITY / H must not overflow.
*             2) The penalty function is applied to _residuals_;
*                the function _squared_ is applied to chi-squared.
*-----------------------------------------------------------------------
      INTEGER NFREE,IBOUND,IGRAD,NPOINT,IERR
      REAL VFREE1(NFREE),VLB(NFREE),VUB(NFREE)
      REAL CHI2,GRAD(NFREE),FVEC(NPOINT)
      EXTERNAL SUBFUN
*-----------------------------------------------------------------------
      INCLUDE 'limits.inc'
      INCLUDE 'parcom.inc'
      INCLUDE 'varcom.inc'
      INCLUDE 'bestcom.inc'
      INCLUDE 'errcodes.inc'
      INCLUDE 'togcom.inc'
 
      REAL INFITY,H
      PARAMETER (INFITY=1.0E10,H=1.0E-4)
c     /* Make sure INFITY divided by H does not overflow! */
 
      INTEGER MSIMS
      PARAMETER (MSIMS=2*MVAR+1)
 
      INTEGER NSIMS,ISIM,I
      REAL VFREES(MVAR,MSIMS),CHI2S(MSIMS)
      REAL PENLTY(MSIMS),DIST
      LOGICAL ZINBND
*-----------------------------------------------------------------------
 
      IERR = NOERROR
 
c     /* Setup the list of global simulations to do (depends on igrad) */
      NSIMS = 1
      CALL VCOPY(NFREE,VFREE1,VFREES(1,NSIMS))
 
      IF ( IGRAD.EQ.1 ) THEN
 
c        /* Forward differences */
        DO 50 I = 1,NFREE
          NSIMS = NSIMS + 1
          CALL VCOPY(NFREE,VFREE1,VFREES(1,NSIMS))
          VFREES(I,NSIMS) = VFREES(I,NSIMS) + H
 50     CONTINUE
      ELSEIF ( IGRAD.EQ.2 ) THEN
 
c        /* Central differences */
        DO 100 I = 1,NFREE
          NSIMS = NSIMS + 1
          CALL VCOPY(NFREE,VFREE1,VFREES(1,NSIMS))
          VFREES(I,NSIMS) = VFREES(I,NSIMS) - H
          NSIMS = NSIMS + 1
          CALL VCOPY(NFREE,VFREE1,VFREES(1,NSIMS))
          VFREES(I,NSIMS) = VFREES(I,NSIMS) + H
 100    CONTINUE
      ENDIF
 
 
c     /* Worry about bounds */
      DO 200 ISIM = 1,NSIMS
        CALL TESTIN(NFREE,VFREES(1,ISIM),VUB,VLB,ZINBND)
 
        IF ( ZINBND .OR. IBOUND.EQ.0 ) THEN
          PENLTY(ISIM) = 1.
        ELSEIF ( IBOUND.LT.0 ) THEN
          CHI2S(ISIM) = INFITY
          GOTO 300
        ELSE
          CALL FORCIN(NFREE,VFREES(1,ISIM),VUB,VLB,IBOUND,DIST)
          PENLTY(ISIM) = 1.0 + DIST
        ENDIF
 200  CONTINUE
 
 
c     /* Do real calculations */
      CALL SUBFUN(NFREE,MVAR,NSIMS,VFREES,NPOINT,PENLTY,CHI2S,FVEC,IERR)
 
c     /* Return chi-squared at initial point */
        CHI2 = CHI2S(1)

c     /* Check for immediate abort */
      IF ( (IERR.EQ.0) .AND. (ABORT.LT.ABORTFIT) ) THEN
 
c     /* Gradient Calculations */
        IF ( IGRAD.EQ.1 ) THEN
          DO 220 I = 1,NFREE
            GRAD(I) = (CHI2S(I+1)-CHI2S(1))/H
 220      CONTINUE
        ELSEIF ( IGRAD.EQ.2 ) THEN
          DO 240 I = 1,NFREE
            GRAD(I) = (CHI2S(2*I+1)-CHI2S(2*I))/(2*H)
 240      CONTINUE
        ENDIF
 
      ENDIF
 
c     /* Return */
 300  RETURN
      END
