*-----------------------------------------------------------------------
      SUBROUTINE CALGA(NFREE,MFREE,NSIMS,VFREES,NPOINT,PENLTY,CHI2S,
     &                 FVEC,IERR)
*-----------------------------------------------------------------------
*--- Contact- OLD: WEB Research Co. /// NEW: WMOSS.ORG
*---orig: 20May94 CS
*---mru : 951224 tk
*-----------------------------------------------------------------------
*     Main calculation routine for global area level.
*
*     nfree  : [input]  Number of independent global variables;
*     mfree  : [input]  Leading declared dimension of VFREES array;
*     nsims  : [input]  Number of global simulations requested;
*     vfrees : [input]  Values for global variables for each simulation;
*     npoint : [input]  Number of unmasked data points;
*     penlty : [input]  Penalty values for simulations;
*     chi2s  : [output] Chi-squared for each simulation;
*     fvec   : [output] Residuals of first simulation: (Data-Theory)/Noise;
*     ierr   : [output] Result code
*            :            0 => No error
*            :          <>0 => Abort global optimization;
*
*     Notes:  1) One local iteration takes place at each
*                global area optimization---This is a change from the
*                old SOPTMZ method of counting in which one global
*                iteration could involve multiple local optimizations.
*                In particular, one call to CALLC may take n+1 or 2n+1
*                local function for a single gradient calculation.
*-----------------------------------------------------------------------
      INTEGER NFREE,MFREE,NSIMS,NPOINT,IERR
      REAL VFREES(MFREE,NSIMS),PENLTY(NSIMS)
      REAL CHI2S(NSIMS),FVEC(NPOINT)
*-----------------------------------------------------------------------
      INCLUDE 'limits.inc'
      INCLUDE 'parcom.inc'
      INCLUDE 'varcom.inc'
      INCLUDE 'bestcom.inc'
      INCLUDE 'errcodes.inc'
      INCLUDE 'togcom.inc'
 
      EXTERNAL CALBAS
 
      INTEGER ISIM,I,J,ISPEC,NSPPTS(LT1),IDUMMY
      REAL FARR(LSPEC,LT1)
*-----------------------------------------------------------------------
 
      IERR = NOERROR
 
C     /* Simulation loop */
      DO 200 ISIM = 1,NSIMS
 
c        /* Construct full global area parameter set PABEST */
        CALL CNSAPP(NFREE,VFREES(1,ISIM),MAPAR,NSPEC,PA0,PALB,PAUB,
     &              PABEST,INDA,KONA,WEIA)
 
c        /* Do global area part of simulation using PABEST parameters */
        CALL ULOGAR()
 
c        /* Do linear fit */
        DO 50 ISPEC = 1,NSPEC
          NSPPTS(ISPEC) = ICAL(1,ISPEC)
 50     CONTINUE
        CALL LINFIT(NLNFR,VLNB,NBLFR,VBLB,NSPPTS,SPEC,SLCSPC,SNOYS,
     &              CALBAS,CHI2S(ISIM),SPEC(1,1,2))
 
c        /* Apply penalty function to chi^2 */
        CHI2S(ISIM) = CHI2S(ISIM)*PENLTY(ISIM)**2
 
c        /* Return fvec if first simulation */
        IF ( ISIM.EQ.1 ) THEN
          DO 80 J = 1,NSPEC
            DO 60 I = 1,NSPPTS(J)
              FARR(I,J) = SPEC(I,J,1) - SPEC(I,J,2)
              IF ( SNOYS(I,J,1).GT.0 ) FARR(I,J) = FARR(I,J)
     &             /SNOYS(I,J,1)
 60         CONTINUE
 80       CONTINUE
          CALL MSKAPP(NSPPTS,FARR,SMASK,FVEC,IDUMMY)
          DO 100 I = 1,NPOINT
            FVEC(I) = FVEC(I)*PENLTY(1)
 100      CONTINUE
        ENDIF
 
c        /* Store and/or report as necessary */
        ITERGA = ITERGA + 1
 
        IF ( CHI2S(ISIM).LT.X2BSTA ) THEN
          X2BSTA = CHI2S(ISIM)
          CALL STORE(GARLEV,NFREE,VFREES(1,ISIM))
          REPLOT(GARLEV) = .TRUE.
        ENDIF
 
c     /* Check number of function evaluations */
        IF ( ITERGA.GE.MAXITA .and. abort.le.abortendopt) 
     2        ABORT = ABORTENDOPT
 
 200  CONTINUE
 
      RETURN
      END
