*-----------------------------------------------------------------------
      SUBROUTINE CALGL(NFREE,MFREE,NSIMS,VFREES,NPOINT,PENLTY,CHI2S,
     &                 FVEC,IERR)
*-----------------------------------------------------------------------
*--- Contact- OLD: WEB Research Co. /// NEW: WMOSS.ORG
*---orig: 23May94 CS
*---mru : 951224 tk
*-----------------------------------------------------------------------
*     Main calculation routine for global level.
*
*     nfree  : [input]  Number of independent global variables;
*     mfree  : [input]  Leading declared dimension of VFREES array;
*     nsims  : [input]  Number of global simulations requested;
*     vfrees : [input]  Values for global variables for each simulation;
*     npoint : [input]  Number of unmasked data points;
*     penlty : [input]  Penalty values for simulations;
*     chi2   : [output] Chi-squared at given point;
*     fvec   : [output] Residuals of first simulation: (Data-Theory)/Noise;
*     ierr   : [output] Result code
*            :            0 => No error
*            :          <>0 => Abort global optimization;
*
*     Notes:  1) Do we have to call INTFUN at every gradient point?
*                (to do anything besides resetting the local site)
*             2) One global iteration takes place at each
*                local optimization---This is a change from the
*                old SOPTMZ method of counting in which one global
*                iteration could involve multiple local optimizations.
*                In particular, one call to CALGL may take n+1 or 2n+1
*                local function for a single gradient calculation.
*-----------------------------------------------------------------------
      INTEGER NFREE,MFREE,NSIMS,NPOINT,IERR
      REAL VFREES(MFREE,NSIMS),PENLTY(NSIMS)
      REAL CHI2S(NSIMS),FVEC(NPOINT)
*-----------------------------------------------------------------------
      INCLUDE 'limits.inc'
      INCLUDE 'parcom.inc'
      INCLUDE 'varcom.inc'
      INCLUDE 'bestcom.inc'
      INCLUDE 'errcodes.inc'
      INCLUDE 'togcom.inc'
      INCLUDE 'mancom.inc'
 
      EXTERNAL CALLC1,CALLC2,CALLC3,CALLC4,CALLC5
 
      INTEGER ISIM,I,DOF
      REAL FVEC1(MAXPTS)
*-----------------------------------------------------------------------
 
      IERR = NOERROR
      DOF = NPOINT - NGFREE - NLFREE - NAFREE - NLNFR - NBLFR
 
C     /* Simulation loop */
      DO 100 ISIM = 1,NSIMS
 
c        /* Construct full global parameter set PGBEST */
        CALL CNSAPP(NFREE,VFREES(1,ISIM),MGPAR,1,PG0,PGLB,PGUB,PGBEST,
     &              INDG,KONG,WEIG)
 
c        /* Do global part of simulation using PGBEST parameters */
        CALL INTFUN(GNLTYP,IERR)
        IF ( IERR.NE.NOERROR ) THEN
          ABORT = ABORTFATAL
          GOTO 200
        ENDIF
        CALL ULOGLO(IERR)
        IF ( IERR.NE.NOERROR ) THEN
          ABORT = ABORTFATAL
          GOTO 200
        ENDIF
 
c        /* Do local fit */
        CALL CALOPT(NLFREE,VL0,VLLB,VLUB,NPOINT,MAXITL,LOCLEV,NOPTMZ,
     &              CALLC1,CALLC2,CALLC3,CALLC4,CALLC5,VLBEST,
     &              CHI2S(ISIM),FVEC1,IERR)
        IF ( IERR.NE.NOERROR ) GOTO 200
 
c        /* Apply global penalty function to residuals; adjust chi^2 */
        CHI2S(ISIM) = CHI2S(ISIM)*PENLTY(ISIM)**2
 
c        /* Return fvec if first simulation */
        IF ( ISIM.EQ.1 ) THEN
          DO 20 I = 1,NPOINT
            FVEC(I) = FVEC1(I)*PENLTY(1)
 20       CONTINUE
        ENDIF
 
c        /* Hopefully speed up convergence of the next local fit */
        CALL VCOPY(NLFREE,VLBEST,VL0)
 
c        /* Store and/or report as necessary */
        ITERG = ITERG + 1
 
        IF ( CHI2S(ISIM).LT.X2BSTG ) THEN
          X2BSTG = CHI2S(ISIM)
          CALL STORE(GLOLEV,NFREE,VFREES(1,ISIM))
          REPLOT(GLOLEV) = .TRUE.
        ENDIF
 
        IF ( NRPRTG.GT.0 ) THEN
          IF ( MOD(ITERG,NRPRTG).EQ.0 ) THEN
            CALL REPORT(GLOLEV,ITERG,X2BSTG/DOF,CHI2S(ISIM)/DOF)
            REPLOT(GLOLEV) = .FALSE.
 
            IF ( (ABORT.EQ.ABORTFIT) .OR. (ABORT.EQ.ABORTALL) ) THEN
              IERR = NOERROR
              GOTO 200
            ELSEIF ( ABORT.EQ.ABORTFATAL ) THEN
              IERR = ERRABORT
              GOTO 200
            ENDIF
          ENDIF
        ENDIF
 
c        /* Check number of function evaluations */
        IF ( ITERG.GT.MAXITG .and. abort.le.abortendopt ) 
     2             ABORT = ABORTENDOPT
 
 100  CONTINUE
 
 200  RETURN
      END
