      SUBROUTINE CALLC(NFREE,MFREE,NSIMS,VFREES,NPOINT,PENLTY,CHI2S,
     &                 FVEC,IERR)
*-----------------------------------------------------------------------
*--- Contact- OLD: WEB Research Co. /// NEW: WMOSS.ORG
*---orig: 23May94 CS
*---mru : 951224 tk
*-----------------------------------------------------------------------
*     Main calculation routine for local level.
*
*     nfree  : [input]  Number of independent global variables;
*     mfree  : [input]  Leading declared dimension of VFREES array;
*     nsims  : [input]  Number of global simulations requested;
*     vfrees : [input]  Values for global variables for each simulation;
*     npoint : [input]  Number of unmasked data points;
*     penlty : [input]  Penalty values for simulations;
*     chi2   : [output] Chi-squared at given point;
*     fvec   : [output] Residuals of first simulation: (Data-Theory)/Noise;
*     ierr   : [output] Result code
*            :            0 => No error
*            :          <>0 => Abort global optimization;
*
*     Notes:  1) One local iteration takes place at each
*                global area optimization---This is a change from the
*                old SOPTMZ method of counting in which one global
*                iteration could involve multiple local optimizations.
*                In particular, one call to CALLC may take n+1 or 2n+1
*                local function for a single gradient calculation.
*-----------------------------------------------------------------------
      INTEGER NFREE,MFREE,NSIMS,NPOINT,IERR
      REAL VFREES(MFREE,NSIMS),PENLTY(NSIMS)
      REAL CHI2S(NSIMS),FVEC(NPOINT)
*-----------------------------------------------------------------------
      INCLUDE 'limits.inc'
      INCLUDE 'parcom.inc'
      INCLUDE 'varcom.inc'
      INCLUDE 'bestcom.inc'
      INCLUDE 'errcodes.inc'
      INCLUDE 'togcom.inc'
 
      EXTERNAL CALGA1,CALGA2,CALGA3,CALGA4,CALGA5
 
      INTEGER ISIM,I,DOF
      REAL FVEC1(MAXPTS)
*-----------------------------------------------------------------------
 
      IERR = NOERROR
      DOF = NPOINT - NGFREE - NLFREE - NAFREE - NLNFR - NBLFR
 
C     /* Simulation loop */
      DO 100 ISIM = 1,NSIMS
 
c        /* Construct full global parameter set PGBEST */
        CALL CNSAPP(NFREE,VFREES(1,ISIM),MLPAR,NSUBS,PL0,PLLB,PLUB,
     &              PLBEST,INDL,KONL,WEIL)
 
c        /* Do local part of simulation using PLBEST parameters */
        CALL ULOLOC()
 
c        /* Do global area fit */
        CALL CALOPT(NAFREE,VA0,VALB,VAUB,NPOINT,MAXITA,GARLEV,NOPTMZ,
     &              CALGA1,CALGA2,CALGA3,CALGA4,CALGA5,VABEST,
     &              CHI2S(ISIM),FVEC1,IERR)
        IF ( IERR.NE.NOERROR ) GOTO 200
 
c        /* Apply local penalty function to residuals; adjust chi^2 */
        CHI2S(ISIM) = CHI2S(ISIM)*PENLTY(ISIM)**2
 
c        /* Return fvec if first simulation */
        IF ( ISIM.EQ.1 ) THEN
          DO 20 I = 1,NPOINT
            FVEC(I) = FVEC1(I)*PENLTY(1)
 20       CONTINUE
        ENDIF
 
c        /* Hopefully speed up convergence of the next area fit */
        CALL VCOPY(NAFREE,VABEST,VA0)
 
c        /* Store and/or report as necessary */
        ITERL = ITERL + 1
 
        IF ( CHI2S(ISIM).LT.X2BSTL ) THEN
          X2BSTL = CHI2S(ISIM)
          CALL STORE(LOCLEV,NFREE,VFREES(1,ISIM))
          REPLOT(LOCLEV) = .TRUE.
        ENDIF
 
 
        IF ( NRPRTL.GT.0 ) THEN
          IF ( MOD(ITERL,NRPRTL).EQ.0 ) THEN
            CALL REPORT(LOCLEV,ITERL,X2BSTL/DOF,CHI2S(ISIM)/DOF)
            REPLOT(LOCLEV) = .FALSE.
 
            IF ( (ABORT.EQ.ABORTFIT) .OR. (ABORT.EQ.ABORTALL) ) THEN
              IERR = NOERROR
              GOTO 200
            ELSEIF ( ABORT.EQ.ABORTFATAL ) THEN
              IERR = ERRABORT
              GOTO 200
            ENDIF
          ENDIF
        ENDIF
 
c        /* Check number of function evaluations */
        IF ( ITERL.GE.MAXITL .and.  ABORT.le.ABORTENDOPT) 
     2                 ABORT = ABORTENDOPT
 
 100  CONTINUE
 
 200  RETURN
      END
