      SUBROUTINE CALOPT(NFREE,VIN,VLB,VUB,NPOINT,NITER,ILEVEL,NOPT,OPT1,
     &                  OPT2,OPT3,OPT4,OPT5,VOUT,CHI2,FVEC,IERR)
*-----------------------------------------------------------------------
*--- Contact- OLD: WEB Research Co. /// NEW: WMOSS.ORG
*---orig: May94 CS
*---mru : 23May94 CS; 960207 tk
*
*     Generic optimization shell for all levels.
*
*     nfree  : [input]  Number of free variables;
*     vin    : [input]  Starting point for search;
*     vlb    : [input]  Lower bounds of variables;
*     vub    : [input]  Upper bounds of variables;
*     npoint : [input]  Total number of unmasked data points;
*     niter  : [input]  Maximum number of iterations;
*     ilevel : [input]  Level of optimization (GLOLEV,LOCLEV,GARLEV);
*     nopt   : [input]  Optimizer to use
*            :            1) SOPTMZ
*            :           -2) LMDIF
*            :            2) LMDIF
*            :            3) MNFB
*            :         xxx 950207 tk   4) VE08AD
*            :            5) N2FB;
*     OPT1   : [input]  Calculation function for SOPTMZ;
*     OPT2   : [input]  Calculation function for LMDIF;
*     OPT3   : [input]  Calculation function for LMQNBC;
*     vout   : [output] Best variable values found;
*     chi2   : [output] Chi-squared at this point;
*     fvec   : [output] Residuals at this point: (data-theory)/noise
*     ierr   : [output] Error code returned.
*-----------------------------------------------------------------------
      INTEGER NFREE,NPOINT,NITER,ILEVEL,NOPT,IERR
      REAL VIN(NFREE),VLB(NFREE),VUB(NFREE)
      REAL VOUT(NFREE),CHI2,FVEC(NPOINT)
      EXTERNAL OPT1,OPT2,OPT3,OPT4,OPT5
*-----------------------------------------------------------------------
      INCLUDE 'limits.inc'
      INCLUDE 'togcom.inc'
      INCLUDE 'bestcom.inc'
      INCLUDE 'parcom.inc'
      INCLUDE 'errcodes.inc'
      INCLUDE 'varcom.inc'
 
c     /* For all optimizers */
      REAL FARR(LSPEC,LT1)
      INTEGER I,J,IDUMMY,NSPPTS(LT1),DOF
 
 
c     /* For LMDIF */
      REAL TOL
      INTEGER LWA
      PARAMETER (TOL=1.0E-3,LWA=MAXPTS*MVAR+5*MVAR+MAXPTS)
 
      INTEGER IWA(MVAR)
      REAL WA(LWA)
 
c     /* For MNFB and N2FB */
      INTEGER LIV,LV
      PARAMETER (LIV=82+MVAR)
      PARAMETER (LV=105+MVAR*(MAXPTS+2*MVAR+17)+2*MAXPTS)
 
      INTEGER IV(LIV),UIDUM(1)
      REAL V(LV),B(2,MVAR),SCALE(MVAR),URDUM(1)
 
*-----------------------------------------------------------------------
 
C---In case of abort, initialize chi2 value to BIG
        CHI2 = 1.0E38

c     /* Initialize optimization for this level */
      IF ( ILEVEL.EQ.GLOLEV ) THEN
        X2BSTG = 1.701403E38
        ITERG = 0
      ELSEIF ( ILEVEL.EQ.LOCLEV ) THEN
        X2BSTL = 1.701403E38
        ITERL = 0
      ELSEIF ( ILEVEL.EQ.GARLEV ) THEN
        X2BSTA = 1.701403E38
        ITERGA = 0
      ELSE
        CALL ERRMSG('Bad level argument to CALOPT!')
        IERR = ERRBADARGUMENT
        ABORT = ABORTFATAL
        GOTO 500
      ENDIF
 
      IF ( NFREE.LE.0 ) THEN
        CALL VCOPY(NFREE,VIN,VOUT)
        CALL OPT1(VIN,NFREE,VUB,VLB,CHI2)
 
      ELSEIF ( NOPT.EQ.1 ) THEN
        CALL SOPTMZ(VUB,VLB,VIN,VOUT,NFREE,OPT1,CHI2,NITER)
 
      ELSEIF ( ABS(NOPT).EQ.2 ) THEN
        CALL VCOPY(NFREE,VIN,VOUT)
        CALL LMDIF1(OPT2,NPOINT,NFREE,VOUT,FVEC,TOL,IERR,IWA,WA,LWA)
 
      ELSEIF ( (NOPT.EQ.3) .OR. (NOPT.EQ.5) ) THEN
 
        DO 50 I = 1,NFREE
          B(1,I) = VLB(I)
          B(2,I) = VUB(I)
          VOUT(I) = VIN(I)
          IF ( VLB(I).LT.VUB(I) ) THEN
            SCALE(I) = 1.0/(VUB(I)-VLB(I))
          ELSE
            SCALE(I) = 1.0
          ENDIF
 50     CONTINUE
 
c        /* Turn off printing, set maximum iterations */
        IF ( NOPT.EQ.3 ) THEN
          CALL IVSET(2,IV,LIV,LV,V)
        ELSE
          CALL IVSET(1,IV,LIV,LV,V)
        ENDIF
 
        IV(17) = (MVAR+2)*NITER
        IV(18) = (MVAR+2)*NITER
        IV(21) = 0
 
        IF ( NOPT.EQ.3 ) THEN
          CALL MNFB(NFREE,SCALE,VOUT,B,OPT3,IV,LIV,LV,V,UIDUM,URDUM,
     &              OPT1)
        ELSE
 
c            do 110 i = 1, nfree
c               v(iv(27) + i - 1) = scale(i)
c  110       continue
 
          CALL N2FB(NPOINT,NFREE,VOUT,B,OPT5,IV,LIV,LV,V,UIDUM,URDUM,
     &              OPT1)
        ENDIF
 
      ELSEIF ( NOPT.EQ.4 ) THEN
*-----------------------------------------------------------------------
*---tk 960207
         call errmsg('Harwell VE08AD not currently licensed.')
 
 
      ELSE
        CALL ERRMSG('Bad Optimization method to CALOPT!')
        IERR = ERRBADARGUMENT
        ABORT = ABORTFATAL
        GOTO 500
      ENDIF
 
c     /* Handle abort flag resulting from the optimization */
      IF ( ABORT.EQ.ABORTFATAL ) THEN
        IERR = ERRABORT
        GOTO 500
      ELSEIF ( ABORT.LE.ABORTENDOPT ) THEN
        ABORT = ABORTOFF
      ENDIF
      IERR = NOERROR
 
c     /* Compute return value: fvec */
      DO 300 J = 1,NSPEC
        NSPPTS(J) = ICAL(1,J)
        DO 200 I = 1,NSPPTS(J)
          FARR(I,J) = (SPEC(I,J,1)-RESPONSE(I,J,0,ILEVEL))
          IF ( SNOYS(I,J,1).GT.0 ) FARR(I,J) = FARR(I,J)/SNOYS(I,J,1)
 200    CONTINUE
 300  CONTINUE
      CALL MSKAPP(NSPPTS,FARR,SMASK,FVEC,IDUMMY)
 
c     /* Compute return value: chi2 */
      CHI2 = 0.
      DO 400 I = 1,NPOINT
        CHI2 = CHI2 + FVEC(I)**2
 400  CONTINUE
 
c     /* Display report */
      IF ( ILEVEL.EQ.LOCLEV ) THEN
        DOF = NPOINT - NLFREE - NAFREE - NLNFR - NBLFR
        CALL REPORT(LOCLEV,ITERL,X2BSTL/DOF,X2BSTL/DOF)
        REPLOT(LOCLEV) = .FALSE.
      ELSEIF ( ILEVEL.EQ.GLOLEV ) THEN
        DOF = NPOINT - NGFREE - NLFREE - NAFREE - NLNFR - NBLFR
        CALL REPORT(GLOLEV,ITERG,X2BSTG/DOF,X2BSTG/DOF)
        REPLOT(GLOLEV) = .FALSE.
      ENDIF
 
 500  RETURN
      END
