*----------------------------------------------------------------------
      subroutine calwa(pos, npts, spc, cwcal, width, area)
*----------------------------------------------------------------------
      real pos, cwcal, width, area, spc(npts)
*----------------------------------------------------------------------
*--- Contact- OLD: WEB Research Co. /// NEW: WMOSS.ORG,
*     CS, mru 27Jul93 cs
*
*     Approximates the best width and area for a Lorentzian/Voigt line
*     given a data spectrum and the line's position.  The width is
*     found via LR scanning of the spectrum, and the area is such that
*     the maximum of the line is exactly equal to the data there.
*     On input, the sign of width determines whether the line is L or V.
*
*     pos    : [Input]  Channel position of the Lorentzian;
*     npts   : [Input]  Number of points in the data spectrum;
*     spc    : [Input]  Data spectrum;
*     cwcal  : [Input]  Constant for calculating Voigt lines;
*     width  : [Output] Approximated width;
*     area   : [Output] Approximated area;
*----------------------------------------------------------------------
      include 'limits.inc'
      include 'parcom.inc'
*----------------------------------------------------------------------
      real halfht, xhalf, xhigh, xlow
      real tmpspc(lspec)
      integer i, ipos, npts, irt, ilt
*----------------------------------------------------------------------
 
      ipos = max(1, min(npts, nint(pos)))
      halfht = spc(ipos) / 2.
 
*---Scan left and right for the closest channel >= half-maximum
      do 100 i = 1, max(ipos-1, npts-ipos)
         irt = max(ipos-i, 1)
         ilt = min(ipos+i, npts)
         if (spc(irt).gt.halfht) then
            xhigh = spc(irt)
            xlow = spc(irt + 1)
            xhalf = irt + (xhigh-halfht)/(xhigh-xlow)
            goto 200
         elseif (spc(ilt).gt.halfht) then
            xhigh = spc(ilt)
            xlow = spc(ilt - 1)
            xhalf = ilt - (xhigh-halfht)/(xhigh-xlow)
            goto 200
         endif
  100 continue
c     ;; None found, use default value.
      width = 10.0
      goto 300
 
c     ;; Give the width the same sign as the input width.
  200 width = sign(2 * abs(pos-xhalf), width)
 
c     ;; Calculate line, and from this the guessed area.
  300 call lnorm(npts, tmpspc, pos, width, cwcal, 0.0, 1.00)
      area = spc(ipos) / tmpspc(ipos)
 
      return
      end
