      SUBROUTINE DEFGRD(KSYM,NTHETA,Kdistr,SigThe,IERR)
*---PURPOSE:  GIVEN KSYM AND NTHETA, DEFINE GRID.
*---          MAXGRD IS SIZE OF GRID STORAGE ARRAYS IN COMMON BLOCK.
*---          KSYM IS SYMMETRY KEY.
*---          NTHETA IS THE NUMBER OF POINTS ALONG THETA DIRECTION.
*---          Kdistr specifies uniform (Kdistr=1) or Gaussian (Kdistr=2)
*---             weights for grid points.  Gaussian centered on Z.
*---          SigThe is HwHm of Gaussian distribution in Theta.
*---          NGRID IS NUMBER OF POINTS IN GRID.
*---          EX, EY, EZ ARE STORAGE FOR DIRECTION COSINES
*---          WEIGHT IS STORAGE FOR SOLID ANGLE ASSOCIATED WITH EACH
*---                 GRID POINT.  SUM OF WEIGHTS EQUALS 1.
*-----------------------------------------------------------------------
*--- Contact- OLD: WEB Research Co. /// NEW: WMOSS.ORG
*---orig SEPT. 4, 1989   TAK
*---MRU 931107 tk
*---   Definition of "Symmetry" parameter for specification of Grid for
*---   averaging over Applied Field and Gamma Beam direction relative to
*---   Molecular or Crystal coordinate frame, XYZ.
*-----------------------------------------------------------------------
*---KSYM  | Symmetry needed at local| TOTAL SOLID ANGLE OF GRID  |NGRID
*---      | site of powder sample.  | (X,Y,Z denote molucular or |Total
*---      |                         | (crystal coordinate frame. |Number
*---      |                         |  applied field rel. to     |of grid
*---      |                         |  molecular axes.)          |points
*-----------------------------------------------------------------------
*---  0   |   OH  (CUBIC, X=Y=Z)    | SINGLE POINT  (0,0,1)      |  1
*---  1   |   D4H (TETRAGONAL, X=Y) | 1-D ARC in XZ plane (Y=0)  |NTHETA
*---  2   |   D2H (ORTHORHOMBIC)    | OCTANT     (X,Y,Z>0)       |  NQ
*---  3   |   C2  (MONOCLINIC)      | QUADRANT   (X,Y  >0)       | 2*NQ
*---  4   |   C1  (TRICLINIC)       | HEMISPHERE (  Y  >0)       | 4*Nq
*-----------------------------------------------------------------------
*---NOTE, IN TABLE ABOVE, NQ = NTHETA*(NTHETA+1)/2
*-----------------------------------------------------------------------
*---externals
      INTEGER KSYM,NTHETA,Kdistr,IERR
      real SigThe
*---Common blocks and parameters
      INCLUDE 'limits.inc'
      INCLUDE 'iocom.inc'
      INCLUDE 'constcom.inc'
      INCLUDE 'gridcom.inc'
*---internals 
      INTEGER NX,NZ,NOCTNT,IG,IZ,ITHETA,NPHI,IPHI
      REAL PIV2,DTHE,SDTV2,SUMW,THETA,DPHI,WWW,PHI,sigthr
*-----------------------------------------------------------------------
 
      IERR = 0
      PIV2 = 0.5*PI
*---if weights are not uniform then
*---check for zero sigma theta and then convert to radians.
      if(sigthe.eq.0.0 .and. Kdistr.ne.1)then
         call errmsg('SIGTHE = 0.0 in DEFGRD')
         ierr=1
         return
      else
        sigthr=sigthe*rpd
      endif
 
*---DEFAULT NUMBER OF OCTANTS OF SPHERE IN GRID.
        NX = 1
        NZ = 1

*---define total number of grid points and octants. 
        IF ( KSYM.EQ.0 ) THEN
          NGRID = 1
          NTHETA = 1
        ELSEIF ( KSYM.EQ.1 ) THEN
          NGRID = NTHETA
        ELSEIF ( KSYM.EQ.2 ) THEN
          NGRID = NTHETA*(NTHETA+1)/2
        ELSEIF ( KSYM.EQ.3 ) THEN
          NGRID = 2*NTHETA*(NTHETA+1)/2
          NZ = 2
        ELSEIF ( KSYM.EQ.4 ) THEN
          NGRID = 4*NTHETA*(NTHETA+1)/2
          NX = 2
          NZ = 2
        ELSE
          CALL ERRMSG('Invalid value for symmetry parameter KSYM.')
        ENDIF
*---NUMBER OF OCTANTS IN TOTAL GRID.
        NOCTNT = NX*NZ

*---CHECK THAT COMMON BLOCK \GRID\ SIZE IS LARGE ENOUGH.
        IF ( NGRID.GT.MAXGRD ) THEN
          CALL ERRMSG('Too many grid points! ')
          IERR = 100
          RETURN
        ENDIF
 
 
*---SCAN GRID
        if(ksym.eq.0)then
           ex(1)=0.0
           ey(1)=0.0
           ez(1)=1.0
           weight(1)=1.0
           sumw=1.0
        elseif(ksym.eq.1)then

        IG = 0
        DTHE = piv2/NTHETA
        SDTV2 = SIN(DTHE/2.)
        SUMW = 0.0
        DO 50 IZ = 1,NZ
           DO 20 ITHETA = 1,NTHETA
              THETA = PI*(IZ-1) + (3-2*IZ)*DTHE*(ITHETA-0.5)
              WWW = 2.*SIN(THETA)*SDTV2/NOCTNT
              if(Kdistr.eq.2)THEN
                 www=www*exp(-(theta/sigthr)**2)
              ELSEIF (Kdistr.eq.3)then
                 www=www*exp(-((piv2-theta)/sigthr)**2)
              endif
              IG = IG + 1
              IF ( IG.GT.MAXGRD ) THEN
                CALL ERRMSG('Too many grid points! ')
                IERR = 100
                RETURN
              ENDIF
              WEIGHT(IG) = WWW
              SUMW = SUMW + WWW
              EX(IG) = SIN(THETA)
              EY(IG) = 0.0
              EZ(IG) = COS(THETA)
 20       CONTINUE
 50     CONTINUE
      else
        IG = 0
        DTHE = piv2/NTHETA
        SDTV2 = SIN(DTHE/2.)
        SUMW = 0.0
        DO 150 IZ = 1,NZ
           DO 120 ITHETA = 1,NTHETA
            THETA = PI*(IZ-1) + (3-2*IZ)*DTHE*(ITHETA-0.5)
            NPHI = NX*ITHETA
            DPHI = PIV2/ITHETA
            WWW = 2.*DPHI*SIN(THETA)*SDTV2/(NOCTNT*PIV2)
              if(Kdistr.eq.2)THEN
*---bias toward Z axis
                 www=www*exp(-(theta/sigthr)**2)
              ELSEIF (Kdistr.eq.3)then
*---bias toward XY plane
                 www=www*exp(-((piv2-theta)/sigthr)**2)
              endif
            DO 110 IPHI = 1,NPHI
              IG = IG + 1
              IF ( IG.GT.MAXGRD ) THEN
                CALL ERRMSG('Too many grid points! ')
                IERR = 100
                RETURN
              ENDIF
              WEIGHT(IG) = WWW
              SUMW = SUMW + WWW
              PHI = DPHI*(IPHI-0.5)
              EX(IG) = SIN(THETA)*COS(PHI)
              EY(IG) = SIN(THETA)*SIN(PHI)
              EZ(IG) = COS(THETA)
 110         CONTINUE
 120       CONTINUE
 150     CONTINUE
      endif

*---normalize sum of weights to unity
      do 200 ig=1,ngrid
         weight(ig)=weight(ig)/sumw
  200 continue
 
      RETURN
      END
