      REAL FUNCTION ENORM(N,X)
      INTEGER N
      REAL X(N)
c     **********
c
c     function enorm
c
c     given an n-vector x, this function calculates the
c     euclidean norm of x.
c
c     the euclidean norm is computed by accumulating the sum of
c     squares in three different sums. the sums of squares for the
c     small and large components are scaled so that no overflows
c     occur. non-destructive underflows are permitted. underflows
c     and overflows do not occur in the computation of the unscaled
c     sum of squares for the intermediate components.
c     the definitions of small, intermediate and large components
c     depend on two constants, rdwarf and rgiant. the main
c     restrictions on these constants are that rdwarf**2 not
c     underflow and rgiant**2 not overflow. the constants
c     given here are suitable for every known computer.
c
c     the function statement is
c
c       real function enorm(n,x)
c
c     where
c
c       n is a positive integer input variable.
c
c       x is an input array of length n.
c
c     subprograms called
c
c       fortran-supplied ... abs,sqrt
c
c     argonne national laboratory. minpack project. march 1980.
c     burton s. garbow, kenneth e. hillstrom, jorge j. more
c
c     **********
      INTEGER I
      REAL AGIANT,FLOATN,ONE,RDWARF,RGIANT,S1,S2,S3,XABS,X1MAX,X3MAX,
     &     ZERO
C_CS 04Jun91 Patch; The suggested rdwarf**2 < minimum real value
C      data one,zero,rdwarf,rgiant /1.0e0,0.0e0,3.834e-20,1.304e19/
      SAVE ONE,ZERO,RDWARF,RGIANT
      DATA ONE,ZERO,RDWARF,RGIANT/1.0E0,0.0E0,1.09E-19,1.304E19/
      S1 = ZERO
      S2 = ZERO
      S3 = ZERO
      X1MAX = ZERO
      X3MAX = ZERO
      FLOATN = N
      AGIANT = RGIANT/FLOATN
      DO 100 I = 1,N
        XABS = ABS(X(I))
        IF ( XABS.GT.RDWARF .AND. XABS.LT.AGIANT ) THEN
c
c           sum for intermediate components.
c
          S2 = S2 + XABS**2
        ELSEIF ( XABS.LE.RDWARF ) THEN
c
c              sum for small components.
c
          IF ( XABS.LE.X3MAX ) THEN
            IF ( XABS.NE.ZERO ) S3 = S3 + (XABS/X3MAX)**2
          ELSE
            S3 = ONE + S3*(X3MAX/XABS)**2
            X3MAX = XABS
          ENDIF
c
c              sum for large components.
c
        ELSEIF ( XABS.LE.X1MAX ) THEN
          S1 = S1 + (XABS/X1MAX)**2
        ELSE
          S1 = ONE + S1*(X1MAX/XABS)**2
          X1MAX = XABS
        ENDIF
 100  CONTINUE
c
c     calculation of norm.
c
      IF ( S1.NE.ZERO ) THEN
        ENORM = X1MAX*SQRT(S1+(S2/X1MAX)/X1MAX)
      ELSEIF ( S2.EQ.ZERO ) THEN
        ENORM = X3MAX*SQRT(S3)
      ELSE
        IF ( S2.GE.X3MAX ) ENORM = SQRT(S2*(ONE+(X3MAX/S2)*(X3MAX*S3)))
        IF ( S2.LT.X3MAX ) ENORM = SQRT(X3MAX*((S2/X3MAX)+(X3MAX*S3)))
      ENDIF
      RETURN
c
c     last card of function enorm.
c
      END
