C > f77.ghcsel : INTERACTER graphics hardcopy driver interface
C
C  *********************************************************
C  **   (c) Interactive Software Services Ltd. 1989-1994  **
C  ** --------------------------------------------------- **
C  ** Licenced users of INTERACTER are granted permission **
C  **  to modify and integrate this source code in their  **
C  **          own INTERACTER-based software              **
C  ** --------------------------------------------------- **
C  ** I.S.S. can be contacted on Tel +44 (0) 785 715588   **
C  **                            Fax +44 (0) 785 714913   **
C  *********************************************************
C
      SUBROUTINE GHCSEL(IACTN,IDRNUM)
      INTEGER IACTN,IDRNUM
C
C  INTERACTER Graphics HardCopy driver SELection routine
C
C  IACTN  = Action code
C           This must ALWAYS be 1 to select a hardcopy driver
C  IDRNUM = Required driver number :
C       1 : HP-GL (default at start up unless DATA statement changed)
C       2 : PostScript
C       3 : Acorn Draw
C       4 : Raster graphics (Epson dot matrix, HP LaserJet, etc.)
C       5 : Tektronix 4014 (DEC LN03 Plus)
C       6 : Raster graphics (Image data file : PCX, etc.)
C       7+: Reserved
C
C  **************************************************************
C  This routine should be called after GRINIT and before GHCOPY
C  if you wish to select an alternative graphics hardcopy driver.
C  e.g.
C        CALL GRINIT(' ',800,600,16)
C        CALL GHCSEL(1,2)
C        CALL GHCOPY(' ')
C  **************************************************************
C
      INTEGER IHCDRV
C
C  To cause GRINIT to select a different default hardcopy driver,
C  change the value of IHCDRV in the following DATA statement.
C
      DATA IHCDRV/1/
C
      IF ( IACTN.NE.1 ) RETURN
      IHCDRV = IDRNUM
C
C***********************************************************************
C
      ENTRY XXHCIF(IACTN)
C
C  INTERACTER Graphics Hardcopy driver interface.
C
C  **********************************************
C  This ENTRY point is solely for internal use by
C  INTERACTER and must NEVER be called directly.
C  **********************************************
C
C  This routine acts as an interface between INTERACTER's device
C  independent calling routines and the device-specific graphics
C  hardcopy drivers. The INTERACTER library already contains a
C  version of this routine which calls ONE or ALL of the available
C  drivers, dependent on the memory addressing capabilities of the
C  target system :
C
C  (B) Virtual memory systems :  All supplied drivers are called
C      (VAX/VMS, Unix, DOS/DBOS)
C
C  (A) Fixed memory systems   :  Only the HP-GL driver is called
C      (Other DOS extenders, real mode DOS, RISC OS)
C
C  By default, only the HP-GL driver is called here, but other
C  combinations of drivers can be activated by decommenting the
C  appropriate driver CALL's which begin with a '*-' comment.
C  Similarly, to disable a driver simply comment out the approriate CALL
C  [Alternatively, the routine can be processed using QMERGE from
C   Polyhedron Software's plusFORT programming tools using a command
C   of the form :
C     QMERGE SELECT="HPGL,RASTER,!" GHCSEL.FOR TO=outfile_name
C   wheree the SELECT= parameters depend on the required drivers]
C
C  Compile and link this routine as you would any other code. Users of
C  Microsoft Fortran 5.x under DOS should use the /NOE linker option.
C
      IF ( IHCDRV.EQ.2 ) THEN
C
C  PostScript
C
*-IF POSTSCRIPT
        CALL XXPOST(IACTN)
*-ENDIF
        RETURN
      ELSEIF ( IHCDRV.EQ.3 ) THEN
C
C  Acorn Draw
C
*-IF DRAW
*-    CALL XXDRAW(IACTN)
*-ENDIF
        RETURN
      ELSEIF ( IHCDRV.EQ.4 ) THEN
C
C  Raster graphics (HP PCL, Epson/IBM dot matrix, etc.)
C
*-IF RASTER
        CALL XXRAST(IACTN)
*-ENDIF
        RETURN
      ELSEIF ( IHCDRV.EQ.5 ) THEN
C
C  Tektronix 4014 (DEC LN03 Plus)
C
*-IF TEK4014
*-      CALL XXTEKX(IACTN)
*-ENDIF
        RETURN
      ELSEIF ( IHCDRV.EQ.6 ) THEN
C
C  Raster graphics (Image data file : currently PCX format)
C
*-IF IMAGE
*-      CALL XXIMAG(IACTN)
*-ENDIF
        RETURN
      ELSE
C
C  HP-GL
C
*-IF HPGL
        CALL XXHPGL(IACTN)
*-ENDIF
        RETURN
      ENDIF
      END
