      SUBROUTINE INT57(UR,UI,C1,C2R,C2I,EX,EY,EZ,IPOL,XINT)
C***
C***    THE SUBROUTINE INT57 CALCULATES THE INTENSITIES FROM THE
C***    57 FE NUCLEAR EIGENSTATES AND THE DIRECTION COSINES OF THE
C***    GAMMA-RAY.
C***
C***    INPUT PARAMETERS
      REAL UR(4,4),UI(4,4), C1, C2R, C2I, EX, EY, EZ
      INTEGER IPOL
      

C***    UR(I,K) AND UI(I,K) ARE THE REAL PART AND THE IMAGINARY PART
C***     OF THE I-TH COMPONENT OF THE EIGENVECTOR K FOR THE 3/2 STATE
C***    C1,C2R,C2I=COEFFICIENTS OF THE EIGENSTATES OF THE 57FE NUCLEAR
C***     GROUND STATE, AS DEFINED IN SUBROUTINE N57G I.E.
C***          !1)=C1 * !1/2) + (C2R+I*C2I) * !-1/2)
C***          !2)=-(C2R-I*C2I) * !1/2) + C1 * !-1/2)
C***          I=SQRT(-1) IN THE ABOVE FORMULA
C***    EX,EY,EZ=VECTOR CHARACTERIZING THE ELECTROMAGNETIC WAVE. TO
C***     BE USED AS DEFINED BY IPOL
C***    IPOL: CHARACTERIZES THE TYPE OF INTENSITY CALCULATION
C***     IPOL=-3 : INTENSITY TENSOR AND INTENSITY VECTOR ARE TO BE
C***               CALCULATED. NO INPUT FOR EX,EY,EZ NECESSARY
C***     IPOL=-2 : POWDER MEAN VALUE; NO INPUT FOR EX,EY,EZ NECESSARY
C***     IPOL=-1 : PERPENDICULAR CONFIGURATION (SPLIT COIL)
C***               EX,EY,EZ IS TO BE PARALLEL TO THE EXTERNAL FIELD
C***     IPOL=0  : UNPOLARIZED GAMMA-RAY PARALLEL TO EX,EY,EZ
C***     IPOL=1  : CIRCULAR POLARISATION,WAVE VECTOR PARALLEL EX,EY,EZ
C***     IPOL=2  : LINEAR POLARISATION; IN 57FE (EX,EY,EZ) IS THE VECTOR
C***               PERPENDICULAR TO THE WAVE VECTOR AND PERPENDICULAR TO
C***               THE VECTOR POTENTIAL A
C***
C***    OUTPUT PARAMETERS
      REAL XINT(8)
C***    THE INTENSITIES BELONGING TO THE !1) TRANSITION ARE STORED
C***    IN XINT(1:4), THOSE BELONGING TO THE !2) TRANSITION
C***    IN XINT(5:8)
C***    FOR IPOL=-3 : INTENSITY TENSOR IS STORED IN THE FOLLOWING WAY
C***    IXX IN 1:8, IYY IN 9:16, IZZ IN 17:24,IXY IN 25:32,IXZ IN 33:40,
C***    IYZ IN 41:48,THE INTENSITY VECTOR IS STORED IN THE FOLLOWING WAY:
C***    IX IN 49:56, IY IN 57:64, IZ IN 65:72
C***

      real sqrt3, hxx, hyy, hzz, hxy, hxz, hyz, esq, esqr
      real ur1, ur2, ur3, ur4, ui1, ui2, ui3, ui4
      real vr1, vr2, vr3, vr4, vi1, vi2, vi3, vi4
      real r1, r2, r3, u1, u2, u3
      real x11, x22, x33, xr23, xi23, xr12, xi12, xr13, xi13
      real h0, h1, h2, rxx, ryy, rzz, rxy, rxz, ryz, xinten
      real rx, ry, rz 
      integer i, ii, k

      SQRT3 = 1.732050808
      IF ( IPOL.GE.-1 ) THEN
        HXX = EX*EX
        HYY = EY*EY
        HZZ = EZ*EZ
        HXY = EX*EY
        HXZ = EX*EZ
        HYZ = EY*EZ
        ESQ = HXX + HYY + HZZ
        IF ( IPOL.EQ.1 ) ESQR = SQRT(ESQ)
      ENDIF
      DO 100 K = 1,4
        UR1 = UR(1,K)
        UR2 = UR(2,K)
        UR3 = UR(3,K)
        UR4 = UR(4,K)
        UI1 = UI(1,K)
        UI2 = UI(2,K)
        UI3 = UI(3,K)
        UI4 = UI(4,K)
        VR2 = C2R*UR2 + C2I*UI2
        VR3 = C2R*UR3 + C2I*UI3
        VR4 = C2R*UR4 + C2I*UI4
        VI2 = C2R*UI2 - C2I*UR2
        VI3 = C2R*UI3 - C2I*UR3
        VI4 = C2R*UI4 - C2I*UR4
        R1 = C1*UR1*SQRT3 + VR2
        R2 = C1*UR2 + VR3
        R3 = C1*UR3 + VR4*SQRT3
        U1 = C1*UI1*SQRT3 + VI2
        U2 = C1*UI2 + VI3
        U3 = C1*UI3 + VI4*SQRT3
        I = K
 50     X22 = R2*R2 + U2*U2
        X33 = R3*R3 + U3*U3
        XR23 = R2*R3 + U2*U3
        XI23 = U2*R3 - R2*U3
        X11 = R1*R1 + U1*U1
        XR12 = R1*R2 + U1*U2
        XR13 = R1*R3 + U1*U3
        XI12 = U1*R2 - R1*U2
        XI13 = U1*R3 - R1*U3
        H0 = (X11+X33)/4.0
        H1 = H0 + X22
        H2 = XR13/2.0
        RXX = H1 + H2
        RYY = H1 - H2
        RZZ = 2.0*H0
        IF ( IPOL.NE.-2 ) THEN
          RXY = -XI13
          RXZ = XR12 - XR23
          RYZ = -XI12 + XI23
          IF ( IPOL.NE.-3 ) THEN
            XINTEN = (RXX*HXX+RYY*HYY+RZZ*HZZ+RXY*HXY+RXZ*HXZ+RYZ*HYZ)
     &               /ESQ
            IF ( IPOL.LT.0 ) THEN
              XINTEN = 0.5*(RXX+RYY+RZZ-XINTEN)
            ELSEIF ( IPOL.NE.0 ) THEN
              IF ( IPOL.EQ.2 ) THEN
                XINTEN = RXX + RYY + RZZ - 2.*XINTEN
              ELSE
                RX = XR12 + XR23
                RY = -XI12 - XI23
                RZ = (X11-X33)/2.0
                XINTEN = XINTEN + (RX*EX+RY*EY+RZ*EZ)/ESQR
              ENDIF
            ENDIF
            XINT(I) = XINTEN
          ELSE
            XINT(I) = RXX
            II = I + 8
            XINT(II) = RYY
            II = I + 16
            XINT(II) = RZZ
            II = I + 24
            XINT(II) = RXY/2.
            II = I + 32
            XINT(II) = RXZ/2.
            II = I + 40
            XINT(II) = RYZ/2.
            II = I + 48
            XINT(II) = XR12 + XR23
            II = I + 56
            XINT(II) = -XI12 - XI23
            II = I + 64
            XINT(II) = (X11+X33)/2.0
          ENDIF
        ELSE
          XINTEN = (RXX+RYY+RZZ)/3.
          XINT(I) = XINTEN
        ENDIF
        IF ( I.LE.4 ) THEN
          I = K + 4
          VR1 = -C2R*UR1 + C2I*UI1
          VR2 = -C2R*UR2 + C2I*UI2
          VR3 = -C2R*UR3 + C2I*UI3
          VI1 = -C2R*UI1 - C2I*UR1
          VI2 = -C2R*UI2 - C2I*UR2
          VI3 = -C2R*UI3 - C2I*UR3
          R1 = VR1*SQRT3 + C1*UR2
          R2 = VR2 + C1*UR3
          R3 = VR3 + C1*UR4*SQRT3
          U1 = VI1*SQRT3 + C1*UI2
          U2 = VI2 + C1*UI3
          U3 = VI3 + C1*UI4*SQRT3
          IF ( I.LT.9 ) GOTO 50
        ENDIF
 100  CONTINUE
      II = 8
      IF ( IPOL.EQ.-3 ) II = 72
      DO 200 I = 1,II
        XINT(I) = XINT(I)/4.
 200  CONTINUE
      RETURN
      END
