*-----------------------------------------------------------------------
      SUBROUTINE NEWBOX(X1,Y1,X2,Y2,NTOP,TOPLST,PWL,QWL,PWU,QWU,SUBS,NG,
     &                  GTEXT,PTITLE,XTITL,XTTITL,YTITL)
*-----------------------------------------------------------------------
*--- Contact- OLD: WEB Research Co. /// NEW: WMOSS.ORG
*---orig: 1992 tk
*---mru : 931130 tk
*
*     Current version of newbox presents no legend or date/time stamp.
*
*     Draws a box with tic marks and outer labels, and sets the
*     correct horizontal/vertical scaling in preparation for a
*     call to DATPLT
*-----------------------------------------------------------------------
*
*---coordinate frames
*    (x,y)  are in data units
*    (u,v)  are in normalized window units
*    (q,p)  are in normalized screen units
*           i.e. the rectangle pl=ql=0.0, pu=qu=1.0 covers
*           the entire screen
*
*---upper and lower limits
*    1 or l specifies a lower limit
*    2 or u specifies an upper limit
*
*---input
*     x1     : minimum value of x in data set ( in data units )
*     y1     : minimum value of y in data set
*     x2     : maximum value of x in data set
*     y2     : maximum value of y in data set
*     pwl    : Window's horizontal lower limit (in normalized screen units)
*     qwl    : Window's vertical lower limit
*     pwu    : Window's horizontal upper limit
*     qwu    : Window's veritcal upper limit
*     subs   : number of subspectra to be plotted above main spectrum
*     ng     : number of data groups to be noted in legend
*     gtext  : text describing data groups to appear in legend
*-----------------------------------------------------------------------
      INTEGER SUBS,NG,NTOP
      REAL X1,Y1,X2,Y2
      REAL PWL,QWL,PWU,QWU,TOPLST(2,*)
      CHARACTER GTEXT(NG)*8
      CHARACTER*72 PTITLE(2),XTITL,YTITL,XTTITL
*-----------------------------------------------------------------------
      INCLUDE 'plotcom.inc'
      INCLUDE 'togcom.inc'
      INCLUDE 'constcom.inc'
 
*---number of steps per axis
      INTEGER NXTIC,NYTIC
*---tic and label sizes
      REAL VXTCSZ,UYTCSZ,VXLBSZ,UYLBSZ
*---date/time stamp strings
      CHARACTER*9 MNAME(12),SMONTH
      CHARACTER*4 SYEAR
      CHARACTER*2 SDAY,SHOUR,SMIN
*---tic mark label storage and formats
      CHARACTER*8 LABOUT
*---corners adjusted for (porig) toggle
      REAL XLOW,YLOW,XHIGH,YHIGH
*---Dual x-scale?
      LOGICAL TOPSCL
 
      INTEGER ISUB,LAXTL,LAYTL,IGRP
      INTEGER IYEAR,IMONTH,IDAY,IHOUR,IMIN,ISEC,IZERO
      INTEGER I,IERR
      REAL UCH1,XTC1,YTC1,XPOS,YPOS,UL
      REAL XSYSZ,YSYSZ,VL
      REAL XW1,YW1,XW2,YW2,YXTCSZ,XYTCSZ,XTPOS
      REAL EXTRA1,EXTRA2
 
      REAL GOODNM
      INTEGER MOLNGT,LENACT
      EXTERNAL MOLNGT,LENACT,GOODNM
 
      SAVE MNAME
      DATA MNAME/'January','February','March','April','May','June',
     &     'July','August','September','October','November','December'/
*-----------------------------------------------------------------------
 
*---Abort if we are not displaying graphics
      IF ( .NOT.ZGRAPH ) RETURN
 
*---copy input window coordinates to common storage
      PW1 = PWL
      QW1 = QWL
      PW2 = PWU
      QW2 = QWU
 
*---Determine whether there is a dual x-scale
      TOPSCL = (NTOP.GT.0)
 
*---calc. width of "size 1.0" character in Norm. Window units.
*---("size 1.0" = 75 char / screen)
      UCH1 = 1.0/(75.*(PW2-PW1))
 
*---reset fast-erase toggles
      TB4 = .FALSE.
      RB4 = .FALSE.
      DO 100 ISUB = 1,M1SUBS
        SB4(ISUB) = .FALSE.
        OB4(ISUB) = .FALSE.
 100  CONTINUE
 
*---calculate the number and separation of  tic marks [data units]
      NXTIC = 8
      NYTIC = 5
      IF ( PORIG .AND. ZSCALE ) THEN
        XLOW = MIN(0.,X1)
        XHIGH = MAX(0.,X2)
        IF ( YFLIP ) THEN
          YLOW = MIN(0.,-Y2)
          YHIGH = MAX(0.,-Y1)
        ELSE
          YLOW = MIN(0.,Y1)
          YHIGH = MAX(0.,Y2)
        ENDIF
      ELSE
        XLOW = X1
        XHIGH = X2
        IF ( YFLIP ) THEN
          YLOW = -Y2
          YHIGH = -Y1
        ELSE
          YLOW = Y1
          YHIGH = Y2
        ENDIF
      ENDIF
 
      CALL DEFAXS(XLOW,XHIGH,XBORDR,XA1,XA2,NXTIC,XTC1,XSTEP)
      CALL DEFAXS(YLOW,YHIGH,YBORDR,YA1,YA2,NYTIC,YTC1,YSTEP)
 
*---Title height , X-title height, Y-title width
*---and legend width [norm. window units]
      VPTLSZ = 0.1
      IF ( TFULL ) THEN
        ULDGSZ = 0.11
      ELSE
        ULDGSZ = 0.0
      ENDIF
 
*---define tic sizes and label sizes [norm. window units]
      UYTCSZ = 0.0125
      VXTCSZ = 0.0125
      VXLBSZ = 0.04
      UYLBSZ = 0.06
 
*---define plot box
      UP1 = UYTLSZ
      VP1 = VXTLSZ
      UP2 = 1.0 - ULDGSZ
      VP2 = 1.0 - VPTLSZ
*---define axes box [norm. window units]
      UA1 = UP1 + UYLBSZ + UYTCSZ
      VA1 = VP1 + VXLBSZ + VXTCSZ
      UA2 = UP2 - UYTCSZ
      VA2 = VP2 - VXTCSZ
      IF ( TOPSCL ) VA2 = VA2 - VXLBSZ - VXTLSZ
*---calculate axes box corners in screen units
      PA1 = PW1 + UA1*(PW2-PW1)
      QA1 = QW1 + VA1*(QW2-QW1)
      PA2 = PW1 + UA2*(PW2-PW1)
      QA2 = QW1 + VA2*(QW2-QW1)
 
*---allow room for subspectra
      NYTIC = NYTIC + SUBS
      YA2 = YA2 + SUBS*YSTEP
*---define data units per axis box
      XPERU = (XA2-XA1)/(UA2-UA1)
      YPERV = (YA2-YA1)/(VA2-VA1)
 
*---make window active graphics area
      CALL GAREA(PW1,QW1,PW2,QW2)
*---define current units to be norm. window units
      CALL GUNIT(0.0,0.0,1.0,1.0)
*---clear area and draw box
      CALL GCLEAR
      CALL GNORMC()
      CALL GFILL(0,1,1)
      CALL GRECTA(UA1,VA1,UA2,VA2)
      CALL GFILL(4,1,1)
      CALL GCHSIZ(1.0,1.0)
 
*---draw titles
 
      IF ( ZTITL ) THEN
*---main title
        CALL GCHJUS('Left')
        XCHSZ = 0.8
        YCHSZ = 0.8
        CALL GCHSIZ(XCHSZ,YCHSZ)
*---place titles above upper left corner of axes box
        CALL GCHOUA(UA1,VP2+VPTLSZ*0.5,PTITLE(1))
        CALL GCHOUA(UA1,VP2+VPTLSZ*0.2,PTITLE(2))
      ENDIF
 
*---axes titles
      IF ( VXTLSZ.GT.0. ) THEN
        XCHSZ = 1.0
        YCHSZ = 1.0
        CALL GCHSIZ(XCHSZ,YCHSZ)
*---x-axis title
        LAXTL = MOLNGT(XTITL)
        XPOS = (UA1+UA2)/2.0 - (XCHSZ*UCH1*LAXTL/2.0)
        LAXTL = LENACT(XTITL)
        CALL MOFONT(XPOS,VXTLSZ/2.0,XTITL(1:LAXTL))
        IF ( TOPSCL ) THEN
          LAXTL = MOLNGT(XTTITL)
          XPOS = (UA1+UA2)/2. - (XCHSZ*UCH1*LAXTL/2.)
          LAXTL = LENACT(XTTITL)
          CALL MOFONT(XPOS,VP2-VXTLSZ/2.,XTTITL(1:LAXTL))
        ENDIF
      ENDIF
*---y-axis title
      IF ( UYTLSZ.GT.0 ) THEN
        LAYTL = MOLNGT(YTITL)
        YPOS = (VA1+VA2)/2.0 - (XCHSZ*UCH1*LAYTL/2.0)
        LAYTL = LENACT(YTITL)
        CALL GCHROT(90.0)
        CALL MOFONT(UYTLSZ/2.0,YPOS,YTITL(1:LAYTL))
        CALL GCHROT(0.0)
      ENDIF
 
*---Draw legend box
      IF ( TFULL .AND. ZSYMBL ) THEN
        CALL GCHJUS('Left')
        UL = 1.0 - 0.8*ULDGSZ
        XCHSZ = 0.8
        YCHSZ = 0.8
        XSYSZ = 1.0
        YSYSZ = 0.5
        DO 150 IGRP = 1,NG
          VL = VP2 - 0.1*IGRP
          CALL GCHSIZ(XSYSZ,YSYSZ)
          CALL GMRKRA(UL,VL,IGRP)
          CALL GCHSIZ(XCHSZ,YCHSZ)
          CALL GCHOUA(UL+0.02*ULDGSZ,VL,GTEXT(IGRP))
 150    CONTINUE
      ENDIF
 
*---Display date/time stamp
      IF ( ZTIMDY ) THEN
        CALL GCHJUS('Right')
        XCHSZ = 0.8
        YCHSZ = 0.8
        CALL GCHSIZ(XCHSZ,YCHSZ)
        CALL OSDATE(IYEAR,IMONTH,IDAY)
        CALL OSTIME(IHOUR,IMIN,ISEC)
 
        CALL INTSTR(IYEAR,SYEAR,'(I4)')
        SMONTH = MNAME(IMONTH)
        CALL INTSTR(IDAY,SDAY,'(I2)')
        CALL JUSTIF(SDAY,'Left')
 
        IZERO = ICHAR('0')
        SHOUR = CHAR(IZERO+IHOUR/10)//CHAR(IZERO+MOD(IHOUR,10))
        SMIN = CHAR(IZERO+IMIN/10)//CHAR(IZERO+MOD(IMIN,10))
 
        CALL GCHOUA(1.0,0.02,SMONTH(1:LENACT(SMONTH))
     &              //' '//SDAY(1:LENACT(SDAY))
     &              //', '//SYEAR//'  '//SHOUR//':'//SMIN)
      ENDIF
 
 
*---define tic label size
      XCHSZ = 1.0
      YCHSZ = 1.0
      CALL GCHSIZ(XCHSZ,YCHSZ)
 
*---redefine units in axes box as data units
      XW1 = XA1 - (UYTLSZ+UYTCSZ+UYLBSZ)*XPERU
      YW1 = YA1 - (VXTLSZ+VXTCSZ+VXLBSZ)*YPERV
      XW2 = XA2 + (UYTCSZ+ULDGSZ)*XPERU
      YW2 = YA2 + (VXTCSZ+VPTLSZ)*YPERV
      IF ( TOPSCL ) YW2 = YW2 + (VXLBSZ+VXTLSZ)*YPERV
      CALL GUNIT(XW1,YW1,XW2,YW2)
 
*---draw tics (and labels)
      CALL GCHJUS('Center')
 
*---x-axis
      CALL GCHJUS('Center')
*---convert tic size to data units
      YXTCSZ = VXTCSZ*YPERV
      YPOS = YA1 - YXTCSZ - VXLBSZ*0.5*YPERV
      DO 200 I = 1,NXTIC
*---tic value
        XPOS = XTC1 + (I-1)*XSTEP
*---convert tick value to string
        CALL CONVRT(XPOS,LABOUT,.TRUE.)
        CALL NOBLNK(LABOUT,IERR)
*---print value
        CALL GCHOUA(XPOS,YPOS,LABOUT(1:LENACT(LABOUT)))
*---draw tick mark on both top and bottom of axes box
        CALL GMOVEA(XPOS,YA1)
        CALL GLINER(0.0,-YXTCSZ)
        IF ( .NOT.TOPSCL ) THEN
          CALL GMOVEA(XPOS,YA2)
          CALL GLINER(0.0,YXTCSZ)
        ENDIF
 200  CONTINUE
 
      IF ( TOPSCL ) THEN
        YPOS = YA2 + YXTCSZ + VXLBSZ*0.5*YPERV
        DO 250 I = 1,NTOP
*---tic value
          XTPOS = TOPLST(1,I)
          XPOS = TOPLST(2,I)
*---convert tick value to string
          CALL CONVRT(XTPOS,LABOUT,.TRUE.)
          CALL NOBLNK(LABOUT,IERR)
*---print value
          CALL GCHOUA(XPOS,YPOS,LABOUT(1:LENACT(LABOUT)))
*---draw tick mark on top of axes box
          CALL GMOVEA(XPOS,YA2)
          CALL GLINER(0.0,YXTCSZ)
 250    CONTINUE
      ENDIF
 
*---Additional tics for channels
      EXTRA1 = GOODNM(XSTEP/5.0,.TRUE.)
      EXTRA2 = EXTRA1/2.0
      IF ( (KXPLOT.EQ.1) .AND. (XSTEP.GE.50) .AND. (PROG.EQ.'WMOSS') )
     &     THEN
        DO 300 I = (2*(NINT(X1/EXTRA2+.4999995)/2)+1)*EXTRA2,
     &    NINT(X2/EXTRA2-.4999995)*EXTRA2,EXTRA2
*---draw tick mark on both top and bottom of axes box
          XPOS = FLOAT(I)
          CALL GMOVEA(XPOS,YA1)
          CALL GLINER(0.0,-YXTCSZ/2)
          CALL GMOVEA(XPOS,YA2)
          CALL GLINER(0.0,YXTCSZ/2)
 300    CONTINUE
      ENDIF
 
      IF ( (KXPLOT.EQ.1) .AND. (XSTEP.GE.10) .AND. (PROG.EQ.'WMOSS') )
     &     THEN
        DO 350 I = NINT(X1/EXTRA1+.4999995)*EXTRA1,
     &    NINT(X2/EXTRA1-.4999995)*EXTRA1,EXTRA1
*---draw tick mark on both top and bottom of axes box
          XPOS = FLOAT(I)
          CALL GMOVEA(XPOS,YA1)
          CALL GLINER(0.0,-YXTCSZ)
          CALL GMOVEA(XPOS,YA2)
          CALL GLINER(0.0,YXTCSZ)
 350    CONTINUE
      ENDIF
 
*---y-axis
      CALL GCHJUS('Right')
*---convert tic size to data units
      XYTCSZ = UYTCSZ*XPERU
 
      XPOS = XA1 - 1.1*XYTCSZ
      DO 400 I = 1,NYTIC
*---tic value
        YPOS = YTC1 + (I-1)*YSTEP
        IF ( ABS(YPOS).LT.(ABS(YTC1)+ABS(NYTIC*YSTEP))/1000. ) YPOS = 0.
        IF ( YFLIP ) THEN
          CALL CONVRT(-YPOS,LABOUT,.TRUE.)
        ELSE
          CALL CONVRT(YPOS,LABOUT,.TRUE.)
        ENDIF
        CALL NOBLNK(LABOUT,IERR)
*---print value
        CALL GCHOUA(XPOS,YPOS,LABOUT(1:LENACT(LABOUT)))
*---draw tick mark on both left and right sides of plot
        CALL GMOVEA(XA1,YPOS)
        CALL GLINER(-XYTCSZ,0.0)
        CALL GMOVEA(XA2,YPOS)
        CALL GLINER(XYTCSZ,0.0)
 400  CONTINUE
 
*---restore justification to 'Left'
      CALL GCHJUS('Left')
*---restore default character size
      XCHSZ = 1.0
      YCHSZ = 1.0
      CALL GCHSIZ(XCHSZ,YCHSZ)
 
      RETURN
      END
