*     PARCOM.INC
*     MOSS/EPR/MAG Version
*--- Contact- OLD: WEB Research Co. /// NEW: WMOSS.ORG
*     mru 27Jul93 cs

*--------------------*
* Fitting Parameters *
*--------------------*--------------------------------------------------
      common /fitpar/ nfipar, filver, jput, maxitg, maxitl, maxita,
     &   nrprtg, nrprtl, neal, nmonte, model, noptmz, ineal,
     &   fitnam, parcmt
*-----------------------------------------------------------------------
      integer ifp(MFPAR), nfipar, filver, jput, maxitg, maxitl, maxita
      integer nrprtg, nrprtl, neal, nmonte, model, noptmz
      integer ineal
      character*10 fitnam(MFPAR)
      character*57 parcmt(4)

      equivalence(nfipar,ifp(1))
*-----------------------------------------------------------------------
* nfipar : Number of fitting parameters read in.
* filver : Version number of parameter file.  THIS MUST BE NEGATIVE!
*        : If the number of the "current" version must change, the
*        : number should be decreased by one.
* jput   : Text status info during a fit if this is nonzero.
* maxitg : Maximum number of optimizer iterations over GLOBAL variables.
* maxitl : Maximum number of optimizer iterations over LOCAL variables.
* maxita : Maximum number of optimizer iterations over GLOBAL AREA vars.
* nrprtg : Global fitting routines call REPORT every nrprtg iterations.
* nrprtl : Local fitting routines call REPORT every nrprtl iterations.
* neal   : Number of annealing iterations.
* nmonte : Number of monte-carlo simulations.
* model  : Physical model (1 = Spin Hamiltonian, 2 = Single Lines,
*        :    3 = doublets).
* noptmz : Optimizer to use (1 = SOPTMZ, 2 = LMDIF, 
*        :    n>2 = (n * #vars) iterations of SOPTMZ, then LMDIF).
* ifp    : The above twelve parameters viewed as an array of integers.
* fitnam : Ascii names for the fitting parameters.
*
* parcmt : Parameter files comments (1&2 = user defined,
*             3 = last chi-squared, 4 = last data files fit)
*-----------------------------------------------------------------------


*-----------------------*
* Parameters file names *
*-----------------------*-----------------------------------------------
      common /fils/ filnam
*-----------------------------------------------------------------------
      character*(MFLLEN) filnam(MFIL)
*-----------------------------------------------------------------------
* filnam : List of param. file names (1 = input file, 2 = output file)
*-----------------------------------------------------------------------



*-----------------------------*
* LEVEL 1 : Global parameters *
*-----------------------------*-----------------------------------------
      common /giprm/ igp, igpnam
      common /grprm/ pg0, pglb, pgub, weig, kong, indg, nglo, pgnam
*-----------------------------------------------------------------------
      integer igp(MIGPAR,MGLO), nsubs, nglo
      character*10 igpnam(MIGPAR,MGLO)
      equivalence(nsubs,igp(3,1))

      real pg0(MGPAR,MGLO), pglb(MGPAR,MGLO), pgub(MGPAR,MGLO)
      real weig(MGPAR,MGLO)
      integer kong(MGPAR,MGLO), indg(MGPAR,MGLO)
      character*10 pgnam(MGPAR,MGLO)
*-----------------------------------------------------------------------
* igp    : Array of integer global parameters.
*        : igp(1,n) = # of integer global parameters for global site #n
*        : igp(2,n) = # of global nonlinear parameters for glob. site #n
* igpnam : ASCII names for each of the global integers.
* nsubs  : Number of subsites in the current parameter set <=> igp(3,n)
*
* pg0    : Array of real global parameters.
* pglb   : Array of lower bounds for the global reals.
* pgub   : Array of upper bounds for the global reals.
* weig   : Weights for the global reals; used when a global real is
*        :    constrained to another parameter.
* kong   : Constraint array for global reals.
* indg   : Constraint array for global reals in CNSAPP form.
* pgnam  : ASCII names for the global real parameters.
* ngpar  : Total number of global real parameters.
*
* (See below for global linear parameters)
*-----------------------------------------------------------------------

*----------------------------*
* LEVEL 2 : Local parameters *
*----------------------------*------------------------------------------
      common /liprm/ ilp, ilpnam
      common /lrprm/ pl0, pllb, plub, weil, konl, indl, plnam
*-----------------------------------------------------------------------
      integer ilp(MILPAR,MLOC)
      character*10 ilpnam(MILPAR,MLOC)

      real pl0(MLPAR,MLOC), pllb(MLPAR,MLOC), plub(MLPAR,MLOC)
      real weil(MLPAR,MLOC)
      integer konl(MLPAR,MLOC), indl(MLPAR,MLOC)
      character*10 plnam(MLPAR,MLOC)
*-----------------------------------------------------------------------
* ilp    : Array of integer local parameters.
*        : ilp(1,n) = # of local integer parameters for subsite #n
*        : ilp(2,n) = # of local nonlinear parm for subsite #n
*        : ilp(3,n) = # of linear area fraction parm for subsite #n
* ilpnam : ASCII names for the local integer parameters.
*
* pl0    : Array of real local parameters.
* pllb   : Array of lower bounds for the local reals.
* plub   : Array of upper bounds for the local reals.
* weil   : Weights for the local reals; used when a local real is
*        :    constrained to another parameter.
* konl   : Constraint array for local reals.
* indl   : Constraint array for local reals in CNSAPP form.
* plnam  : ASCII names for the local real parameters.
* nlpar  : Total number of local real parameters.
*
* (See below for local linear parameters)
*-----------------------------------------------------------------------


*------------------------*
* LEVEL 3 : Global areas *
*------------------------*----------------------------------------------
      COMMON /giaprm/ iap
      COMMON /gaprm/  pa0, palb, paub, weia, kona, inda, panam
*-----------------------------------------------------------------------
      integer iap(2,LT1)

      real pa0(MAPAR,LT1), palb(MAPAR,LT1), paub(MAPAR,LT1)
      real weia(MAPAR,LT1)
      integer kona(MAPAR,LT1), inda(MAPAR,LT1)
      character*10 panam(MAPAR,LT1)
*-----------------------------------------------------------------------
* ilp    : Integer global area parameters.
*        : ilp(1,n) = # of ilp parameters for spectrum n
*        : ilp(2,n) = # of global area parameters for spectrum n
*
* pa0    : Array of global area parameters.
* palb   : Array of lower bounds for the global areas.
* paub   : Array of upper bounds for the global areas.
* weia   : Weights for the global areas; used when a global area is
*        :    constrained to another parameter.
* kona   : Constraint array for global areas.
* inda   : Constraint array for global areas in CNSAPP form.
* panam  : ASCII names for the global area parameters.
*-----------------------------------------------------------------------


*--------------------------------------------*
* LEVEL 4 : Global & Local Linear parameters *
*--------------------------------------------*--------------------------
      common /lrlprm/ pln0, plnlb, plnub, weiln, konln, indln,
     &   iblp, pbl0, pbllb, pblub, weibl, konbl, indbl, plnnam, pblnam
*-----------------------------------------------------------------------
      real pln0(MLLPAR,MLOC), plnlb(MLLPAR,MLOC), plnub(MLLPAR,MLOC)
      real weiln(MLLPAR,MLOC)
      real pbl0(MBLPAR,LT1), pbllb(MBLPAR,LT1), pblub(MBLPAR,LT1)
      real weibl(MBLPAR,LT1)
      integer konln(MLLPAR,MLOC), indln(MLLPAR,MLOC)
      integer konbl(MBLPAR,LT1), indbl(MBLPAR,LT1), iblp(MIBPAR,LT1)
      character*10 plnnam(MLLPAR,MLOC), pblnam(MBLPAR,LT1)
*-----------------------------------------------------------------------
* pln0   : Array of global area parameters.
* plnlb  : Array of lower bounds for the global areas.
* plnub  : Array of upper bounds for the global areas.
* weiln  : Weights for the global areas; used when a global area is
*        : constrained to another parameter.
* konln  : Constraint array for global areas.
* indln  : Constraint array for area fractions in CNSAPP form.
* plnnam : ASCII names for the area fraction parameters.
* nlnpar : Total number of LOCAL AREA-FRACTION parameters (NO baselines)
* iblp   : 
* pbl0   :
* pbllb  :
* pblub  :
* weibl  :
* konbl  :
* indbl  :
* pblnam :
*-----------------------------------------------------------------------

*-----------*
* Fit table *
*-----------*-----------------------------------------------------------
      COMMON /fittbl/ spec, spec0, snoys, specx, locspc, slcspc, cal,
     &   ical, nvec, smask, nspec, nfrpt, plcopy, sname, scomnt, ftitle
*-----------------------------------------------------------------------
      real spec(LSPEC,LT1,2), spec0(LSPEC,LT1,1), snoys(LSPEC,LT1,2)
      real specx(LSPEC,LT1)
      real locspc(LSPEC,LT1,MLOC), slcspc(LSPEC,LT1,MLOC)
      real cal(MCAL,LT1), nvec(MAXPTS)
      real plcopy(MLPAR,MLOC)
      logical smask(LSPEC,LT1)
      integer nspec, ical(MCAL,LT1), nfrpt
      character*(MFLLEN) sname(LT1)
      character*72 scomnt(6,LT1)
      character*72 ftitle(2)
*-----------------------------------------------------------------------
* spec   : Data to fit to in spec(*,*,1); calculated theory in (*,*,2).
* spec0  : Original data with no artificial noise added.
* snoys  : Array of noise values for the data in (*,*,1).
* specx  : Array of x-positions for the data being fitted
* locspc : Local subsites output from the local calculations.
* slcspc : Scaled subsites output from the global area calculations.
* cal    : Real calibration constants for the spectra being fitted.
* ical   : Integer calibration constants for the spectra being fitted.
* nvec   : Noise vector: snoys with masked or missing points deleted.
* smask  : List of masks for the data being fitted.
* nspec  : Number of spectra being fitted.
* nfrpt  : Number of unmasked points being fitted. (# of free points)
* plcopy : Complete set of local params from the last local calculation
*        :    (used to prevent calculating the exact same subsite 2x).
* sname  : List of names for the data being fitted.
* scomnt : List of comment sets for the data being fitted.
* ftitle : Fitting title.
*-----------------------------------------------------------------------
           

*------------------*
* Parameter vector *
*------------------*----------------------------------------------------
      common /pvec/ nptot, parvec, parub, parlb, partyp, paridx, parnam 
*-----------------------------------------------------------------------
      integer nptot
      character*10 parnam(PARMAX)
      real parvec(PARMAX), parub(PARMAX), parlb(PARMAX)
      integer partyp(PARMAX), paridx(PARMAX)
*-----------------------------------------------------------------------
* nptot  : Total number of parameters in vector.
* parvec : List of parameter values.
* parub  : List of parameter upper bounds.
* parlb  : List of parameter lower bounds.
* partyp : Type of parameter (see mancom.inc for codes)
* paridx : Number of parameter within a given type.
* parnam : List of parameter names.
*-----------------------------------------------------------------------
