*-----------------------------------------------------------------------
      real function cubic(p,q,r)
*-----------------------------------------------------------------------
*--- Contact- OLD: WEB Research Co. /// NEW: WMOSS.ORG
*    15Sep92 CS,  mru 19Jun93 CS
*
*     Returns a single real solution of the equation:
*          3    2
*         x + px + qx + r = 0
*
*     Where p, q, and r are real coefficients.
*     See CRC Mathematics Handbook for method.
*-----------------------------------------------------------------------
      real p,q,r
*-----------------------------------------------------------------------
      real a, b, temp, ans, a1, b1
      complex t1
*-----------------------------------------------------------------------

      a = q - (p*p)/3.
      b = p*(2*(p*p)-9*q)/27. + r
      temp = sqrt(max(0.,-b*b/4.-(a**3)/27.))

c      a1 = exp(log(cmplx(-b/2.,temp))/3.)
c      b1 = exp(log(cmplx(-b/2,-temp))/3.)
      t1 = log(cmplx(-b/2.,temp))/3.0
      a1 = exp(real(t1))*cos(aimag(t1))
      t1 = log(cmplx(-b/2.,-temp))/3.0
      b1 = exp(real(t1))*cos(aimag(t1))

      ans = a1 + b1 - p/3.

*---Polish root (1 newton iteration)
      ans = ans - (((ans+p)*ans+q)*ans+r) / ((3*ans+2*p)*ans+q)

      cubic = ans

      return
      end

*-----------------------------------------------------------------------
      subroutine poly4(b,c,d,x)
*-----------------------------------------------------------------------
*--- Contact- OLD: WEB Research Co. /// NEW: WMOSS.ORG
*     15Sep92 CS,  mru 19Jun93 CS
*
*     Returns four real solutions to a fourth-degree polynomial with
*     real coefficients:
*      4    2
*     x + bx + cx + d = 0
*
*     See CRC Mathematics handbook for method.
*-----------------------------------------------------------------------
      real b,c,d,x(4)

      real temp2, temp, y, rr, r, d1, e1, r2, e2, d2

      real cubic
      external cubic
      
      y = cubic(-b,-4*d,4*b*d-c**2)

      rr = y - b
      r = sqrt(rr)

      temp2 = -2 * b - rr

      if (r.gt.0) then
         temp = -2 * c / r
         d1 = sqrt(temp2+temp)
         e1 = sqrt(temp2-temp)
      else
         temp = 2*sqrt(y**2-4*d)
         d1 = sqrt(temp2+temp)
         e1 = sqrt(temp2-temp)
      endif

      r2 = r / 2
      d2 = d1 / 2
      e2 = e1 / 2
      x(1) = r2 + d2
      x(2) = r2 - d2
      x(3) = -r2 + e2
      x(4) = -r2 - e2

*---Polish roots: 1 newton iteration per root
      x(1) = x(1) - (((x(1)**2 + b) * x(1) + c)* x(1) + d) /
     &   ((4*x(1)**2 + 2*b) * x(1) + c)
      x(2) = x(2) - (((x(2)**2 + b) * x(2) + c)* x(2) + d) /
     &   ((4*x(2)**2 + 2*b) * x(2) + c)
      x(3) = x(3) - (((x(3)**2 + b) * x(3) + c)* x(3) + d) /
     &   ((4*x(3)**2 + 2*b) * x(3) + c)
      x(4) = x(4) - (((x(4)**2 + b) * x(4) + c)* x(4) + d) /
     &   ((4*x(4)**2 + 2*b) * x(4) + c)
      
      return
      end

*----------------------------------------------------------------------
      subroutine eigval4(coef,eigv)
*----------------------------------------------------------------------
*--- Contact- OLD: WEB Research Co. /// NEW: WMOSS.ORG
*    15Sep92 CS,  mru 19Jun93 CS
*
*     Determines the eigenvalues of a 4x4 matrix of the form:
*
*  [ c +3c             c +c - i(c +c )   c - i(c  )        0             ]
*  [  1   2             3  4     5  6     9     10                       ]
*  [                                                                     ]
*  [ c +c + i(c +c )   c - c             c - i(c )         c - i(c  )    ]
*  [  3  4     5  6     2   1             7     8           9     10     ]
*  [                                                                     ]
*  [ c + i(c  )        c + i(c )        -c - c             c -c +i(c -c) ]
*  [  9     10          7     8           1   2             4  3    5  6 ]
*  [                                                                     ]
*  [ 0                 c + i(c  )        c -c + i(c -c )   c -3c         ]
*  [                    9     10          4  3     6  5     1   2        ]
*
*  where the c's are all real values.
*
*  Notation: the letters {a,b,c,d,e,f,g,h,j,k} represent c1..c10 resp.
*----------------------------------------------------------------------
      real coef(10),eigv(4)
      real aa,bb,cc,dd,ee,ff,gg,hh,jj,kk,ab,cd,ef
      real ccdd, eeff, gghh, jjkk, gjhk, hjgk, ccmdd, eemff
      real c0,c1,c2

*---Define subexpressions which occur repeatedly.
      aa = coef(1)**2
      bb = coef(2)**2
      cc = coef(3)**2
      dd = coef(4)**2
      ee = coef(5)**2
      ff = coef(6)**2
      gg = coef(7)**2
      hh = coef(8)**2
      jj = coef(9)**2
      kk = coef(10)**2

      ab = coef(1)*coef(2)
      cd = coef(3)*coef(4)
      ef = coef(5)*coef(6)

      ccdd = cc + dd
      eeff = ee + ff
      gghh = gg + hh
      jjkk = jj + kk
      
*---Determine c2, c1, and c0, the coeffecients of the
*---characteristic equation

      c2 = -2*(aa+ccdd+eeff+jjkk)-10*bb-gghh

      gjhk = coef(7)*coef(9) + coef(8)*coef(10)
      hjgk = coef(8)*coef(9) - coef(7)*coef(10)

      c1 = -16*coef(2)*(ab+cd+ef) + 2*coef(1)*gghh - 
     &   4*(coef(4)*gjhk - coef(6)*hjgk)

      ccmdd = cc - dd
      eemff = ee - ff

      c0 = (aa-9*bb)*(aa-bb) + (ccmdd)**2 + (eemff)**2 + jjkk**2 +
     &   aa*(2*(ccdd + eeff + jjkk) - gghh) -
     &   8*(ab*(cd+ef) + cd*ef)
      c0 = c0 + 4*coef(1)*(coef(4)*gjhk - coef(6)*hjgk) -
     &   6*bb*(ccdd + eeff - jjkk - 1.5*gghh) -
     &   12*coef(2)*(coef(3)*gjhk - coef(5)*hjgk)
      c0 = c0 + 2*(jjkk*(ccmdd+eemff)+ccdd*eeff)

*---Solve the characteristic equation for the eigenvalues.
      call poly4(c2, c1, c0, eigv)

      return
      end

*----------------------------------------------------------------------
      subroutine eigvec4(coef,eigval,eigvec,ierr)
*----------------------------------------------------------------------
*--- Contact- OLD: WEB Research Co. /// NEW: WMOSS.ORG
*    15Sep92 CS,  mru 19Jun93 CS
*
*     Given the eigenvalue of a matrix with entries as defined in
*     EIGVAL4, attempts to determine the eigenvectors by using
*     a quick method using the cofactors; if this method fails,
*     an error code is returned so that the more stable x4 routines
*     can be called for this particular matrix.
*----------------------------------------------------------------------
      real coef(10),eigval(4)
      complex eigvec(4,4)
      integer ierr
*-----------------------------------------------------------------------
      complex a(4,4), c1, c2, c3, c4
      complex m12, m13, m14, m23, m24
      real m34
      real a11, a33, a44

      integer i
      real size
*-----------------------------------------------------------------------
      real mag2
      complex zz
      mag2(zz) = real(zz)**2 + aimag(zz)**2
*-----------------------------------------------------------------------

*---Check for duplicate eigenvalues:  This method will not generate
*   orthogonal eigenvectors for a degenerate eigenvalue.
      if (abs(eigval(1)-eigval(2)).lt.1e-4) then
         ierr = 12
      elseif (abs(eigval(1)-eigval(3)).lt.1e-4) then
         ierr = 13
      elseif (abs(eigval(1)-eigval(4)).lt.1e-4) then
         ierr = 14
      elseif (abs(eigval(2)-eigval(3)).lt.1e-4) then
         ierr = 23
      elseif (abs(eigval(2)-eigval(4)).lt.1e-4) then
         ierr = 24
      elseif (abs(eigval(3)-eigval(4)).lt.1e-4) then
         ierr = 34
      else
         ierr = 0
      
*---Calculate the eigenvectors
         do 100 i=1,4

*---Second row and zero values are never referenced, and hence not stored.
            a11 = coef(1)+3*coef(2)-eigval(i)
c              a(2,1) = cmplx(coef(3)+coef(4),coef(5)+coef(6))
            a(3,1) = cmplx(coef(9),coef(10))
c              a(4,1) = 0.

            a(1,2) = cmplx(coef(3)+coef(4),-coef(5)-coef(6))
c              a22    = coef(2)-coef(1)-eigval(i)
            a(3,2) = cmplx(coef(7),coef(8))
            a(4,2) = a(3,1)

            a(1,3) = conjg(a(3,1))
c              a(2,3) = conjg(a(3,2))
            a33    = -coef(1)-coef(2)-eigval(i)
            a(4,3) = cmplx(coef(4)-coef(3),coef(6)-coef(5))

            a(1,4) = 0.
c              a(2,4) = conjg(a(4,2))
            a(3,4) = conjg(a(4,3))
            a44    = coef(1)-3*coef(2)-eigval(i)

*---Calculate all 2x2 minors in the last 2 rows
            m12 = a(3,1)**2
            m13 = a(3,1)*a(4,3)
            m14 = a(3,1)*a44
            m23 = a(3,2)*a(4,3) - a33*a(4,2)
            m24 = a(3,2)*a44 - a(3,4)*a(4,2)
            m34 = a33*a44 - mag2(a(3,4))

*---Determine eigenvector; the coordinates are proportional to the
*---cofactors as we expand along the second row (second row so
*---that we can use the fact that a(1,4) = a(4,1) = 0)
            c1 = a(1,2)* m34 - a(1,3) * m24
            c2 = -a11  * m34 + a(1,3) * m14
            c3 = a11   * m24 - a(1,2) * m14
            c4 = -a11  * m23 + a(1,2) * m13 - a(1,3) * m12

*---Normalize vector, but don't force the 4th component to be real
            size = sqrt(mag2(c1) + mag2(c2) + mag2(c3) + mag2(c4))
            if (size.gt.1.0e-4) then
               eigvec(4,i) = c4 / size
               eigvec(3,i) = c3 / size
               eigvec(2,i) = c2 / size
               eigvec(1,i) = c1 / size
            else
               ierr = i
               goto 1000
            endif
  100    continue
 1000 endif
      return
      end
