      SUBROUTINE PSUBAY(IERR)
*-----------------------------------------------------------------------
*--- Contact- OLD: WEB Research Co. /// NEW: WMOSS.ORG
*---orig: 900123 TAK
*--- mru: 27May94 cs; 960527 tk
*-----------------------------------------------------------------------
*
*     Subroutine PSUBAY performs a least squares fit of
*        a user supplied model to a given data set.  A pseudo
*        bayesian technique is used in that for each point in
*        the non-linear parameter space a non-iterative fit is
*        performed for the linear parameters.
*
*     notes:
*     1. Noise in data must be defined.  Precise noise values
*        are required for proper calculation of chi^2.
*     2. Option for annealing of final fit is provided.
*     3. Option for analyzing correlation of free parameters via
*        monte-carlo simulation is provided.
*-----------------------------------------------------------------------
      INCLUDE 'limits.inc'
      INCLUDE 'iocom.inc'
      INCLUDE 'parcom.inc'
      INCLUDE 'speccom.inc'
      INCLUDE 'constcom.inc'
      INCLUDE 'varcom.inc'
      INCLUDE 'bestcom.inc'
      INCLUDE 'togcom.inc'
      INCLUDE 'errcodes.inc'
 
      INTEGER I,J,ISPEC
      INTEGER ISUB
      REAL RX2MC0
 
      LOGICAL WASOPN
 
*---STORAGE FOR PARAMETER VECTORS GENERATED BY MONTE CARLO SIMULATIONS
      REAL VFREE(MVAR,MMONTE),X2MC(MMONTE)
      CHARACTER VNAME(MVAR)*10,CHISTR*8
      INTEGER NTOTFR
 
*---STORAGE OF CURRENT XI^2 VALUE DURING ANNEALLING
      REAL X2NOW,RX2NOW
 
*---ARRAY FOR BEST VALUES OF FREE VARIABLES AFTER ANNEALING
      REAL VGBA(MVAR)
      REAL VLBA(MVAR)
      REAL VABA(MVAR)
      REAL VLINBA(MVAR)
      REAL VBLBA(MVAR)
 
*---switch for choosing variable inialization method.
      LOGICAL SETRAN
*---Count of number of Monte Carlo results written to disk
      INTEGER MCCNT
 
*---Error flags
      INTEGER IERR,NERR,IERR1
*---Return flag from printer checking routine
      LOGICAL RESULT
 
      REAL DIFSPC(MAXPTS)
 
      EXTERNAL CALGL1,CALGL2,CALGL3,CALGL4,CALGL5
*-----------------------------------------------------------------------
      INTEGER IJ2KON
      IJ2KON(I,J) = I + 100*J
*-----------------------------------------------------------------------
*----EXECUTABLE STATEMENTS START HERE.
*-----------------------------------------------------------------------
 
*_Probably not necessary in general, harmful for iron cal which doesn't
*_have any data on the list yet.
*c     ;; Make sure all table info is up-to-date
*      call maktab(0)
 
c     ;; Initialize user model at all levels
      CALL INTFUN(0,IERR)
      IF ( IERR.NE.0 ) RETURN
 
c     ;; Construct constraint index vectors from kon vector
      CALL CNSTRN(KONG,INDG,NGFREE,MGPAR,NGLO,IERR,NERR)
      IF ( IERR.NE.0 ) THEN
        CALL ERRMSG('Error:  Invalid global constraint!')
        RETURN
      ENDIF
      CALL CNSTRN(KONL,INDL,NLFREE,MLPAR,NSUBS,IERR,NERR)
      IF ( IERR.NE.0 ) THEN
        CALL ERRMSG('Error:  Invalid local constraint!')
        RETURN
      ENDIF
      CALL CNSTRN(KONA,INDA,NAFREE,MAPAR,NSPEC,IERR,NERR)
      IF ( IERR.NE.0 ) THEN
        CALL ERRMSG('Error:  Invalid global area constraint!')
        RETURN
      ENDIF
      CALL CNSTRN(KONLN,INDLN,NLNFR,MLLPAR,NSUBS,IERR,NERR)
      IF ( IERR.NE.0 ) THEN
        CALL ERRMSG('Error:  Invalid area fraction constraint!')
        RETURN
      ENDIF
      CALL CNSTRN(KONBL,INDBL,NBLFR,MBLPAR,NSPEC,IERR,NERR)
      IF ( IERR.NE.0 ) THEN
        CALL ERRMSG('Error:  Invalid baseline constraint!')
        RETURN
      ENDIF
 
c     ;; Direct Annealling log
      IF ( JPUT.NE.0 ) THEN
        CALL CHKPRN(TXPORT,RESULT)
        IF ( RESULT ) THEN
          JGO = JPRN
          CALL OPNTXT(WASOPN,IERR1)
          RESULT = (IERR1.EQ.0)
        ENDIF
      ENDIF
 
c     ;; Initialize count of accepted Monte Carlo iterations
      MCCNT = 0
 
c     ;; Initialize upper and lower bounds of free variables
      CALL VBOUND(MGPAR,NGLO,PGLB,PGUB,KONG,INDG,MVAR,VGLB,VGUB)
      CALL VBOUND(MLPAR,NSUBS,PLLB,PLUB,KONL,INDL,MVAR,VLLB,VLUB)
      CALL VBOUND(MAPAR,NSPEC,PALB,PAUB,KONA,INDA,MVAR,VALB,VAUB)
      CALL VBOUND(MLLPAR,NSUBS,PLNLB,PLNUB,KONLN,INDLN,MVAR,VLNLB,VLNUB)
      CALL VBOUND(MBLPAR,NSPEC,PLNLB,PLNUB,KONLN,INDLN,MVAR,VBLLB,VBLUB)
 
*---Setup free variable names
      IF ( NMONTE.GT.0 ) THEN
        NTOTFR = 0
c        ;; Global Nonlinear
        DO 50 J = 1,NGLO
          DO 20 I = 1,IGP(2,J)
            IF ( KONG(I,J).EQ.IJ2KON(I,J) ) THEN
              NTOTFR = NTOTFR + 1
              VNAME(NTOTFR) = PGNAM(I,J)
            ENDIF
 20       CONTINUE
 50     CONTINUE
 
c     ;; Local
        DO 100 ISUB = 1,NSUBS
          DO 60 I = 1,ILP(2,ISUB)
            IF ( KONL(I,ISUB).EQ.IJ2KON(I,ISUB) ) THEN
              NTOTFR = NTOTFR + 1
              VNAME(NTOTFR) = PLNAM(I,ISUB)
            ENDIF
 60       CONTINUE
          DO 80 I = 1,ILP(3,ISUB)
            IF ( KONLN(I,ISUB).EQ.IJ2KON(I,ISUB) ) THEN
              NTOTFR = NTOTFR + 1
              VNAME(NTOTFR) = PLNNAM(I,ISUB)
            ENDIF
 80       CONTINUE
 100    CONTINUE
 
c     ;; Spectrum-related
        DO 150 ISPEC = 1,NSPEC
c        ;; Global Areas
          DO 120 I = 1,IAP(2,ISPEC)
            IF ( KONA(I,ISPEC).EQ.IJ2KON(I,ISPEC) ) THEN
              NTOTFR = NTOTFR + 1
              VNAME(NTOTFR) = PANAM(I,ISPEC)
            ENDIF
 120      CONTINUE
 
c        ;; Baselines
          DO 140 I = 1,IBLP(2,ISPEC)
            IF ( KONBL(I,ISPEC).EQ.IJ2KON(I,ISPEC) ) THEN
              NTOTFR = NTOTFR + 1
              VNAME(NTOTFR) = PBLNAM(I,ISPEC)
            ENDIF
 140      CONTINUE
 150    CONTINUE
      ENDIF
 
*------------------------------------
*---Annealing Loop
*------------------------------------
      RX2BST = 1.7E38
 
      DO 300 INEAL = 0,NEAL
        ABORT = ABORTOFF
 
c           ;; Initialize Variables:
c           ;; Start at initial guess if ineal = 0, and at a random
c           ;; point within bounds otherwise
c 930923 & 9310113 tk:     "setran = (ineal.gt.0)" changed to
        SETRAN = (INEAL.GT.0 .AND. NMONTE.EQ.0)
 
        CALL INTVAR(SETRAN,MGPAR,NGLO,PG0,MVAR,VG0,VGLB,VGUB,KONG,INDG)
        CALL INTVAR(SETRAN,MLPAR,NSUBS,PL0,MVAR,VL0,VLLB,VLUB,KONL,INDL)
        CALL INTVAR(SETRAN,MAPAR,NSPEC,PA0,MVAR,VA0,VALB,VAUB,KONA,INDA)
        CALL INTVAR(SETRAN,MLLPAR,NSUBS,PLN0,MVAR,VLN0,VLNLB,VLNUB,
     &              KONLN,INDLN)
        CALL INTVAR(SETRAN,MBLPAR,NSPEC,PBL0,MVAR,VBL0,VBLLB,VBLUB,
     &              KONBL,INDBL)
 
c        ;; If Monte Carlo switch set, add noise to original fit (theory).
        CALL NUDATA((NMONTE.NE.0) .AND. (INEAL.GT.0))
 
c        ;; Display data
        CALL REPPLT(.FALSE.)
        REPLOT(LOCLEV) = .TRUE.
        REPLOT(GLOLEV) = .TRUE.
 
*-------------------------------
*---Optimization
*-------------------------------
 
        CALL CALOPT(NGFREE,VG0,VGLB,VGUB,NFRPT,MAXITG,GLOLEV,NOPTMZ,
     &              CALGL1,CALGL2,CALGL3,CALGL4,CALGL5,VGBEST,X2NOW,
     &              DIFSPC,IERR)
 
        IF ( ABORT.EQ.ABORTFATAL ) THEN
          IERR = ERRABORT
          GOTO 800
        ENDIF
 
        RX2NOW = X2NOW/FLOAT(NFRPT-NLFREE-NLNFR-NGFREE)
 
        IF ( INEAL.EQ.0 ) RX2MC0 = RX2NOW
 
c        ;; Annealing Log Header
        IF ( JPUT.NE.0 ) THEN
          IF ( INEAL.EQ.0 ) WRITE (JGO,*) 'Annealing Log:'
          WRITE (JGO,9001) INEAL
          WRITE (JGO,9002) RX2NOW
        ENDIF
 
c        ;; Save new best variables
        IF ( (NMONTE.EQ.0 .AND. RX2NOW.LT.RX2BST) .OR. INEAL.EQ.0 ) THEN
          CALL VCOPY(NGFREE,BVAR(1,GLOLEV,GLOLEV),VGBA)
          CALL VCOPY(NLFREE,BVAR(1,LOCLEV,GLOLEV),VLBA)
          CALL VCOPY(NAFREE,BVAR(1,GARLEV,GLOLEV),VABA)
          CALL VCOPY(NLNFR,BVAR(1,LINLEV,GLOLEV),VLINBA)
          CALL VCOPY(NBLFR,BVAR(1,BLLEV,GLOLEV),VBLBA)
          DO 180 ISUB = 0,NSUBS
            DO 160 ISPEC = 1,NSPEC
              CALL VCOPY(ICAL(1,ISPEC),RESPONSE(1,ISPEC,ISUB,1),
     &                   RESPONSE(1,ISPEC,ISUB,0))
 160        CONTINUE
 180      CONTINUE
 
          RX2BST = RX2NOW
        ENDIF
 
c        ;; Prints best parm from last anneal; not best overall
        IF ( (NEAL.GT.0) .AND. (JPUT.NE.0) ) THEN
          WRITE (JGO,9003) RX2BST,(BVAR(I,GLOLEV,GLOLEV),I=1,NGFREE)
          WRITE (JGO,9004) (BVAR(I,LOCLEV,GLOLEV),I=1,NLFREE)
          WRITE (JGO,9010) (BVAR(I,GARLEV,GLOLEV),I=1,NAFREE)
          WRITE (JGO,9005) (BVAR(I,LINLEV,GLOLEV),I=1,NLNFR)
          WRITE (JGO,9009) (BVAR(I,BLLEV,GLOLEV),I=1,NBLFR)
        ENDIF
 
c        ;; If Monte-Carlo simulations are being done and if fit is
c        ;; not lousy, keep these parameters
        IF ( (NMONTE.NE.0) .AND. ((RX2NOW-RX2MC0).LT.2.0) ) THEN
          MCCNT = MCCNT + 1
          NTOTFR = 0
 
c           ;; Global Nonlinear
          DO 200 J = 1,NGLO
            DO 190 I = 1,IGP(2,J)
              IF ( KONG(I,J).EQ.IJ2KON(I,J) ) THEN
                NTOTFR = NTOTFR + 1
                VFREE(NTOTFR,MCCNT) = BVAR(INDG(I,J),GLOLEV,GLOLEV)
              ENDIF
 190        CONTINUE
 200      CONTINUE
 
c           ;; Local
          DO 240 ISUB = 1,NSUBS
            DO 210 I = 1,ILP(2,ISUB)
              IF ( KONL(I,ISUB).EQ.IJ2KON(I,ISUB) ) THEN
                NTOTFR = NTOTFR + 1
                VFREE(NTOTFR,MCCNT) = BVAR(INDL(I,ISUB),LOCLEV,GLOLEV)
              ENDIF
 210        CONTINUE
            DO 220 I = 1,ILP(3,ISUB)
              IF ( KONLN(I,ISUB).EQ.IJ2KON(I,ISUB) ) THEN
                NTOTFR = NTOTFR + 1
                VFREE(NTOTFR,MCCNT) = BVAR(INDLN(I,ISUB),LINLEV,GLOLEV)
              ENDIF
 220        CONTINUE
 240      CONTINUE
 
c           ;; Spectrum-related
          DO 280 ISPEC = 1,NSPEC
c              ;; Global Areas
            DO 250 I = 1,IAP(2,ISPEC)
              IF ( KONA(I,ISPEC).EQ.IJ2KON(I,ISPEC) ) THEN
                NTOTFR = NTOTFR + 1
                VFREE(NTOTFR,MCCNT) = BVAR(INDA(I,ISPEC),GARLEV,GLOLEV)
              ENDIF
 250        CONTINUE
 
c              ;; Baselines
            DO 260 I = 1,IBLP(2,ISPEC)
              IF ( KONBL(I,ISPEC).EQ.IJ2KON(I,ISPEC) ) THEN
                NTOTFR = NTOTFR + 1
                VFREE(NTOTFR,MCCNT) = BVAR(INDBL(I,ISPEC),BLLEV,GLOLEV)
              ENDIF
 260        CONTINUE
 280      CONTINUE
 
c           ;; Store reduced chi-squared for this monte-carlo iteration
          X2MC(MCCNT) = RX2NOW
        ENDIF
 
        IF ( ABORT.EQ.ABORTALL ) GOTO 400
 
  300 CONTINUE
*---correct value of ineal defined by last increment by DO loop.
      if(ineal.gt.neal) ineal= neal 
*--------------------------------------
*---End of Annealing Loop
*--------------------------------------
 
c     ;; Copy results of annealed fit to "*best" arrays
 400  CALL CNSAPP(NGFREE,VGBA,MGPAR,NGLO,PG0,PGLB,PGUB,PGBEST,INDG,KONG,
     &            WEIG)
      CALL CNSAPP(NLFREE,VLBA,MLPAR,NSUBS,PL0,PLLB,PLUB,PLBEST,INDL,
     &            KONL,WEIL)
      CALL CNSAPP(NAFREE,VABA,MAPAR,NSPEC,PA0,PALB,PAUB,PABEST,INDA,
     &            KONA,WEIA)
      CALL CNSAPP(NLNFR,VLINBA,MLLPAR,NSUBS,PLN0,PLNLB,PLNUB,PLINB,
     &            INDLN,KONLN,WEILN)
      CALL CNSAPP(NBLFR,VBLBA,MBLPAR,NSPEC,PBL0,PBLLB,PBLUB,PBBEST,
     &            INDBL,KONBL,WEIBL)
 
c     ;; Now move these to a permanent place in the "*0" arrays
      DO 500 I = 1,NGLO
        CALL VCOPY(IGP(2,I),PGBEST(1,I),PG0(1,I))
 500  CONTINUE
      DO 600 I = 1,NSPEC
        CALL VCOPY(IAP(2,I),PABEST(1,I),PA0(1,I))
        CALL VCOPY(IBLP(2,I),PBBEST(1,I),PBL0(1,I))
 600  CONTINUE
      DO 700 ISUB = 1,NSUBS
        CALL VCOPY(ILP(2,ISUB),PLBEST(1,ISUB),PL0(1,ISUB))
        CALL VCOPY(ILP(3,ISUB),PLINB(1,ISUB),PLN0(1,ISUB))
 700  CONTINUE
 
c     ;; Perform any post optimization operations
      CALL POSTOP
 
*---Save xi-squared and files fit in current parameters
      CALL CONVRT(RX2BST,CHISTR,.FALSE.)
      WRITE (PARCMT(3),9008) CHISTR
 
c     ;; Write new parameters
      
      if( zcrip ) then
        call errmsg('Free Demo program does not save parameters to'//
     &               ' disk.')
      else
        CALL PWRITE(FILNAM(2),IERR)
      endif
      IF ( IERR.NE.0 ) THEN
        CALL ERRMSG('Error:  Could not save parameters!')
        IERR = 0
      ENDIF
 
c     ;; Write Monte-Carlo disk file
      IF ( NMONTE.NE.0 ) CALL STOMC(MCCNT,MVAR,NTOTFR,VFREE,X2MC,VNAME)
 
*---Restore spectra in fit table to original data
      CALL NUDATA(.FALSE.)
 
*---Show final (annealed) result
      CALL REPORT(GLOLEV,ITERG,RX2BST,RX2BST)
 
*---Print parameters if required...
      IF ( JPUT.NE.0 ) THEN
        IF ( RESULT ) THEN
          CALL PRTPAR(.FALSE.)
          DO 720 J = 1,NSPEC
            DO 710 I = 1,IAP(2,J)
              WRITE (JPRN,9006) IJ2KON(I,J),PANAM(I,J),PA0(I,J),
     &                          KONA(I,J),PALB(I,J),PAUB(I,J)
 710        CONTINUE
 720      CONTINUE
 
          DO 740 J = 1,NSPEC
            DO 730 I = 1,IBLP(2,J)
              WRITE (JPRN,9007) IJ2KON(I,J),PBLNAM(I,J),PBL0(I,J),
     &                          KONBL(I,J)
 730        CONTINUE
 740      CONTINUE
 
          IF ( .NOT.UNIX ) WRITE (JPRN,*) CHAR(12)
          CALL CLSTXT(WASOPN)
        ENDIF
      ENDIF
 
 800  RETURN
 
 9001 FORMAT (1X/'ineal =',I5)
 9002 FORMAT (1X,' this chi^2 = ',F10.3)
 9003 FORMAT (1X,' best chi^2 = ',F10.3/'  global = ',25F10.3)
 9004 FORMAT (1X,' local  = ',25F10.3)
 9005 FORMAT (1X,' linear = ',25F10.3)
 9006 FORMAT (5X,I5,1X,A10,1X,F8.3,1X,I5,1X,F8.3,1X,F8.3)
 9007 FORMAT (5X,I5,1X,A10,1X,F8.3,1X,I5)
 9008 FORMAT ('Reduced Chi squared = ',A8,' for fit to data:')
 9009 FORMAT (1X,' blines = ',25F10.3)
 9010 FORMAT (1X,' g area =',25F10.3)
      END
