      SUBROUTINE qsdplt
*---Given current parameter value for QSD/VBF model (model #6),
*---generate probability distributions for each species dEq and shift.
*---Write to disk file qsd.txt.
*---Plot on screen.
*-----------------------------------------------------------------------
*---WEB Research Co.
*---orig: 19980304 tk
*--- mru: 19980326 tk
*-----------------------------------------------------------------------
      INCLUDE 'limits.inc'
      INCLUDE 'iocom.inc'
      INCLUDE 'constcom.inc'
      INCLUDE 'keycom.inc'
      INCLUDE 'scrncom.inc'
      INCLUDE 'parcom.inc'
      include 'speccom.inc'
      include 'togcom.inc'

*---internals
      logical zbar0, zline0, zpt0, zsmbl0
      integer npts, ipt, isub, i
      parameter (npts = 256)
      REAL dEq(npts),Pdeq(npts,mloc)
      real Ptot(npts)
      real sum
*---storage for plot info
      integer ichr
      real x1, x2
      real step, deq0, ddeq
      integer INOCAL(mcal)
      logical nomask(npts)
      real NOCAL(mcal)
      real NONOYS(npts)
      character*(mfllen) nam(2)
      character*72 cmt(mcommt)
 
*---external functions
      real gaussn
      external gaussn
*-----------------------------------------------------------------------
*---executables
*---store plot switches and set switches for data line plot
*      logical zbar0, zline0, zpt0, zsmbl0
      zbar0 = zbar
      zline0 = zline
      zpt0 = zpoint
      zsmbl0 = zsymbl
      zbar = .false.
      zline = .true.
      zpoint = .false.
      zsymbl = .false.

      if( model .ne. 6 ) then
         call errmsg('Loaded Parameter Set is not a QSD model.')
         return
      endif
*---initialize fake cal info
      
      cmt(1) = 'Quadrupole Splitting Distribution'
      do 5 i=2,mcommt
      cmt(i) = ' '
5     continue

      do 10 i=1,mcal
      inocal(i)=0
      nocal(i) = 0.0
10    continue
      inocal(1) = npts
*---define data type for SHODAT
      inocal(10) = 202

*---Set X-range to include +/- 2*FWHM
        dEq0 = pl0(2,1)
        ddEq = pl0(3,1)
      x1 =  dEq0 - 1.0
      x2 =  dEq0 + 1.0
      do 12 isub=1,nsubs
        dEq0 = pl0(2,isub)
        ddEq = pl0(3,isub)
          if(ddEq.lt. 0.02) then
      call errmsg('Distribution width too small in QSD Plot.')
      return
          endif
        x1 = amin1(x1,dEq0-2.0*ddEq)
        x2 = amax1(x2,dEq0+2.0*ddEq)
12    continue
      step = (X2-X1) /(npts-1)
      nocal(7) = step
      do 20 i=1,npts
         nonoys(i) = 0
         nomask(i) = .true.
         dEq(i) = X1 + (i-1)*step
         Ptot(i) = 0.0
20    continue


*---Loop over local sites
      do 100 isub=1,nsubs
        dEq0 = pl0(2,isub)
        ddEq = pl0(3,isub)
        sum = 0
        do 50 ipt=1,npts
            Pdeq(ipt,isub) = gaussn(dEq(ipt)-dEq0, ddEq)
            sum = sum + Pdeq(ipt,isub)
50      continue
*---normalize Pdeq [per mm/s] for this local site.
*---apply area fractions
*---add to total
        do 60 ipt=1,npts
           Pdeq(ipt,isub) = pln0(1,isub)*Pdeq(ipt,isub)/(sum*step)
           Ptot(ipt) = Ptot(ipt) + Pdeq(ipt,isub)
60      continue

100   continue

*---menu loop
700   continue

      nam(1) = 'QSD'
      nam(2) = ' '
      CALL CLSCRN
*---Prompt
        CALL MESSAGE('~K~eep,  ~P~rint Params,  Print P~l~ot,'//
     &   '  ~Z~oom,  ~O~ptions,  ~ESC~ape.')

*---plot total distribution on screen
       CALL SHODAT(.true.,Npts,dEq,Ptot,NONOYS,INOCAL,NOCAL,
     &   NOMASK,1.0,0,0,NAM,CMT,.FALSE.)
      if(shosub)then
*---plot contribution on each local site
      do 200 isub=1,nsubs
       CALL SHODAT(.false.,Npts,dEq,Pdeq(1,isub),NONOYS,INOCAL,
     &   NOCAL,NOMASK,1.0,0,0,NAM,CMT,.FALSE.)
200   continue
      endif
*---wait for key stroke
      CALL INKEY(ICHR)

      IF ( (ICHR.EQ.ICHAR('K')) .OR. (ICHR.EQ.ICHAR('k')) ) THEN
*---copy currently shown simulation to LIST so that it wiil be available
*---after QSDPLT is exited.
*---increment slot pointers occupied
         if(spctot.eq.mmain)then
            call errmsg('List is full!')
         else
           SPCTOT = SPCTOT + 1
           CALL AD2LST(SPCTOT, NAM,CMT, Ptot, nonoys,
     &      dEq, NOMASK, noCAL, inocal)
         endif
*---copy sub distributioins to list?
        if(stosub)then
          do 250 isub=1,nsubs
           if(spctot.eq.mmain)then
            call errmsg('List is full!')
           else
            SPCTOT = SPCTOT + 1
            nam(1) = 'QSD '//char(48+isub)
            CALL AD2LST(SPCTOT, NAM,CMT, Pdeq(1,isub), nonoys,
     &       dEq, NOMASK, noCAL, inocal)
           endif
250       continue
         endif
      ELSEIF ( (ICHR.EQ.ICHAR('P') .OR. ICHR.EQ.ICHAR('p'))
     &          .and. .not. zcrip) THEN
        CALL GETVEC
        CALL LSTPAR
      elseif ( (ichr.eq.ichar('o')) .or. (ichr.eq.ichar('O'))) then
         call options
      ELSEIF ( (ichr.eq.ichar('l') .or. ichr.eq.ichar('L')
     &          .or. ICHR.EQ.CTRLP) .and. .not. zcrip) THEN
        CALL INTPRN
        call normal
        call garea(0.,0.,1.,1.)
        call gclear
*---plot total distribution on screen
       CALL SHODAT(.true.,Npts,dEq,Ptot,NONOYS,INOCAL,NOCAL,
     &   NOMASK,1.0,0,0,NAM,CMT,.FALSE.)
      if(shosub)then
*---plot contribution on each local site
      do 300 isub=1,nsubs
       CALL SHODAT(.false.,Npts,dEq,Pdeq(1,isub),NONOYS,INOCAL,
     &   NOCAL,NOMASK,1.0,0,0,NAM,CMT,.FALSE.)
300   continue
      endif
        call clsprn
        call garea(0.,0.,1.,1.)
        call gclear

c	PION Refresh SCRN
	ELSEIF ( ICHR.EQ.ALTR ) THEN
      CALL CLSCRN
*---Prompt
        CALL MESSAGE('~K~eep,  ~P~rint Params,  Print P~l~ot,'//
     &   '  ~Z~oom,  ~O~ptions,  ~ESC~ape.')

*---plot total distribution on screen
       CALL SHODAT(.true.,Npts,dEq,Ptot,NONOYS,INOCAL,NOCAL,
     &   NOMASK,1.0,0,0,NAM,CMT,.FALSE.)
      if(shosub)then
*---plot contribution on each local site
      do 350 isub=1,nsubs
       CALL SHODAT(.false.,Npts,dEq,Pdeq(1,isub),NONOYS,INOCAL,
     &   NOCAL,NOMASK,1.0,0,0,NAM,CMT,.FALSE.)
350   continue
      endif
c
      
	elseif (ichr.eq.ichar('z') .or. ichr.eq.ichar('Z')
     &        .or. ichr.eq.F8) then
         if (zscale) then
            call zoom
         else
            zscale = .true.
         endif
      elseif (ichr.eq.esc) then
*---Update screen with new plot of active spectrum and numerical info.
*---restore plot switches
*      logical zbar0, zline0, zpt0, zsmbl0
      zbar = zbar0
      zline = zline0
      zpoint = zpt0
      zsymbl = zsmbl0
*---Update List.
        call sholst
*---Plot active spectrum
        CALL SHOACT(.FALSE.)
*---Then return to topmnu.
        RETURN
      ENDIF

      GOTO 700

 
      END

