      subroutine scldat(itype, npts, x, y, ynoi, mskin, cal, kxplot, 
     &   kyplot, xs, ys, ysnoi, mskout, ntop, toplst)
*-----------------------------------------------------------------------
*--- Contact- OLD: WEB Research Co. /// NEW: WMOSS.ORG
*     orig: 1991 tk,
*     mru : 12Aug93 cs; 19980312 tk
*-----------------------------------------------------------------------
*     Scales the raw data in order to plot it.
*     This version is specifically for WMOSS
*     itype  : [input]  data type
*     npts   : [Input]  number of data points;
*            : [Output] adjusted number of data points;
*     x      : [Input]  x-coordinates of data points;
*     y      : [Input]  y-coordinates of data points;
*     ynoi   : [Input]  noise values of data points;
*     mskin  : [Input]  spectrum mask;
*     cal    : [Input]  real calibration values for data spectrum;
*     kxplot : [Input]  x-scaling type (1--4);
*     kyplot : [Input]  y-scaling type (1--4);
*     xs     : [Output] scaled x-values;
*     ys     : [Output] scaled y-values;
*     ysnoi  : [Output] scaled noise values;
*     mskout : [Output] scaled mask;
*     ntop   : [Output] number of top x-axis ticks;
*     toplst : [Output] top x-axis tick pos. in lower & upper x-coords.
*-----------------------------------------------------------------------
*---externals
      real x(npts), y(npts), ynoi(npts), xs(npts), ys(npts), ysnoi(npts)
      real toplst(2,*), cal(*)
      logical mskin(npts), mskout(npts)
      integer itype, npts, kxplot, kyplot, ntop
*-----------------------------------------------------------------------
      integer i
*-----------------------------------------------------------------------
 
*---Disable a different top scale
      ntop = 0

c     ;; Define output mask
      do 30 i = 1, npts
         mskout(i) = mskin(i)
   30 continue

*---Define xs
      if (kxplot.eq.1) then
*---use Channel numbers
         do 50 i = 1, npts
            xs(i) = i
   50    continue
      else
*----use input x-values
         do 60 i = 1, npts 
            xs(i) = x(i)
   60    continue
      endif

*---define ys and ysnoi; this depends on data type
      if (itype.lt.200 .and. itype.ne.60) then
*---y= area normalized Mossbauer absorption spectrum.      
      if (kyplot.eq.1) then
*---y [10^5 counts]
         do 200 i = 1, npts
            ys(i) = ((y(i) * cal(4)) + cal(3)) / 100000.
            ysnoi(i) = ynoi(i) * cal(4) / 100000.
  200    continue
      else
*---y [% absorption]
         do 100 i = 1, npts
            ys(i) = y(i)*cal(4)/cal(3) * 100.0
            ysnoi(i) = ynoi(i) *cal(4) / cal(3) * 100.0 
  100    continue
      endif
      elseif(itype.eq.60)then
*---file type #6 loaded, no baseline info.
*---do nothing.
         do 240 i = 1, npts
            ys(i) = y(i)
            ysnoi(i) = ynoi(i)
  240    continue

      elseif(itype.ge.201 .and. itype.le.203)then
*---Distribution probability
         do 250 i = 1, npts
            ys(i) = y(i)
            ysnoi(i) = ynoi(i)
  250    continue

      elseif(itype.ge.4000 .and. itype.lt.5000)then
*---y = raw pulse height spectrum
         do 300 i =1,npts
            ys(i) = y(i)
            ysnoi(i) = ynoi(i)
300      continue
      else
         call errmsg('Invalid Data Type in subroutine SCLDAT.')
      endif

      return
      end
