*-----------------------------------------------------------------------
      subroutine shape(nchan, stick, hwhm, hwcal, spec)
*-----------------------------------------------------------------------
      integer nchan
      real hwhm, hwcal, spec(nchan), stick(nchan)
*-----------------------------------------------------------------------
*--- Contact- OLD: WEB Research Co. /// NEW: WMOSS.ORG
*    1992 cs, mru 13Aug93 cs
*
*     Convolutes a given stick spectrum with Lorentzian or Voigt line
*     shapes.  Calculates explicit forward transforms for these shapes.
*     The area of the stick spectrum is preserved.
*
*     nchan  : [Input]  Number of points in stick spectrum;
*     stick  : [Input]  Stick spectrum;
*     hwhm   : [Input]  hwhm > 0 : Lorentzian of width hwhm
*                       hwhm < 0 : Voigt = Gaussian(hwhm)*Lorentz(hwcal);
*     hwcal  : [Input]  Lorentzian width for Voigt line;
*     spec   : [Output] Convolved spectrum.
*-----------------------------------------------------------------------
      include 'limits.inc'
      include 'constcom.inc'
      include 'fftcom.inc'
 
      integer MLEN
      parameter(MLEN = 16 * LSPEC)

      integer i
      real shp(MLEN), spec1(MLEN), stk2(MLEN)
      real expnt, lorexp, lexp1

c     ;; Storage for areas
      real astick, aspec, anorm
*-----------------------------------------------------------------------

c     ;; Make sure that input spectrum is not too large.
      if (2*nchan.gt.MLEN) then
         call errmsg('~INTERNAL ERROR~:  nchan too big in SHAPE')
         goto 1000
      endif

c     ;; Initialize Fast Fourier Transform
      if (lstfft.ne.2*nchan) then
         call rffti(2*nchan, wsave)
         lstfft = 2*nchan
      endif

c     ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; 
c     ;; Calculate shape function ;;
c     ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

c     ;; Scaling for analytic forward transforms
      expnt  = ( PI / nchan / 2) * hwhm
      lorexp = (-PI / nchan) * hwcal
      lexp1  = (-PI / nchan) * hwhm
 
      if (hwhm.gt.0) then
c        ;; Shape = transformed lorentzian
         shp(1) = 1.0
         shp(2) = exp(lexp1)
c        ;; Zero elements are not defined, since they're never used.       
         do 50 i = 2, nchan
            shp(2*i) = exp(lexp1*i)
   50    continue
 
      else
c        ;; Shape = Transformed Voigt
c        ;; Multiply transformed Gaussian and Lorenztian.
         shp(1) = 1.0
         shp(2*nchan) = exp(lorexp * nchan - (expnt * nchan)**2)
         do 100 i = 1, nchan - 1
            shp(2*i) = exp(lorexp * i - (expnt * i)**2)
  100    continue
      endif
 
c     ;; Multiply shape function with stick spectrum (convolution)
      do 200 i = 1, nchan
         stk2(i) = stick(i)
         stk2(i + nchan) = 0.0
  200 continue
      call rfftf(2 * nchan, stk2, wsave)
 
      do 300 i = 1, nchan - 1
c        ;; Simplified since shp(2*i+1) should be 0.0 for i in 1..nchan-1,
c        ;; spec1(2*i)   = stk2(2*i)*shp(2*i) - stk2(2*i+1)*shp(2*i+1)
c        ;; spec1(2*i+1) = stk2(2*i+1)*shp(2*i) + stk2(2*i)*shp(2*i+1)
         spec1(2*i)   = stk2(2*i)   * shp(2*i)
         spec1(2*i+1) = stk2(2*i+1) * shp(2*i)
  300 continue
      spec1(2*nchan) = stk2(2*nchan) * shp(2*nchan)
      spec1(1) = stk2(1) * shp(1)
 
      call rfftb(2*nchan, spec1, wsave)

c     ;; Renormalize area to that of original stick spectrum
      astick = 0.0
      aspec = 0.0
      do 400 i = 1, nchan
         astick = astick + stick(i)
         aspec = aspec + spec1(i)
  400 continue
      if (aspec.ne.0) then
         anorm = abs(astick/aspec)
      else
         anorm = 0.
      endif
      do 500 i = 1,nchan
         spec(i) = anorm * spec1(i)
  500 continue
 
 1000 return
      end
