*-----------------------------------------------------------------------
      subroutine smgrid(mde, nse, esx, esy, esz, bpop, a, v, shift,
     &   fwhm, kpol, nd, nchan, rcal, spec1)
*-----------------------------------------------------------------------
      include 'limits.inc'
      include 'gridcom.inc'

      integer mde, nse, kpol, nd, nchan
      real esx(mde,MAXGRD), esy(mde,MAXGRD), esz(mde,MAXGRD)
      real bpop(mde,MAXGRD), A(3,3), V(3,3), shift, fwhm, rcal(MCAL)
      real spec1(nd)
*-----------------------------------------------------------------------
*--- Contact- OLD: WEB Research Co. /// NEW: WMOSS.ORG
*     910409 tk, mru 28Jul93 cs
*
*     Calculates mossbauer spectrum.
*     Note:  For slow relaxing electronic system, nse = 2S + 1.
*            For fast relaxing electronic system, nse = 1.
*
*     mde   : [Input]  Declared size of storage for electronic states;
*     nse   : [Input]  Number of electronic states for current problem;
*     esx   : [Input]   } Cartesian components of spin expectation
*     esy   : [Input]   } values for the nse states;
*     esz   : [Input]   }
*     bpop  : [Input]  Thermal populations of electronic states;
*     A     : [Input]  Magnetic hyperfine tensor;
*     V     : [Input]  EFG tensor;
*     shift : [Input]  Isomer shift [mm/s];
*     fwhm  : [Input]  Linewidth:  full width at half maximum [mm/s];
*     kpol  : [Input]  Polarization and direction of gamma beam switch;
*     nd    : [Input]  Declared length of spectrum storage;
*     nchan : [Input]  Actual length of spectrum storage;
*     rcal  : [Input]  Calibration values for this spectrum;
*     spec1 : [Output] Generated spectrum.
*---------------------------------------------------------------------------
      include 'parcom.inc'
      include 'constcom.inc'

      integer MMULT, MSTICK
      parameter (MMULT = 4, MSTICK = MMULT * LSPEC)

c     ;; Calibration constants from data file
      real mmpspc, czero, c0, dv
      real enuc(8), xint(8), stik(MSTICK), spec2(MSTICK)
c     ;; Applied field
      real hext
c     ;; Storage for two stick interpolation of fractional channel pos.
      integer klo, khi
      real f
 
      integer i, ig, ist, nchan2
      real hx, hy, hz, bp, sx, sy, sz
      real hefx, hefy, hefz, energy, chwhm, chwcal
      real engtot, area, signal
 
c     ;; Zero the stick spectrum.
      nchan2 = MMULT * nchan
      do 100 i = 1, nchan2
         stik(i) = 0.0
  100 continue

c     ;; Retrieve calibration
      czero = rcal(6)
      mmpspc = rcal(7)
c     ;; Calculate calibration for extended spectrum
      c0 = 1.0 + MMULT * (czero - 1.0)
      dv = mmpspc / MMULT

c     ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
c     ;; Calculate POWDER SIMULATION ;;
c     ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

c     ;; Define externally applied field
      hext = rcal(1)

c     ;; Loop over angular grid
      engtot = 0.
      do 200 ig = 1, ngrid
         hx = ex(ig) * hext
         hy = ey(ig) * hext
         hz = ez(ig) * hext
c        ;; Set pop. to unity in case of fast relaxing (nse=1) case
         bp = 1.0

c        ;; Loop over electronic states
         do 150 ist = 1, nse
c           ;; Define Boltzman population of this state
            if (nse.ne.1) bp = bpop(ist,ig)
c           ;; If ist state is less than 0.2% populated, skip it.
            if (bp.ge.0.002) then
c           ;; Define components of spin expectation values.
               sx = esx(ist,ig)
               sy = esy(ist,ig)
               sz = esz(ist,ig)
c              ;; Define components of effective field.
               hefx = -a(1,1)*sx - a(1,2)*sy - a(1,3)*sz + hx
               hefy = -a(1,2)*sx - a(2,2)*sy - a(2,3)*sz + hy
               hefz = -a(1,3)*sx - a(2,3)*sy - a(3,3)*sz + hz
c              ;; Calc. 57Fe nuclear transition energies and probabilities
               call mo57(v(1,1), v(2,2), v(3,3), v(1,2), v(1,3), v(2,3),
     &            hefx, hefy, hefz, ex(ig), ey(ig), ez(ig), kpol, enuc,
     &            xint)
 
c              ;; Generate the stick spectrum:
c              ;; Represent fractional channel positions as weighted sum
c              ;; of two sticks that bracket energy
               do 110 i = 1, 8
c                 ;; convert from mm/s to channels.
                  energy = (enuc(i)+shift)/dv + c0
                  klo = int(energy)
                  khi = klo + 1
                  f = energy - klo
                  signal = bp * xint(i) * weight(ig)
                  if (klo.ge.1 .and. klo.le.nchan2) then
                     stik(klo) = stik(klo) - (1.0 - f) * signal
                     engtot = engtot - (1.0 - f) * signal
                  endif
                  if (khi.ge.1 .and. khi.le.nchan2) then
                     stik(khi) = stik(khi) - f * signal
                     engtot = engtot - f * signal
                  endif
  110          continue
            endif
  150    continue
  200 continue
 
c     ;; Convert linewidth from FWHM [mm/s] to HWHM [channels]
      chwhm = fwhm / (2. * dv)
c     ;; Lorenztian width used in Voigt shape.
      chwcal = fwhml0 / (2. * dv)
c     ;; Convolute shape function with stick spectrum
      call shape(nchan2, stik, chwhm, chwcal, spec2)

c     ;; Interpolate spectrum back down to nchan channels
      area = 0.
      do 300 i = 1, nchan
         klo = 1 + MMULT * (i-1)
         spec1(i) = spec2(klo)
         area = area + spec1(i)
  300 continue

c     ;; Renormalize spectrum
      if (abs(area).gt.0.) then
         do 400 i = 1, nchan
            spec1(i) = spec1(i) * abs(engtot / area)
  400    continue
      endif

      return
      end
