      SUBROUTINE SOPTMZ(UB,LB,BG,VBEST,NVAR,EVAL,ERROR,ITMAX)
*--- Contact- OLD: WEB Research Co. /// NEW: WMOSS.ORG
*---orig:  Adapted 1990 by TK from Bob Thrift's code(GFWBI/UMN 1981)
*---mru 19May94 CS,  960105 tk
*----------------------------------------------------------
C
C       PURPOSE:
C       OPTIMIZE AN ARBITRARY FUNCTION OF UP TO MVAR VARIABLES
C       BY A DIRECTED SEARCH OF THE PARAMETER SPACE, WITH
C       PROGRESS MEASURED BY COMPARISON OF THE GENERATED
C       FUNCTION WITH AN EXTERNAL STANDARD, SUCH AS AN
C       ARRAY OF EXPERIMENTAL DATA
C
C       INPUT:
C       UB      VECTOR OF UPPER BOUNDS FOR THE VARIABLES
C       LB      VECTOR OF LOWER BOUNDS FOR THE VARIABLES
C       BG      VECTOR OF VARIABLES REPRESENTING USER'S "BEST GUESS"
C       NVAR    NUMBER OF PARAMETERS (VARIABLES) IN THE FUNCTION
C       EVAL    EVALUATION SUBROUTINE WHICH GENERATES THE FUNCTION
C               AND RETURNS AN ESTIMATE OF ERROR
C       ITMAX   MAXIMUM NO. OF ITERS PERMITTED
C               (INSURES TERMINATION IN CASE OF NONCONVERGENCE)
c               (960105 tk: Define the number of iterations to be
c                           equal to the number of calls to EVAL.)
C
C       OUTPUT:
C       VBEST   VECTOR OF BEST VALUES FOUND FOR FUNCTION VARIABLES
C       ERROR   ERROR OF FUNCTION USING PARAMETER VALUES IN VBEST
C
C       METHOD:
C       NELDER-MEAD SIMPLEX SEARCH METHOD
C
C       SUBROUTINES AND FUNCTION SUBPROGRAMS CALLED:
C       EVAL    EVALUATION ROUTINE PROVIDED BY USER AND PASSED
C               BY NAME TO SOPTMZ ROUTINE
C       REPORT  USER SUPPLIED ROUTINE TO REPORT CURRENT STATUS OF FIT.
C
C  FWBI  RLT   11/81     MRU 901029 TK
*----------------------------------------------------------------------
      INCLUDE 'limits.inc'
      INCLUDE 'iocom.inc'
      INCLUDE 'togcom.inc'
      INTEGER BEST,WORST,VERTEX,RESP,PRVWORST
      EXTERNAL EVAL
      INTEGER MVAR1
      PARAMETER (MVAR1=MVAR+1)
      REAL REFL(MVAR1),EXPND(MVAR1),CRCON(MVAR1),CWCON(MVAR1),
     &     SPLEX(MVAR1,MVAR1),CENTROID(MVAR)
      REAL UB(NVAR),LB(NVAR),BG(NVAR),VBEST(NVAR)
 
      INTEGER ITMAX,NVAR,IO,NVERT,ITER
      INTEGER IVAR,NXTWORST,ITEMP,NSPAN,ITB4
      REAL ERROR,FNP,FNEAR,VHI,VLO,SUM,C,W
 
*---STORAGE FOR HISTORICAL ERROR
      REAL ERRB4,FSPAN
*----------------------------------------------------------------------
*---executables

*---951226 tk: Initialize value of ERROR just in case Abort occurs.
      ERROR = 1.0e38
      best = 1
 
*---SET DIAGNOSTIC PRINT LEVEL
      IO = 0
 
*---CHECK NVAR <= MVAR
      IF ( NVAR.GT.MVAR ) THEN
        CALL ERRMSG('Too many variables to optimize!  Aborting.')
        ABORT = ABORTFATAL
        GOTO 999
      ENDIF
C
C       NO. VERTICES IN SIMPLEX IS ONE MORE THAN NO. OF PARAMETERS
C
      NVERT = NVAR + 1
      RESP = NVERT
      FNP = FLOAT(NVAR)
      ITER = 0
 
C IF ITMAX=0, THEN EVALUATE XI-SQUARE OF BEST GUESS AND RETURN
      IF ( ITMAX.LT.1 ) THEN
        DO 50 IVAR = 1,NVAR
          VBEST(IVAR) = BG(IVAR)
 50     CONTINUE
        CALL EVAL(VBEST,NVAR,UB,LB,ERROR)
        iter = iter + 1
        RETURN
      ENDIF
C
C       INITIALIZE THE SIMPLEX
C       ASSIGN BEST GUESS AS VERTEX 1
C       FILL REMAINDER OF SIMPLEX WITH VERTICES SPREAD OVER
C       FSPAN OF PARAMETER SPACE SURROUNDING BEST GUESS
C       (USUALLY, THE INITIAL SIMPLEX SPANS 1/3 OF THE SPACE.)
*_PUT BEST GUESS INTO LAST VERTEX SO THAT ITER=0 REPORT REFLECTS
*_BEST GUESS
      best=nvert
      DO 100 IVAR = 1,NVAR
        SPLEX(IVAR,NVERT) = BG(IVAR)
 100  CONTINUE
      FSPAN = 0.3
      DO 200 VERTEX = 1,NVERT - 1
        DO 150 IVAR = 1,NVAR
          IF ( VERTEX-IVAR.NE.1 ) THEN
            SPLEX(IVAR,VERTEX) = (1.-FSPAN)*BG(IVAR) + FSPAN*LB(IVAR)
          ELSE
            SPLEX(IVAR,VERTEX) = (1.-FSPAN)*BG(IVAR) + FSPAN*UB(IVAR)
          ENDIF
 150    CONTINUE
 200  CONTINUE
C
C       EVALUATE EACH VERTEX OF THE INITIAL SIMPLEX
C
      IF ( IO.GE.10 ) WRITE (JOUT,*) 'in soptmz, initial simplex:'
*---reverse vertex loop so that best guess vertex is evaluated first
*---(in case of abort on first evaluation)
      DO 300 VERTEX = NVERT,1,-1
        DO 250 IVAR = 1,NVAR
          REFL(IVAR) = SPLEX(IVAR,VERTEX)
 250    CONTINUE
        CALL EVAL(REFL,NVAR,UB,LB,ERROR)
        iter = iter + 1
        SPLEX(RESP,VERTEX) = ERROR
        if(abort.ge.abortfit)goto 800
        IF ( IO.GE.10 ) THEN
          WRITE (JOUT,*) ' vertex #',VERTEX
          DO 260 IVAR = 1,NVERT
            WRITE (JOUT,*) SPLEX(IVAR,VERTEX)
 260      CONTINUE
        ENDIF
 300  CONTINUE
C
C 
C       RANK ERROR EVALUATIONS TO FIND BEST AND WORST
C
 400  BEST = 1
      WORST = BEST
      PRVWORST = 0
 
      DO 500 VERTEX = 2,NVERT
        IF ( SPLEX(RESP,VERTEX).LT.SPLEX(RESP,BEST) ) BEST = VERTEX
        IF ( SPLEX(RESP,VERTEX).GT.SPLEX(RESP,WORST) ) WORST = VERTEX
 500  CONTINUE
 
C
C       LOCATE VERTEX WITH NEXT-WORST ERROR
C
      NXTWORST = BEST
      DO 600 VERTEX = 1,NVERT
        IF ( VERTEX.NE.WORST ) THEN
          IF ( SPLEX(RESP,VERTEX).GT.SPLEX(RESP,NXTWORST) )
     &         NXTWORST = VERTEX
        ENDIF
 600  CONTINUE
 
C  SKIP TO EXIT TEST ON FIRST ITER
C
      IF ( ITER.GT.Nvert ) THEN
c960102tk      IF ( ITER.GT.0 ) THEN
C
C       TO AVOID BEING STRANDED ON A RIDGE:
C       IF WORST VERTEX IS ALSO PREVIOUS WORST VERTEX,
C       SWAP WORST AND NEXT-WORST
C
        IF ( WORST.EQ.PRVWORST ) THEN
          ITEMP = WORST
          WORST = NXTWORST
          NXTWORST = ITEMP
       ENDIF

C960105 tk: move this test down to place where iter is incremented.
C  IF ITER COUNT EXCEEDS LIMIT, RETURN
C
c        IF ( ITER.GE.ITMAX ) then
c        call errmsg(' ITER .GE. ITMAX : OPTIMIZATION STOPPED.')
c       GOTO 800
c        endif

      ENDIF
 
*---EXIT TESTS
*---IF BEST ERROR HAS NOT DECREASED BY 1% OVER LAST NSPAN*NVAR ITERATIONS,
*---THEN EXIT.
      NSPAN = 10*NVAR
      IF ( ITER.EQ.Nvert ) THEN
*---1st time through
        ERRB4 = SPLEX(RESP,BEST)
        ITB4 = ITER
      ELSEIF ( SPLEX(RESP,BEST).LT.0.99*ERRB4 ) THEN
        ERRB4 = SPLEX(RESP,BEST)
        ITB4 = ITER
      ELSEIF ( ITER-ITB4.GE.NSPAN ) THEN
*         WRITE(JOUT,*)' ERROR HAS NOT DECREASE 1% OVER LAST',NSPAN
*         WRITE(JOUT,*)' ITERATIONS.  OPTIMIZATION STOPPED.'
        GOTO 800
      ENDIF
C
C       TEST TO DETERMINE IF STOPPING CRITERION IS MET:
C       IF THE SIMPLEX HAS CONTRACTED TO THE POINT AT
C       WHICH THE HIGHEST AND LOWEST VALUES FOR EACH
C       PARAMETER DIFFER BY FNEAR*(UB-LB) OR LESS, EXIT WITH PARAMETER
C       VALUES FOR THE BEST VERTEX
C
      FNEAR = 0.002
      DO 700 IVAR = 1,NVAR
        VHI = SPLEX(IVAR,1)
        VLO = VHI
        DO 650 VERTEX = 2,NVERT
          VHI = AMAX1(VHI,SPLEX(IVAR,VERTEX))
          VLO = AMIN1(VLO,SPLEX(IVAR,VERTEX))
 650    CONTINUE
        IF ( (VHI-VLO).GT.FNEAR*(UB(IVAR)-LB(IVAR)) ) GOTO 1000
 700  CONTINUE
*      WRITE(JOUT,*)' ALL VERTICES WITHIN ',FNEAR,'*(UB-LB)'
*      WRITE(JOUT,*)' OPTIMIZATION STOPPED.'
C
C       EXIT POINT FOR SOPTMZ SUBROUTINE
C
 800  DO 900 IVAR = 1,NVAR
        VBEST(IVAR) = SPLEX(IVAR,BEST)
 900  CONTINUE
      ERROR = SPLEX(RESP,BEST)
 999     RETURN
C
C       COMPUTE CENTROID OF HYPERFACE REPRESENTING ALL
C       THE VERTICES EXCEPT THE WORST
C
 1000 DO 1100 IVAR = 1,NVAR
        SUM = 0.0
        DO 1050 VERTEX = 1,NVERT
          IF ( VERTEX.NE.WORST ) SUM = SUM + SPLEX(IVAR,VERTEX)
 1050   CONTINUE
        CENTROID(IVAR) = SUM/FNP
 1100 CONTINUE
C
C       COMPUTE THE POSSIBLE NEW VERTICES
C
      DO 1200 IVAR = 1,NVAR
        C = CENTROID(IVAR)
        W = SPLEX(IVAR,WORST)
        REFL(IVAR) = 2.0*C - W
        EXPND(IVAR) = 3.0*C - 2.0*W
        CRCON(IVAR) = 1.5*C - 0.5*W
        CWCON(IVAR) = 0.5*(C+W)
 1200 CONTINUE
C
C       EVALUATE ERROR OF REFLECTED VERTEX
C
      CALL EVAL(REFL,NVAR,UB,LB,ERROR)
        iter= iter+1
        if(abort.ge.abortfit)goto 800

      REFL(RESP) = ERROR
      IF ( ABORT.EQ.ABORTOFF ) THEN
C
C       IF ERROR OF REFLECTED VERTEX IS BETTER THAN
C      PREVIOUS BEST, THEN TRY EXPANDING
C
        IF ( ERROR.LT.SPLEX(RESP,BEST) ) THEN
C
C       EVALUATE EXPANDED VERTEX
C
           CALL EVAL(EXPND,NVAR,UB,LB,ERROR)
        iter= iter+1
        if(abort.ge.abortfit)goto 800
          EXPND(RESP) = ERROR
C
C       IF EXPANDED VERTEX IS BETTER THAN REFLECTED, KEEP IT
C
          IF ( ERROR.LE.REFL(RESP) ) THEN
            DO 1210 IVAR = 1,RESP
              SPLEX(IVAR,WORST) = EXPND(IVAR)
 1210       CONTINUE
            IF ( IO.GE.20 ) WRITE (JOUT,*) 'expand , error = ',
     &                             EXPND(RESP)
            GOTO 1400
          ENDIF
C
C       IF ERROR OF REFLECTED VERTEX IS WORSE THAN
C       CURRENT WORST, CONTRACTION IS INDICATED
C
        ELSEIF ( ERROR.GT.SPLEX(RESP,WORST) ) THEN
C
C       CONTRACTION OF SIMPLEX
C       CONTRACT TOWARD REFLECTION VERTEX UNLESS RESULT OF THAT
C       CONTRACTION IS WORSE THAN CURRENT WORST, IN WHICH
C       CASE CONTRACT TOWARD WORST VERTEX
C
           CALL EVAL(CRCON,NVAR,UB,LB,ERROR)
        iter= iter+1
        if(abort.ge.abortfit)goto 800
          CRCON(RESP) = ERROR
          IF ( (ERROR.GT.SPLEX(RESP,WORST)) .AND. ABORT.EQ.ABORTOFF )
     &         THEN
 
             CALL EVAL(CWCON,NVAR,UB,LB,ERROR)
        iter= iter+1
        if(abort.ge.abortfit)goto 800
            CWCON(RESP) = ERROR
            DO 1220 IVAR = 1,RESP
              SPLEX(IVAR,WORST) = CWCON(IVAR)
 1220       CONTINUE
            IF ( IO.GE.20 ) WRITE (JOUT,*)
     &                              'contract worst vertex ,  error = ',
     &                             CWCON(RESP)
          ELSE
            DO 1230 IVAR = 1,RESP
              SPLEX(IVAR,WORST) = CRCON(IVAR)
 1230       CONTINUE
            IF ( IO.GE.20 ) WRITE (JOUT,*)
     &                              'contract refl. vert., error=',
     &                             CRCON(RESP)
          ENDIF
          GOTO 1400
        ENDIF
      ENDIF
C
C       REPLACE WORST VERTEX WITH REFLECTED VERTEX
C
      DO 1300 IVAR = 1,RESP
        SPLEX(IVAR,WORST) = REFL(IVAR)

 1300 CONTINUE
      IF ( IO.GE.20 ) WRITE (JOUT,*) ' reflect, error = ',REFL(RESP)
 
 1400 PRVWORST = WORST
c---Old def. of iter :      ITER = ITER + 1
*---New def. increments iter after each call to EVAL.  This should
*---bring iter in SOPTMZ into synch with ITERL in REPORT.  tk 960115
C
C  IF ITER COUNT EXCEEDS LIMIT, RETURN
C
        IF ( ITER.GT.ITMAX ) then
        call errmsg(' ITER .GE. ITMAX : OPTIMIZATION STOPPED.')
       GOTO 800
        endif

      GOTO 400
      END
