      subroutine ulolc6(isub)
*---Voigt Based Fitting ala' Rancourt: Quadrupole Splitting Distribution
*--- Contact- OLD: WEB Research Co. /// NEW: WMOSS.ORG
*---orig: 19980225 tk
*---mru : 19980317 tk
*Calculates the subspectra for Voigt Based Fitting with a single
*Gaussian distribution over dEq.  Isomer shift may be
*linearly correlated with dEq.  See papers of Denis Rancourt.
*---Ref.: Rancourt, D. G. and Ping, J. Y., Nuclear Instr. Meth. B58
*---      (1991) 85-97.
*---      Note: Rancourt uses an approximation for a Voigt lineshape
*---      (see Martin, P. and Puerta, J., Applied Optics 20 (1981) 3923.)
*---Here, Voigt is calculated explicitly via convolution of Lorenztian
*---and stick spectrum generated by sampling Gaussian distribution of Hint.
*-----------------------------------------------------------------------
      integer isub
*-----------------------------------------------------------------------
*     isub  : Subsite to calculate.
*-----------------------------------------------------------------------
      include 'limits.inc'
      include 'parcom.inc'
      include 'constcom.inc'
      real plbest(MLPAR,MLOC)
      common /rlbest/ plbest

*---internal storage
*---Variables for this model.
      real GammaL, dEq0, GamdEq, delta0, delta1, h2vh1
*---Line positions and relative areas of lines of sextet 
      real P(2), w(2)
*---Distribution value
      real weight


*---GammaL == FWHM of intrinsic Lorentzian
*---dEq == Center value of Gassian distribution over dEq [mm/s]
*---GamdEq == Full Width at half maximum of dEq distribution.
*---delta0 == Center value of Shift .
*---delta1 == slope of Shift vs. Hint line: Shift = delta0 + delta1 * z

*---internals
      real stick(lspec,lt1)
      integer i, npts, ispec, Ndist, N0, idist
      real dEq, ddEq, Shift
      real ch, dch, czero, step, sum
      integer ich

*-----------------------------------------------------------------------
*---constants
*------------------------------------------------------------------
*---parms ala' Rancourt
*---Lorentzian intrinsic linewidth [mm/s]
         gammaL  = plbest(1,isub)
*---Quadrupole splitting distributions parms. [mm/s]
         dEq0  = plbest(2,isub)
         GamdEq = plbest(3,isub)
*---Shift correlation coeffs. [mm/s]
         delta0  = plbest(4,isub)
         delta1  = plbest(5,isub)
*---line 2 to line 3 area ratio
         h2vh1 = plbest(6,isub)

*---Relative transition probabilities for lines in sextet
         w(1) = 1.0
         w(2) = h2vh1

*---zero storage for spectra
       do 22 ispec=1,nspec
       npts = ical(1,ispec)
       do 20 ich=1,lspec
         stick(ich,ispec) = 0.0
20     continue
22     continue

*---step size for internal field distribution [mm/s]
         ddEq = 0.1
*---Loop over distribution in dEq, calculating stick spectrum.
         Ndist = 3.0 * abs(GamdEq) / ddEq
*---Force Ndist to be odd.  This also avoids Ndist = 0.
         if( (Ndist/2)*2 .eq. Ndist) Ndist = Ndist + 1
         N0 = Ndist/2 + 1

*---LOOP over distribution.
         do 100 idist = 1, Ndist
*---QS value at this point in distribution
           dEq = dEq0 + (idist - N0)*ddEq
*---Weight for this point in distribution. Weight negative for absorption.
           if(Ndist.le.1)then
              weight = -1
           else
              weight = -gaussn(dEq-dEq0,GamdEq)
           endif
*---Shift value
           Shift = delta0 + delta1 * dEq 

*---Line positions of doublet
           P(1) = Shift - dEq/2.0
           P(2) = Shift + dEq/2.0

*---loop over each spectrum being fitted.
      do 50 ispec = 1,nspec
         npts = ical(1,ispec)
         czero = cal(6,ispec)
         step = cal(7,ispec)

*---Add sextet for this distribution point to the stick spectrum.
         do 30 i=1,2
           ch = P(i)/step + czero
*---truncate position to nearest lower channel
           ich = ch
*---calc. distance from nearest lower channel
           dch = ch - ich
*---If this line in bounds, add contribution to nearest channels.
           if(ich.ge.1 .and. ich.le.npts)then
              stick(ich,ispec) =
     &              stick(ich,ispec) + weight * w(i) * (1.0 - dch)
           endif
           if(ich+1.ge.1 .and. ich+1 .le. npts)then
              stick(ich+1,ispec) =
     &              stick(ich+1,ispec) + weight * w(i) * dch
           endif
   30    continue

   50 continue

*---end distribution loop
  100 continue

      do 200 ispec=1,nspec
        npts = ical(1,ispec)
        step = cal(7,ispec)

*---Convolute each stick spectrum with intrinsic Lorentzian.
        call shape(npts,stick(1,ispec),0.5*gammal/step,1.0,
     &             locspc(1,ispec,isub) )

*---Normalize area of locspc(1..npts,ispec,isub) to unity.
        sum = 0.0
        do 150 i=1,npts
          sum = sum + locspc(i,ispec,isub)
  150   continue
        sum = abs(sum)
        if(sum.ne.0)then
        do 160 i=1,npts
           locspc(i,ispec,isub) = locspc(i,ispec,isub)/sum
  160   continue
        endif
200   continue

      return
      end
