      subroutine ulolc7(isub)
*---Voigt Based Fitting ala' Rancourt
*--- Contact- OLD: WEB Research Co. /// NEW: WMOSS.ORG
*---orig: 19980216 tk
*---mru : 19980317 tk
*Calculates the subspectra for Voigt Based Fitting with a single
*Gaussian distribution over Hint.  Vzz and/or isomer shift may be
*linearly correlated with Hint.  See papers of Denis Rancourt.
*---Ref.: Rancourt, D. G. and Ping, J. Y., Nuclear Instr. Meth. B58
*---      (1991) 85-97.
*---      Note: Rancourt uses an approximation for a Voigt lineshape
*---      (see Martin, P. and Puerta, J., Applied Optics 20 (1981) 3923.)
*---Here, Voigt is calculated explicitly via convolution of Lorenztian
*---and stick spectrum generated by sampling Gaussian distribution of Hint.
*-----------------------------------------------------------------------
      integer isub
*-----------------------------------------------------------------------
*     isub  : Subsite to calculate.
*-----------------------------------------------------------------------
      include 'limits.inc'
      include 'parcom.inc'
      include 'constcom.inc'
      real plbest(MLPAR,MLOC)
      common /rlbest/ plbest

*---internal storage
*---Variables for this model.
      real GammaL, z0, Gammaz, eps0, eps1, delta0, delta1
      real h2vh3, h1vh3
*---57Fe nuclear energy levels [mm/s]
      real eg1, eg2, ee1, ee2, ee3, ee4
*---Line positions and relative areas of lines of sextet 
      real P(6), w(6)
*---Distribution value
      real weight


*---GammaL == FWHM of intrinsic Lorentzian
*---z0 == Center value of Hint Gassian distribution [mm/s]
*---Gammaz == Full Width at half maximum of Hint distribution.
*---z     == |gn1| * Bn * Hint   [mm/s]
*---eps0 == Center value of EFG component along Hint
*---eps1 == slope of Vhh vs. Hint line:  Vhh = eps0 + eps1 * z
*---delta0 == Center value of Shift .
*---delta1 == slope of Shift vs. Hint line: Shift = delta0 + delta1 * z

*---internals
      real stick(lspec,lt1)
      integer i, npts, ispec, Ndist, N0, idist
      real Bn, gn0, gn1, agn0Bn, agn1Bn, a01
      real Vhh, Shift,  dz , z
      real ch, dch, czero, step, sum
      integer ich

*---Statement functions
      real Gaussn, x, y
*---Gaussian line shape where y is linewidth (FWHM).
      Gaussn(x,y) = exp(-0.6931472*(2.0*x/y)**2)
*-----------------------------------------------------------------------
*---constants
*---speed of light [mm/s]
*      c = 2.99792458 * 10**11
*---Nuclear magneton [eV/kG]
*      Bnev = 3.152452 * 10**(-9)
*---Energy of 57Fe I=3/2 excited state [eV] , Reference ??
*      E0 = 14340.0
*---Nucl. magneton [ (mm/s) / kG ]
*      Bn = Bnev * ( c / E0 )
       Bn = 6.59053*10.**(-2)
*---57Fe ground state nuclear gyromagnetic ratio
*---Lang, 1970    gn0 = 0.1806(14)
*---      gn0 = 0.1806
*---CRC Handbook of Chem. & Phys., 1995, pg 11-45
      gn0 = 0.1812
*---57Fe 14.34 keV excited state nuclear gyromagnetic ratio
*---Lang, 1970   gn1 = -0.1033(8)
      gn1 = -0.1033

      agn0Bn = abs(gn0*Bn)
      agn1Bn = abs(gn1*Bn)
      a01 = abs(gn0/gn1)
*------------------------------------------------------------------
*---parms ala' Rancourt
*---Lorentzian intrinsic linewidth [mm/s]
         gammaL  = plbest(1,isub)
*---parms
*---Hyperfine field distributions parms. [mm/s]
         z0  = plbest(2,isub)
         Gammaz = plbest(3,isub)
*---Shift correlation coeffs. [mm/s]
         delta0  = plbest(4,isub)
         delta1  = plbest(5,isub)
*---Efg correlation coeffs.  [mm/s]
         eps0  = plbest(6,isub)
         eps1  = plbest(7,isub)
*---line 2 to line 3 area ratio
         h2vh3 = plbest(8,isub)
*---line 1 to line 3 area ratio
         h1vh3 = plbest(9,isub)
*---Relative transition probabilities for lines in sextet
         w(1) = h1vh3
         w(2) = h2vh3
         w(3) = 1.0
         w(4) = w(3)
         w(5) = w(2)
         w(6) = w(1)

*---zero storage for spectra
       do 22 ispec=1,nspec
       npts = ical(1,ispec)
       do 20 ich=1,lspec
         stick(ich,ispec) = 0.0
20     continue
22     continue

*---step size for internal field distribution [mm/s]
         dz = 1.0 * agn1Bn
*---Loop over distribution in Hint, calculating stick spectrum.
*---Assume randomly oriented sample with eleltric quadrupole interaction
*---being small perturbation of the magnetic hyperfine interaction.
         Ndist = 3.0 * abs(Gammaz) / dz
*---Force Ndist to be odd.  This also avoids Ndist = 0.
         if( (Ndist/2)*2 .eq. Ndist) Ndist = Ndist + 1
         N0 = Ndist/2 + 1

*---LOOP over distribution.
         do 100 idist = 1, Ndist
*---Internal field value at this point in distribution
           z = z0 + (idist - N0)*dz
*---Weight for this point in distribution. Weight negative for absorption.
           if(Ndist.le.1)then
              weight = -1
           else
              weight = -gaussn(z-z0,Gammaz)
           endif
*---Shift value
           Shift = delta0 + delta1 * z         
*---Vhh value
           Vhh = eps0 + eps1 * z
*---Energies of 57Fe Nuclear States within EFG first order perturbation.
*---I=1/2, M=+1/2
           Eg1 = -0.5 * a01 * z
*---I=1/2, M=-1/2
           Eg2 = +0.5 * a01 * z
*---I=3/2, M= -3/2
           Ee1 = shift -1.5 * z  + Vhh/2
*---I=3/2, M= -1/2
           Ee2 = shift -0.5 * z  - Vhh/2
*---I=3/2, M= +1/2
           Ee3 = shift +0.5 * z  - Vhh/2
*---I=3/2, M= +3/2
           Ee4 = shift +1.5 * z  + Vhh/2

*---Line positions of sextet
*---(1/2,-1/2) --> (3/2, -3/2)
           P(1) = ee1 - eg2
*---(1/2,-1/2) --> (3/2, -1/2)
           P(2) = ee2 - eg2
*---(1/2,-1/2) --> (3/2, +1/2)
           P(3) = ee3 - eg2
*---(1/2,+1/2) --> (3/2, -1/2)
           P(4) = ee2 - eg1
*---(1/2,+1/2) --> (3/2, +1/2)
           P(5) = ee3 - eg1
*---(1/2,+1/2) --> (3/2, +3/2)
           P(6) = ee4 - eg1

*---loop over each spectrum being fitted.
      do 50 ispec = 1,nspec
         npts = ical(1,ispec)
         czero = cal(6,ispec)
         step = cal(7,ispec)

*---Add sextet for this distribution point to the stick spectrum.
         do 30 i=1,6
           ch = P(i)/step + czero
*---truncate position to nearest lower channel
           ich = ch
*---calc. distance from nearest lower channel
           dch = ch - ich
*---If this line in bounds, add contribution to nearest channels.
           if(ich.ge.1 .and. ich.le.npts)then
              stick(ich,ispec) =
     &              stick(ich,ispec) + weight * w(i) * (1.0 - dch)
           endif
           if(ich+1.ge.1 .and. ich+1 .le. npts)then
              stick(ich+1,ispec) =
     &              stick(ich+1,ispec) + weight * w(i) * dch
           endif
   30    continue

   50 continue

*---end distribution loop
  100 continue

      do 200 ispec=1,nspec
        npts = ical(1,ispec)
        step = cal(7,ispec)

*---Convolute each stick spectrum with intrinsic Lorentzian.
        call shape(npts,stick(1,ispec),0.5*gammal/step,1.0,
     &             locspc(1,ispec,isub) )

*---Normalize area of locspc(1..npts,ispec,isub) to unity.
        sum = 0.0
        do 150 i=1,npts
          sum = sum + locspc(i,ispec,isub)
  150   continue
        sum = abs(sum)
        do 160 i=1,npts
           locspc(i,ispec,isub) = locspc(i,ispec,isub)/sum
  160   continue
200   continue

      return
      end
