      SUBROUTINE X4HTRIBK(AR,AI,TAU,ZR,ZI)
C
      REAL AR(4,4),AI(4,4),TAU(2,4),ZR(4,4),ZI(4,4)
      REAL H,S,SI
C
C     THIS SUBROUTINE IS A TRANSLATION OF A COMPLEX ANALOGUE OF
C     THE ALGOL PROCEDURE TRBAK1, NUM. MATH. 11, 181-195(1968)
C     BY MARTIN, REINSCH, AND WILKINSON.
C     HANDBOOK FOR AUTO. COMP., VOL.II-LINEAR ALGEBRA, 212-226(1971).
C
C     THIS SUBROUTINE FORMS THE EIGENVECTORS OF A COMPLEX HERMITIAN
C     MATRIX BY BACK TRANSFORMING THOSE OF THE CORRESPONDING
C     REAL SYMMETRIC TRIDIAGONAL MATRIX DETERMINED BY  HTRIDI.
C
C     ON INPUT
C
C        NM MUST BE SET TO THE ROW DIMENSION OF TWO-DIMENSIONAL
C          ARRAY PARAMETERS AS DECLARED IN THE CALLING PROGRAM
C          DIMENSION STATEMENT.
C
C        N IS THE ORDER OF THE MATRIX.
C
C        AR AND AI CONTAIN INFORMATION ABOUT THE UNITARY TRANS-
C          FORMATIONS USED IN THE REDUCTION BY  HTRIDI  IN THEIR
C          FULL LOWER TRIANGLES EXCEPT FOR THE DIAGONAL OF AR.
C
C        TAU CONTAINS FURTHER INFORMATION ABOUT THE TRANSFORMATIONS.
C
C        M IS THE NUMBER OF EIGENVECTORS TO BE BACK TRANSFORMED.
C
C        ZR CONTAINS THE EIGENVECTORS TO BE BACK TRANSFORMED
C          IN ITS FIRST M COLUMNS.
C
C     ON OUTPUT
C
C        ZR AND ZI CONTAIN THE REAL AND IMAGINARY PARTS,
C          RESPECTIVELY, OF THE TRANSFORMED EIGENVECTORS
C          IN THEIR FIRST M COLUMNS.
C
C     NOTE THAT THE LAST COMPONENT OF EACH RETURNED VECTOR
C     IS REAL AND THAT VECTOR EUCLIDEAN NORMS ARE PRESERVED.
C
C     QUESTIONS AND COMMENTS SHOULD BE DIRECTED TO BURTON S. GARBOW,
C     MATHEMATICS AND COMPUTER SCIENCE DIV, ARGONNE NATIONAL LABORATORY
C
C     THIS VERSION DATED AUGUST 1983.
C
C     ------------------------------------------------------------------
C
C     .......... TRANSFORM THE EIGENVECTORS OF THE REAL SYMMETRIC
C                TRIDIAGONAL MATRIX TO THOSE OF THE HERMITIAN
C                TRIDIAGONAL MATRIX. ..........
 
*---Do 50 Loop
*---K = 1 [50]
      ZI(1,1) = -ZR(1,1)*TAU(2,1)
      ZR(1,1) = ZR(1,1)*TAU(1,1)
      ZI(1,2) = -ZR(1,2)*TAU(2,1)
      ZR(1,2) = ZR(1,2)*TAU(1,1)
      ZI(1,3) = -ZR(1,3)*TAU(2,1)
      ZR(1,3) = ZR(1,3)*TAU(1,1)
      ZI(1,4) = -ZR(1,4)*TAU(2,1)
      ZR(1,4) = ZR(1,4)*TAU(1,1)
*---K = 2 [50]
      ZI(2,1) = -ZR(2,1)*TAU(2,2)
      ZR(2,1) = ZR(2,1)*TAU(1,2)
      ZI(2,2) = -ZR(2,2)*TAU(2,2)
      ZR(2,2) = ZR(2,2)*TAU(1,2)
      ZI(2,3) = -ZR(2,3)*TAU(2,2)
      ZR(2,3) = ZR(2,3)*TAU(1,2)
      ZI(2,4) = -ZR(2,4)*TAU(2,2)
      ZR(2,4) = ZR(2,4)*TAU(1,2)
*---K = 3 [50]
      ZI(3,1) = -ZR(3,1)*TAU(2,3)
      ZR(3,1) = ZR(3,1)*TAU(1,3)
      ZI(3,2) = -ZR(3,2)*TAU(2,3)
      ZR(3,2) = ZR(3,2)*TAU(1,3)
      ZI(3,3) = -ZR(3,3)*TAU(2,3)
      ZR(3,3) = ZR(3,3)*TAU(1,3)
      ZI(3,4) = -ZR(3,4)*TAU(2,3)
      ZR(3,4) = ZR(3,4)*TAU(1,3)
*---K = 4 [50]
**      ZI(4,1) = -ZR(4,1)*TAU(2,4)
      ZI(4,1) = 0.
**      ZR(4,1) = ZR(4,1)*TAU(1,4)
**      ZI(4,2) = -ZR(4,2)*TAU(2,4)
      ZI(4,2) = 0.
**      ZR(4,2) = ZR(4,2)*TAU(1,4)
**      ZI(4,3) = -ZR(4,3)*TAU(2,4)
      ZI(4,3) = 0.
**      ZR(4,3) = ZR(4,3)*TAU(1,4)
**      ZI(4,4) = -ZR(4,4)*TAU(2,4)
      ZI(4,4) = 0.
**      ZR(4,4) = ZR(4,4)*TAU(1,4)
*---End 50 Loop
C
C     .......... RECOVER AND APPLY THE HOUSEHOLDER MATRICES ..........
 
*---Do 60 Loop
*---I = 2
*---L = 1
      H = AI(2,2)
      IF ( H.NE.0.0E0 ) THEN
C
*---Do 55 Loop
*---J = 1 [55]
 
*----Do 52 Loop; K = 1
        S = AR(2,1)*ZR(1,1) - AI(2,1)*ZI(1,1)
        SI = AR(2,1)*ZI(1,1) + AI(2,1)*ZR(1,1)
*----End 52 Loop
C     .......... DOUBLE DIVISIONS AVOID POSSIBLE UNDERFLOW ..........
        S = (S/H)/H
        SI = (SI/H)/H
C
*----Do 54 Loop ; K = 1
        ZR(1,1) = ZR(1,1) - S*AR(2,1) - SI*AI(2,1)
        ZI(1,1) = ZI(1,1) - SI*AR(2,1) + S*AI(2,1)
*----End 54 Loop
 
*---J = 2 [55]
 
*----Do 52 Loop; K = 1
        S = AR(2,1)*ZR(1,2) - AI(2,1)*ZI(1,2)
        SI = AR(2,1)*ZI(1,2) + AI(2,1)*ZR(1,2)
*----End 52 Loop
 
C     .......... DOUBLE DIVISIONS AVOID POSSIBLE UNDERFLOW ..........
        S = (S/H)/H
        SI = (SI/H)/H
C
*----Do 54 Loop; K = 1
        ZR(1,2) = ZR(1,2) - S*AR(2,1) - SI*AI(2,1)
        ZI(1,2) = ZI(1,2) - SI*AR(2,1) + S*AI(2,1)
*----End 54 Loop
 
 
*---J = 3 [55]
 
*----Do 52 Loop; K = 1
        S = AR(2,1)*ZR(1,3) - AI(2,1)*ZI(1,3)
        SI = AR(2,1)*ZI(1,3) + AI(2,1)*ZR(1,3)
*----End 52 Loop
C     .......... DOUBLE DIVISIONS AVOID POSSIBLE UNDERFLOW ..........
        S = (S/H)/H
        SI = (SI/H)/H
C
*----Do 54 Loop; K = 1
        ZR(1,3) = ZR(1,3) - S*AR(2,1) - SI*AI(2,1)
        ZI(1,3) = ZI(1,3) - SI*AR(2,1) + S*AI(2,1)
*----End 54 Loop
 
*---J = 4 [55]
 
*----Do 52 Loop; K = 1
        S = AR(2,1)*ZR(1,4) - AI(2,1)*ZI(1,4)
        SI = AR(2,1)*ZI(1,4) + AI(2,1)*ZR(1,4)
*----End 52 Loop
C     .......... DOUBLE DIVISIONS AVOID POSSIBLE UNDERFLOW ..........
        S = (S/H)/H
        SI = (SI/H)/H
C
*----Do 54 Loop; K = 1
        ZR(1,4) = ZR(1,4) - S*AR(2,1) - SI*AI(2,1)
        ZI(1,4) = ZI(1,4) - SI*AR(2,1) + S*AI(2,1)
*----End 54 Loop
      ENDIF
 
 
*---I = 3 [60]
*---L = 2 [60]
      H = AI(3,3)
      IF ( H.NE.0.0E0 ) THEN
 
*---Do 55 Loop
*---J = 1 [55]
 
*----Do 52 Loop
*----K = 1 [52]
        S = AR(3,1)*ZR(1,1) - AI(3,1)*ZI(1,1)
        SI = AR(3,1)*ZI(1,1) + AI(3,1)*ZR(1,1)
*----K = 2 [52]
        S = S + AR(3,2)*ZR(2,1) - AI(3,2)*ZI(2,1)
        SI = SI + AR(3,2)*ZI(2,1) + AI(3,2)*ZR(2,1)
*----End 52 Loop
C     .......... DOUBLE DIVISIONS AVOID POSSIBLE UNDERFLOW ..........
        S = (S/H)/H
        SI = (SI/H)/H
 
*----Do 54 Loop
*----K = 1 [54]
        ZR(1,1) = ZR(1,1) - S*AR(3,1) - SI*AI(3,1)
        ZI(1,1) = ZI(1,1) - SI*AR(3,1) + S*AI(3,1)
*----K = 2 [54]
        ZR(2,1) = ZR(2,1) - S*AR(3,2) - SI*AI(3,2)
        ZI(2,1) = ZI(2,1) - SI*AR(3,2) + S*AI(3,2)
*----End 54 Loop
 
*---J = 2 [55]
 
*----Do 52 Loop
*----K = 1 [52]
        S = AR(3,1)*ZR(1,2) - AI(3,1)*ZI(1,2)
        SI = AR(3,1)*ZI(1,2) + AI(3,1)*ZR(1,2)
*----K = 2 [52]
        S = S + AR(3,2)*ZR(2,2) - AI(3,2)*ZI(2,2)
        SI = SI + AR(3,2)*ZI(2,2) + AI(3,2)*ZR(2,2)
*----End 52 Loop
C     .......... DOUBLE DIVISIONS AVOID POSSIBLE UNDERFLOW ..........
        S = (S/H)/H
        SI = (SI/H)/H
 
*----Do 54 Loop
*----K = 1 [54]
        ZR(1,2) = ZR(1,2) - S*AR(3,1) - SI*AI(3,1)
        ZI(1,2) = ZI(1,2) - SI*AR(3,1) + S*AI(3,1)
*----K = 2 [54]
        ZR(2,2) = ZR(2,2) - S*AR(3,2) - SI*AI(3,2)
        ZI(2,2) = ZI(2,2) - SI*AR(3,2) + S*AI(3,2)
*----End 54 Loop
 
*---J = 3 [55]
 
*----Do 52 Loop
*----K = 1 [52]
        S = AR(3,1)*ZR(1,3) - AI(3,1)*ZI(1,3)
        SI = AR(3,1)*ZI(1,3) + AI(3,1)*ZR(1,3)
*----K = 2 [52]
        S = S + AR(3,2)*ZR(2,3) - AI(3,2)*ZI(2,3)
        SI = SI + AR(3,2)*ZI(2,3) + AI(3,2)*ZR(2,3)
*----End 52 Loop
C     .......... DOUBLE DIVISIONS AVOID POSSIBLE UNDERFLOW ..........
        S = (S/H)/H
        SI = (SI/H)/H
 
*----Do 54 Loop
*----K = 1 [54]
        ZR(1,3) = ZR(1,3) - S*AR(3,1) - SI*AI(3,1)
        ZI(1,3) = ZI(1,3) - SI*AR(3,1) + S*AI(3,1)
*----K = 2 [54]
        ZR(2,3) = ZR(2,3) - S*AR(3,2) - SI*AI(3,2)
        ZI(2,3) = ZI(2,3) - SI*AR(3,2) + S*AI(3,2)
*----End 54 Loop
 
*---J = 4 [55]
 
*----Do 52 Loop
*----K = 1 [52]
        S = AR(3,1)*ZR(1,4) - AI(3,1)*ZI(1,4)
        SI = AR(3,1)*ZI(1,4) + AI(3,1)*ZR(1,4)
*----K = 2 [52]
        S = S + AR(3,2)*ZR(2,4) - AI(3,2)*ZI(2,4)
        SI = SI + AR(3,2)*ZI(2,4) + AI(3,2)*ZR(2,4)
*----End 52 Loop
C     .......... DOUBLE DIVISIONS AVOID POSSIBLE UNDERFLOW ..........
        S = (S/H)/H
        SI = (SI/H)/H
 
*----Do 54 Loop
*----K = 1 [54]
        ZR(1,4) = ZR(1,4) - S*AR(3,1) - SI*AI(3,1)
        ZI(1,4) = ZI(1,4) - SI*AR(3,1) + S*AI(3,1)
*----K = 2 [54]
        ZR(2,4) = ZR(2,4) - S*AR(3,2) - SI*AI(3,2)
        ZI(2,4) = ZI(2,4) - SI*AR(3,2) + S*AI(3,2)
*----End 54 Loop
 
*---End 55 Loop
      ENDIF
 
*---I = 4 [60]
*---L = 3 [60]
      H = AI(4,4)
      IF ( H.NE.0.0E0 ) THEN
*---Do 55 Loop
*---J = 1 [55]
 
*----Do 52 Loop
*----K = 1
**        S = AR(4,1)*ZR(1,1) - AI(4,1)*ZI(1,1)
**        SI = AR(4,1)*ZI(1,1) + AI(4,1)*ZR(1,1)
*----K = 2
**        S = S + AR(4,2)*ZR(2,1) - AI(4,2)*ZI(2,1)
        S = AR(4,2)*ZR(2,1) - AI(4,2)*ZI(2,1)
**        SI = SI + AR(4,2)*ZI(2,1) + AI(4,2)*ZR(2,1)
        SI = AR(4,2)*ZI(2,1) + AI(4,2)*ZR(2,1)
*----K = 3
        S = S + AR(4,3)*ZR(3,1) - AI(4,3)*ZI(3,1)
        SI = SI + AR(4,3)*ZI(3,1) + AI(4,3)*ZR(3,1)
*----End 52 Loop
 
C     .......... DOUBLE DIVISIONS AVOID POSSIBLE UNDERFLOW ..........
        S = (S/H)/H
        SI = (SI/H)/H
 
*----Do 54 Loop
*----K = 1 [54]
**        ZR(1,1) = ZR(1,1) - S*AR(4,1) - SI*AI(4,1)
**        ZI(1,1) = ZI(1,1) - SI*AR(4,1) + S*AI(4,1)
*----K = 2 [54]
        ZR(2,1) = ZR(2,1) - S*AR(4,2) - SI*AI(4,2)
        ZI(2,1) = ZI(2,1) - SI*AR(4,2) + S*AI(4,2)
*----K = 3 [54]
        ZR(3,1) = ZR(3,1) - S*AR(4,3) - SI*AI(4,3)
        ZI(3,1) = ZI(3,1) - SI*AR(4,3) + S*AI(4,3)
*----End 54 Loop
 
*---J = 2 [55]
 
*----Do 52 Loop
*----K = 1
**        S = AR(4,1)*ZR(1,2) - AI(4,1)*ZI(1,2)
**        SI = AR(4,1)*ZI(1,2) + AI(4,1)*ZR(1,2)
*----K = 2
**        S = S + AR(4,2)*ZR(2,2) - AI(4,2)*ZI(2,2)
        S = AR(4,2)*ZR(2,2) - AI(4,2)*ZI(2,2)
**        SI = SI + AR(4,2)*ZI(2,2) + AI(4,2)*ZR(2,2)
        SI = AR(4,2)*ZI(2,2) + AI(4,2)*ZR(2,2)
*----K = 3
        S = S + AR(4,3)*ZR(3,2) - AI(4,3)*ZI(3,2)
        SI = SI + AR(4,3)*ZI(3,2) + AI(4,3)*ZR(3,2)
*----End 52 Loop
 
C     .......... DOUBLE DIVISIONS AVOID POSSIBLE UNDERFLOW ..........
        S = (S/H)/H
        SI = (SI/H)/H
 
*----Do 54 Loop
*----K = 1 [54]
**        ZR(1,2) = ZR(1,2) - S*AR(4,1) - SI*AI(4,1)
**        ZI(1,2) = ZI(1,2) - SI*AR(4,1) + S*AI(4,1)
*----K = 2 [54]
        ZR(2,2) = ZR(2,2) - S*AR(4,2) - SI*AI(4,2)
        ZI(2,2) = ZI(2,2) - SI*AR(4,2) + S*AI(4,2)
*----K = 3 [54]
        ZR(3,2) = ZR(3,2) - S*AR(4,3) - SI*AI(4,3)
        ZI(3,2) = ZI(3,2) - SI*AR(4,3) + S*AI(4,3)
*----End 54 Loop
 
*---J = 3 [55]
 
*----Do 52 Loop
*----K = 1
**        S = AR(4,1)*ZR(1,3) - AI(4,1)*ZI(1,3)
**        SI = AR(4,1)*ZI(1,3) + AI(4,1)*ZR(1,3)
*----K = 2
**        S = S + AR(4,2)*ZR(2,3) - AI(4,2)*ZI(2,3)
        S = AR(4,2)*ZR(2,3) - AI(4,2)*ZI(2,3)
**        SI = SI + AR(4,2)*ZI(2,3) + AI(4,2)*ZR(2,3)
        SI = AR(4,2)*ZI(2,3) + AI(4,2)*ZR(2,3)
*----K = 3
        S = S + AR(4,3)*ZR(3,3) - AI(4,3)*ZI(3,3)
        SI = SI + AR(4,3)*ZI(3,3) + AI(4,3)*ZR(3,3)
*----End 52 Loop
 
C     .......... DOUBLE DIVISIONS AVOID POSSIBLE UNDERFLOW ..........
        S = (S/H)/H
        SI = (SI/H)/H
 
*----Do 54 Loop
*----K = 1 [54]
**        ZR(1,3) = ZR(1,3) - S*AR(4,1) - SI*AI(4,1)
**        ZI(1,3) = ZI(1,3) - SI*AR(4,1) + S*AI(4,1)
*----K = 2 [54]
        ZR(2,3) = ZR(2,3) - S*AR(4,2) - SI*AI(4,2)
        ZI(2,3) = ZI(2,3) - SI*AR(4,2) + S*AI(4,2)
*----K = 3 [54]
        ZR(3,3) = ZR(3,3) - S*AR(4,3) - SI*AI(4,3)
        ZI(3,3) = ZI(3,3) - SI*AR(4,3) + S*AI(4,3)
*----End 54 Loop
 
*---J = 4 [55]
 
*----Do 52 Loop
*----K = 1
**        S = AR(4,1)*ZR(1,4) - AI(4,1)*ZI(1,4)
**        SI = AR(4,1)*ZI(1,4) + AI(4,1)*ZR(1,4)
*----K = 2
**        S = S + AR(4,2)*ZR(2,4) - AI(4,2)*ZI(2,4)
        S = AR(4,2)*ZR(2,4) - AI(4,2)*ZI(2,4)
**        SI = SI + AR(4,2)*ZI(2,4) + AI(4,2)*ZR(2,4)
        SI = AR(4,2)*ZI(2,4) + AI(4,2)*ZR(2,4)
*----K = 3
        S = S + AR(4,3)*ZR(3,4) - AI(4,3)*ZI(3,4)
        SI = SI + AR(4,3)*ZI(3,4) + AI(4,3)*ZR(3,4)
*----End 52 Loop
 
C     .......... DOUBLE DIVISIONS AVOID POSSIBLE UNDERFLOW ..........
        S = (S/H)/H
        SI = (SI/H)/H
 
*----Do 54 Loop
*----K = 1 [54]
**        ZR(1,4) = ZR(1,4) - S*AR(4,1) - SI*AI(4,1)
**        ZI(1,4) = ZI(1,4) - SI*AR(4,1) + S*AI(4,1)
*----K = 2 [54]
        ZR(2,4) = ZR(2,4) - S*AR(4,2) - SI*AI(4,2)
        ZI(2,4) = ZI(2,4) - SI*AR(4,2) + S*AI(4,2)
*----K = 3 [54]
        ZR(3,4) = ZR(3,4) - S*AR(4,3) - SI*AI(4,3)
        ZI(3,4) = ZI(3,4) - SI*AR(4,3) + S*AI(4,3)
*----End 54 Loop
*---End 55 Loop
 
      ENDIF
 
*---End 60 Loop
 
      RETURN
      END
