      SUBROUTINE X4HTRIDI(AR,AI,D,E,E2,TAU)
C
      REAL AR(4,4),AI(4,4),D(4),E(4),E2(4),TAU(2,4)
      REAL F,G,H,FI,GI,HH,SI,SCALE,PYTHAG
C
C     THIS SUBROUTINE IS A TRANSLATION OF A COMPLEX ANALOGUE OF
C     THE ALGOL PROCEDURE TRED1, NUM. MATH. 11, 181-195(1968)
C     BY MARTIN, REINSCH, AND WILKINSON.
C     HANDBOOK FOR AUTO. COMP., VOL.II-LINEAR ALGEBRA, 212-226(1971).
C
C     THIS SUBROUTINE REDUCES A COMPLEX HERMITIAN MATRIX
C     TO A REAL SYMMETRIC TRIDIAGONAL MATRIX USING
C     UNITARY SIMILARITY TRANSFORMATIONS.
C
C     ON INPUT
C
C        NM MUST BE SET TO THE ROW DIMENSION OF TWO-DIMENSIONAL
C          ARRAY PARAMETERS AS DECLARED IN THE CALLING PROGRAM
C          DIMENSION STATEMENT.
C
C        N IS THE ORDER OF THE MATRIX.
C
C        AR AND AI CONTAIN THE REAL AND IMAGINARY PARTS,
C          RESPECTIVELY, OF THE COMPLEX HERMITIAN INPUT MATRIX.
C          ONLY THE LOWER TRIANGLE OF THE MATRIX NEED BE SUPPLIED.
C
C     ON OUTPUT
C
C        AR AND AI CONTAIN INFORMATION ABOUT THE UNITARY TRANS-
C          FORMATIONS USED IN THE REDUCTION IN THEIR FULL LOWER
C          TRIANGLES.  THEIR STRICT UPPER TRIANGLES AND THE
C          DIAGONAL OF AR ARE UNALTERED.
C
C        D CONTAINS THE DIAGONAL ELEMENTS OF THE THE TRIDIAGONAL MATRIX.
C
C        E CONTAINS THE SUBDIAGONAL ELEMENTS OF THE TRIDIAGONAL
C          MATRIX IN ITS LAST N-1 POSITIONS.  E(1) IS SET TO ZERO.
C
C        E2 CONTAINS THE SQUARES OF THE CORRESPONDING ELEMENTS OF E.
C          E2 MAY COINCIDE WITH E IF THE SQUARES ARE NOT NEEDED.
C
C        TAU CONTAINS FURTHER INFORMATION ABOUT THE TRANSFORMATIONS.
C
C     CALLS PYTHAG FOR  SQRT(A*A + B*B) .
C
C     QUESTIONS AND COMMENTS SHOULD BE DIRECTED TO BURTON S. GARBOW,
C     MATHEMATICS AND COMPUTER SCIENCE DIV, ARGONNE NATIONAL LABORATORY
C
C     THIS VERSION DATED AUGUST 1983.
C
C     ------------------------------------------------------------------
C
*_CS 17Jun91 patch: modified pythag routine

**CS 10Jun93 patch: Assumes AR(4,1) = AI(4,1) = 0.0,
**                   takes advantage of the fact that AI(i,i)=0 for i=1,4, 
**                   plus other propogations.

      real x, y
      PYTHAG(X,Y) = SQRT(X**2+Y**2)
 
      TAU(1,4) = 1.0E0
      TAU(2,4) = 0.0E0
C
*---Do 100 Loop
      D(1) = AR(1,1)
      D(2) = AR(2,2)
      D(3) = AR(3,3)
      D(4) = AR(4,4)
*---End 100 Loop
 
*---Do 500 Loop
*---I = 4 [500]
*---L = 3 [500]
      H = 0.0E0
C     .......... SCALE ROW (ALGOL TOL THEN NOT NEEDED) ..........
**      SCALE = ABS(AR(4,1)) + ABS(AI(4,1)) + ABS(AR(4,2)) + ABS(AI(4,2))
**     &        + ABS(AR(4,3)) + ABS(AI(4,3))
      SCALE = ABS(AR(4,2)) + ABS(AI(4,2)) + ABS(AR(4,3)) + ABS(AI(4,3))
 
C
      IF ( SCALE.NE.0.0E0 ) THEN
C
*---Do 130 Loop
**        AR(4,1) = AR(4,1)/SCALE
**        AI(4,1) = AI(4,1)/SCALE
        AR(4,2) = AR(4,2)/SCALE
        AI(4,2) = AI(4,2)/SCALE
        AR(4,3) = AR(4,3)/SCALE
        AI(4,3) = AI(4,3)/SCALE
**        H = AR(4,1)*AR(4,1) + AI(4,1)*AI(4,1) + AR(4,2)*AR(4,2)
**     &      + AI(4,2)*AI(4,2) + AR(4,3)*AR(4,3) + AI(4,3)*AI(4,3)
        H = AR(4,2)*AR(4,2) + AI(4,2)*AI(4,2) + AR(4,3)*AR(4,3) +
     &     AI(4,3)*AI(4,3)
*---End 130 Loop
C
        E2(4) = SCALE*SCALE*H
        G = SQRT(H)
        E(4) = SCALE*G
        F = PYTHAG(AR(4,3),AI(4,3))
C     .......... FORM NEXT DIAGONAL ELEMENT OF MATRIX T ..........
        IF ( F.EQ.0.0E0 ) THEN
**          TAU(1,3) = -TAU(1,4)
          TAU(1,3) = -1.0e0
**          SI = TAU(2,4)
          SI = 0.0e0
          AR(4,3) = G
        ELSE
**          TAU(1,3) = (AI(4,3)*TAU(2,4)-AR(4,3)*TAU(1,4))/F
          TAU(1,3) = -AR(4,3)/F
**          SI = (AR(4,3)*TAU(2,4)+AI(4,3)*TAU(1,4))/F
          SI = AI(4,3)/F
          H = H + F*G
          G = 1.0E0 + G/F
          AR(4,3) = G*AR(4,3)
          AI(4,3) = G*AI(4,3)
        ENDIF
**        F = 0.0E0
 
*---DO 200 Loop
*---J = 1 [200]
C     .......... FORM ELEMENT OF A*U ..........
**        G = AR(1,1)*AR(4,1) + AI(1,1)*AI(4,1) + AR(2,1)*AR(4,2)
**     &      - AI(2,1)*AI(4,2) + AR(3,1)*AR(4,3) - AI(3,1)*AI(4,3)
        G = AR(2,1)*AR(4,2) - AI(2,1)*AI(4,2) + AR(3,1)*AR(4,3) -
     &     AI(3,1)*AI(4,3)
**        GI = -AR(1,1)*AI(4,1) + AI(1,1)*AR(4,1) - AR(2,1)*AI(4,2)
**     &       - AI(2,1)*AR(4,2) - AR(3,1)*AI(4,3) - AI(3,1)*AR(4,3)
        GI = - AR(2,1)*AI(4,2) - AI(2,1)*AR(4,2) - AR(3,1)*AI(4,3) -
     &     AI(3,1)*AR(4,3)
 
C     .......... FORM ELEMENT OF P ..........
        E(1) = G/H
        TAU(2,1) = GI/H
**        F = F + E(1)*AR(4,1) - TAU(2,1)*AI(4,1)
 
*---J = 2 [200]
C     .......... FORM ELEMENT OF A*U ..........
**        G = AR(2,1)*AR(4,1) + AI(2,1)*AI(4,1) + AR(2,2)*AR(4,2)
**     &      + AI(2,2)*AI(4,2) + AR(3,2)*AR(4,3) - AI(3,2)*AI(4,3)
        G = AR(2,2)*AR(4,2) + AR(3,2)*AR(4,3) - AI(3,2)*AI(4,3)
**        GI = -AR(2,1)*AI(4,1) + AI(2,1)*AR(4,1) - AR(2,2)*AI(4,2)
**     &       + AI(2,2)*AR(4,2) - AR(3,2)*AI(4,3) - AI(3,2)*AR(4,3)
        GI = - AR(2,2)*AI(4,2) - AR(3,2)*AI(4,3) - AI(3,2)*AR(4,3)
 
C     .......... FORM ELEMENT OF P ..........
        E(2) = G/H
        TAU(2,2) = GI/H
**        F = F + E(2)*AR(4,2) - TAU(2,2)*AI(4,2)
        F = E(2)*AR(4,2) - TAU(2,2)*AI(4,2)
 
*---J = 3 [200]
C     .......... FORM ELEMENT OF A*U ..........
**        G = AR(3,1)*AR(4,1) + AI(3,1)*AI(4,1) + AR(3,2)*AR(4,2)
**     &      + AI(3,2)*AI(4,2) + AR(3,3)*AR(4,3) + AI(3,3)*AI(4,3)
        G = AR(3,2)*AR(4,2) + AI(3,2)*AI(4,2) + AR(3,3)*AR(4,3)
**        GI = -AR(3,1)*AI(4,1) + AI(3,1)*AR(4,1) - AR(3,2)*AI(4,2)
**     &       + AI(3,2)*AR(4,2) - AR(3,3)*AI(4,3) + AI(3,3)*AR(4,3)
        GI = - AR(3,2)*AI(4,2) + AI(3,2)*AR(4,2) - AR(3,3)*AI(4,3)
C
C     .......... FORM ELEMENT OF P ..........
        E(3) = G/H
        TAU(2,3) = GI/H
        F = F + E(3)*AR(4,3) - TAU(2,3)*AI(4,3)
*---End 250 Loop
 
        HH = F/(H+H)
C     .......... FORM REDUCED A ..........
 
*---Do 250 Loop
*---J = 1 [250]
**        F = AR(4,1)
**        G = E(1) - HH*F
**        E(1) = G
**        FI = -AI(4,1)
**        GI = TAU(2,1) - HH*FI
**        TAU(2,1) = -GI
        TAU(2,1) = -tau(2,1)
 
*---DO 220 Loop
*---K = 1 [220]
**        AR(1,1) = AR(1,1) - F*E(1) - G*AR(4,1) + FI*TAU(2,1)
**     &            + GI*AI(4,1)
**        AI(1,1) = AI(1,1) - F*TAU(2,1) - G*AI(4,1) - FI*E(1)
**     &            - GI*AR(4,1)
*---End 220
 
*---J = 2 [250]
        F = AR(4,2)
        G = E(2) - HH*F
        E(2) = G
        FI = -AI(4,2)
        GI = TAU(2,2) - HH*FI
        TAU(2,2) = -GI
 
*---Do 220 Loop
*---K = 1 [220]
**        AR(2,1) = AR(2,1) - F*E(1) - G*AR(4,1) + FI*TAU(2,1)
**     &            + GI*AI(4,1)
        AR(2,1) = AR(2,1) - F*E(1) + FI*TAU(2,1)
**        AI(2,1) = AI(2,1) - F*TAU(2,1) - G*AI(4,1) - FI*E(1)
**     &            - GI*AR(4,1)
        AI(2,1) = AI(2,1) - F*TAU(2,1) - FI*E(1)
*---K = 2 [220]
**        AR(2,2) = AR(2,2) - F*E(2) - G*AR(4,2) + FI*TAU(2,2)
**     &            + GI*AI(4,2)
        AR(2,2) = AR(2,2) + 2*(GI*AI(4,2) - G*AR(4,2))
**        AI(2,2) = AI(2,2) - F*TAU(2,2) - G*AI(4,2) - FI*E(2)
**     &            - GI*AR(4,2)
*---End 220 loop
 
*---J = 3 [250]
        F = AR(4,3)
        G = E(3) - HH*F
        E(3) = G
        FI = -AI(4,3)
        GI = TAU(2,3) - HH*FI
        TAU(2,3) = -GI
 
*---Do 220 Loop
*---K = 1 [220]
**        AR(3,1) = AR(3,1) - F*E(1) - G*AR(4,1) + FI*TAU(2,1)
**     &            + GI*AI(4,1)
        AR(3,1) = AR(3,1) - F*E(1) - G*AR(4,1) + FI*TAU(2,1)
**        AI(3,1) = AI(3,1) - F*TAU(2,1) - G*AI(4,1) - FI*E(1)
**     &            - GI*AR(4,1)
        AI(3,1) = AI(3,1) - F*TAU(2,1) - G*AI(4,1) - FI*E(1)
*---K = 2 [220]
        AR(3,2) = AR(3,2) - F*E(2) - G*AR(4,2) + FI*TAU(2,2)
     &            + GI*AI(4,2)
        AI(3,2) = AI(3,2) - F*TAU(2,2) - G*AI(4,2) - FI*E(2)
     &            - GI*AR(4,2)
*---K = 3 [220]
*        AR(3,3) = AR(3,3) - F*E(3) - G*AR(4,3) + FI*TAU(2,3)
*     &            + GI*AI(4,3)
        AR(3,3) = AR(3,3) + 2 * (GI*AI(4,3) - G*AR(4,3))
*        AI(3,3) = AI(3,3) - F*TAU(2,3) - G*AI(4,3) - FI*E(3)
*     &            - GI*AR(4,3)
*---End 220 loop
*---End 250 loop
 
*---Do 350 Loop
*---K = 1 [350]
**        AR(4,1) = SCALE*AR(4,1)
**        AI(4,1) = SCALE*AI(4,1)
*---K = 2 [350]
        AR(4,2) = SCALE*AR(4,2)
        AI(4,2) = SCALE*AI(4,2)
*---K = 3 [350]
        AR(4,3) = SCALE*AR(4,3)
        AI(4,3) = SCALE*AI(4,3)
*---End 350 Loop
C
        TAU(2,3) = -SI
      ELSE
        TAU(1,3) = 1.0E0
        TAU(2,3) = 0.0E0
        E(4) = 0.0E0
        E2(4) = 0.0E0
      ENDIF
**    HH = D(4)
**    D(4) = AR(4,4)
**    AR(4,4) = HH
      AI(4,4) = SCALE*SQRT(H)
 
*---I = 3 [500]
*---L = 2 [500]
C     .......... SCALE ROW (ALGOL TOL THEN NOT NEEDED) ..........
      H = 0.0E0
*---Do 120 Loop
      SCALE = ABS(AR(3,1)) + ABS(AI(3,1)) + ABS(AR(3,2)) + ABS(AI(3,2))
*---End 120 Loop
 
      IF ( SCALE.NE.0.0E0 ) THEN
*---Do 130 Loop
*---K = 1 [130]
        AR(3,1) = AR(3,1)/SCALE
        AI(3,1) = AI(3,1)/SCALE
*---K = 2 [130]
        AR(3,2) = AR(3,2)/SCALE
        AI(3,2) = AI(3,2)/SCALE
*---Combined K=1,2 [130]
        H = AR(3,1)*AR(3,1) + AI(3,1)*AI(3,1) + AR(3,2)*AR(3,2)
     &      + AI(3,2)*AI(3,2)
*---End 130 Loop
C
        E2(3) = SCALE*SCALE*H
        G = SQRT(H)
        E(3) = SCALE*G
        F = PYTHAG(AR(3,2),AI(3,2))
C     .......... FORM NEXT DIAGONAL ELEMENT OF MATRIX T ..........
        IF ( F.EQ.0.0E0 ) THEN
          TAU(1,2) = -TAU(1,3)
          SI = TAU(2,3)
          AR(3,2) = G
        ELSE
          TAU(1,2) = (AI(3,2)*TAU(2,3)-AR(3,2)*TAU(1,3))/F
          SI = (AR(3,2)*TAU(2,3)+AI(3,2)*TAU(1,3))/F
          H = H + F*G
          G = 1.0E0 + G/F
          AR(3,2) = G*AR(3,2)
          AI(3,2) = G*AI(3,2)
        ENDIF
**        F = 0.0E0
 
*---Do 200 Loop
*---J = 1 [200]
 
*---Do 160,170 Loops
*---K = 1,2 [160,170]
C     .......... FORM ELEMENT OF A*U ..........
**        G = AR(1,1)*AR(3,1) + AI(1,1)*AI(3,1) + AR(2,1)*AR(3,2)
**     &      - AI(2,1)*AI(3,2)
        G = AR(1,1)*AR(3,1) + AR(2,1)*AR(3,2) - AI(2,1)*AI(3,2)
**        GI = -AR(1,1)*AI(3,1) + AI(1,1)*AR(3,1) - AR(2,1)*AI(3,2)
**     &       - AI(2,1)*AR(3,2)
        GI = -AR(1,1)*AI(3,1) - AR(2,1)*AI(3,2) - AI(2,1)*AR(3,2)
*---End 160,170 Loops
 
C     .......... FORM ELEMENT OF P ..........
        E(1) = G/H
        TAU(2,1) = GI/H
**        F = F + E(1)*AR(3,1) - TAU(2,1)*AI(3,1)
        F = E(1)*AR(3,1) - TAU(2,1)*AI(3,1)
 
*---J = 2 [200]
C     .......... FORM ELEMENT OF A*U ..........
*---Do 160 Loop
*---K = 1,2 [160]
**        G = AR(2,1)*AR(3,1) + AI(2,1)*AI(3,1) + AR(2,2)*AR(3,2)
**     &      + AI(2,2)*AI(3,2)
        G = AR(2,1)*AR(3,1) + AI(2,1)*AI(3,1) + AR(2,2)*AR(3,2)
**        GI = -AR(2,1)*AI(3,1) + AI(2,1)*AR(3,1) - AR(2,2)*AI(3,2)
**     &       + AI(2,2)*AR(3,2)
        GI = -AR(2,1)*AI(3,1) + AI(2,1)*AR(3,1) - AR(2,2)*AI(3,2)
*---End 160 Loop
 
C     .......... FORM ELEMENT OF P ..........
        E(2) = G/H
        TAU(2,2) = GI/H
        F = F + E(2)*AR(3,2) - TAU(2,2)*AI(3,2)
*---End 200 Loop
 
        HH = F/(H+H)
C     .......... FORM REDUCED A ..........
*---Do 250 Loop
*---J = 1 [250]
**        F = AR(3,1)
**        G = E(1) - HH*F
        G = E(1) - HH*AR(3,1)
        E(1) = G
**        FI = -AI(3,1)
**        GI = TAU(2,1) - HH*FI
        GI = TAU(2,1) + HH * AI(3,1)
        TAU(2,1) = -GI
 
*---Do 220 Loop
*---K = 1 [220]
**        AR(1,1) = AR(1,1) - F*E(1) - G*AR(3,1) + FI*TAU(2,1)
**     &            + GI*AI(3,1)
        AR(1,1) = AR(1,1) + 2 * (GI*AI(3,1) - G*AR(3,1))
**        AI(1,1) = AI(1,1) - F*TAU(2,1) - G*AI(3,1) - FI*E(1)
**     &            - GI*AR(3,1)
*---End 220 Loop
 
*---J = 2 [250]
        F = AR(3,2)
        G = E(2) - HH*F
        E(2) = G
        FI = -AI(3,2)
        GI = TAU(2,2) - HH*FI
        TAU(2,2) = -GI
C
*---Do 220 Loop
*---K = 1
        AR(2,1) = AR(2,1) - F*E(1) - G*AR(3,1) + FI*TAU(2,1)
     &            + GI*AI(3,1)
        AI(2,1) = AI(2,1) - F*TAU(2,1) - G*AI(3,1) - FI*E(1)
     &            - GI*AR(3,1)
*---K = 2 [220]
**        AR(2,2) = AR(2,2) - F*E(2) - G*AR(3,2) + FI*TAU(2,2)
**     &            + GI*AI(3,2)
        AR(2,2) = AR(2,2) + 2 * (GI*AI(3,2) - G*AR(3,2))
**        AI(2,2) = AI(2,2) - F*TAU(2,2) - G*AI(3,2) - FI*E(2)
**     &            - GI*AR(3,2)
*---End 220 Loop
*---End 250 Loop
 
*---Do 350 Loop
*---K = 1 [350]
        AR(3,1) = SCALE*AR(3,1)
        AI(3,1) = SCALE*AI(3,1)
*---K = 2 [350]
        AR(3,2) = SCALE*AR(3,2)
        AI(3,2) = SCALE*AI(3,2)
*---End 350 Loop
 
        TAU(2,2) = -SI
      ELSE
        TAU(1,2) = 1.0E0
        TAU(2,2) = 0.0E0
        E(3) = 0.0E0
        E2(3) = 0.0E0
      ENDIF
      HH = D(3)
      D(3) = AR(3,3)
      AR(3,3) = HH
      AI(3,3) = SCALE*SQRT(H)
 
*---I = 2 [500]
*---L = 1 [500]
      H = 0.0E0
*---Do 120 Loop
*---K = 1 [120]
C     .......... SCALE ROW (ALGOL TOL THEN NOT NEEDED) ..........
      SCALE = ABS(AR(2,1)) + ABS(AI(2,1))
*---End 120 Loop
 
      IF ( SCALE.NE.0.0E0 ) THEN
 
*---Do 130 Loop
*---K = 1 [130]
        AR(2,1) = AR(2,1)/SCALE
        AI(2,1) = AI(2,1)/SCALE
        H = AR(2,1)*AR(2,1) + AI(2,1)*AI(2,1)
*---End 130 Loop
 
        E2(2) = SCALE*SCALE*H
        G = SQRT(H)
        E(2) = SCALE*G
**        F = PYTHAG(AR(2,1),AI(2,1))
        F = G
C     .......... FORM NEXT DIAGONAL ELEMENT OF MATRIX T ..........
** note: since at this point SCALE.ne.0, we know that F cannot be 0.
**
**        IF ( F.EQ.0.0E0 ) THEN
**          TAU(1,1) = -TAU(1,2)
**          SI = TAU(2,2)
**          AR(2,1) = G
**          F = 0.0E0
** 
***---Do 200 Loop
***---J = 1 [200]
** 
***---Do 160 Loop
***---K = 1 [160]
**C     .......... FORM ELEMENT OF A*U ..........
**          G = AR(1,1)*AR(2,1) + AI(1,1)*AI(2,1)
**          GI = -AR(1,1)*AI(2,1) + AI(1,1)*AR(2,1)
***---End 160 Loop
** 
**C     .......... FORM ELEMENT OF P ..........
**          E(1) = G/H
**          TAU(2,1) = GI/H
**          F = F + E(1)*AR(2,1) - TAU(2,1)*AI(2,1)
***---End 200 Loop
**
**          HH = F/(H+H)
**C     .......... FORM REDUCED A ..........
***---Do 250 Loop
***---J = 1 [250]
**          F = AR(2,1)
**          G = E(1) - HH*F
**          E(1) = G
**          FI = -AI(2,1)
**          GI = TAU(2,1) - HH*FI
**          TAU(2,1) = -GI
***---Do 220 Loop
***---K = 1 [220]
**          AR(1,1) = AR(1,1) - F*E(1) - G*AR(2,1) + FI*TAU(2,1)
**     &              + GI*AI(2,1)
**          AI(1,1) = AI(1,1) - F*TAU(2,1) - G*AI(2,1) - FI*E(1)
**     &              - GI*AR(2,1)
**        ELSE
          TAU(1,1) = (AI(2,1)*TAU(2,2)-AR(2,1)*TAU(1,2))/F
          SI = (AR(2,1)*TAU(2,2)+AI(2,1)*TAU(1,2))/F
**          H = H + F*G
          H = 2 * H
**          G = 1.0E0 + G/F     (=2.0)
**          AR(2,1) = G*AR(2,1)
          AR(2,1) = 2*AR(2,1)
**          AI(2,1) = G*AI(2,1)
          AI(2,1) = 2*AI(2,1)
**        ENDIF
*---End 220 Loop
*---End 250 Loop
 
*---Do 350 Loop
*---K = 1 [350]
        AR(2,1) = SCALE*AR(2,1)
        AI(2,1) = SCALE*AI(2,1)
*---End 350 Loop
 
        TAU(2,1) = -SI
      ELSE
        TAU(1,1) = 1.0E0
        TAU(2,1) = 0.0E0
        E(2) = 0.0E0
        E2(2) = 0.0E0
      ENDIF
      HH = D(2)
      D(2) = AR(2,2)
      AR(2,2) = HH
      AI(2,2) = SCALE*SQRT(H)
 
*---I = 1 [500]
*---L = 0 [500]
**      H = 0.0E0
**      SCALE = 0.0E0
      E(1) = 0.0E0
      E2(1) = 0.0E0
*---Jump to line 400
      HH = D(1)
      D(1) = AR(1,1)
      AR(1,1) = HH
**      AI(1,1) = SCALE*SQRT(H)
      AI(1,1) = 0.0e0
*---End 500 Loop
 
      RETURN
      END
