      SUBROUTINE X4TQL2(D,E,Z,IERR)
C
      INTEGER I,J,M,IERR
      REAL D(4),E(4),Z(4,4)
      REAL C,C2,C3,DL1,EL1,F,G,H,P,R,S,S2,TST1,TST2,PYTHAG
C
C     THIS SUBROUTINE IS A TRANSLATION OF THE ALGOL PROCEDURE TQL2,
C     NUM. MATH. 11, 293-306(1968) BY BOWDLER, MARTIN, REINSCH, AND
C     WILKINSON.
C     HANDBOOK FOR AUTO. COMP., VOL.II-LINEAR ALGEBRA, 227-240(1971).
C
C     THIS SUBROUTINE FINDS THE EIGENVALUES AND EIGENVECTORS
C     OF A SYMMETRIC TRIDIAGONAL MATRIX BY THE QL METHOD.
C     THE EIGENVECTORS OF A FULL SYMMETRIC MATRIX CAN ALSO
C     BE FOUND IF  TRED2  HAS BEEN USED TO REDUCE THIS
C     FULL MATRIX TO TRIDIAGONAL FORM.
C
C     ON INPUT
C
C        NM MUST BE SET TO THE ROW DIMENSION OF TWO-DIMENSIONAL
C          ARRAY PARAMETERS AS DECLARED IN THE CALLING PROGRAM
C          DIMENSION STATEMENT.
C
C        N IS THE ORDER OF THE MATRIX.
C
C        D CONTAINS THE DIAGONAL ELEMENTS OF THE INPUT MATRIX.
C
C        E CONTAINS THE SUBDIAGONAL ELEMENTS OF THE INPUT MATRIX
C          IN ITS LAST N-1 POSITIONS.  E(1) IS ARBITRARY.
C
C        Z CONTAINS THE TRANSFORMATION MATRIX PRODUCED IN THE
C          REDUCTION BY  TRED2, IF PERFORMED.  IF THE EIGENVECTORS
C          OF THE TRIDIAGONAL MATRIX ARE DESIRED, Z MUST CONTAIN
C          THE IDENTITY MATRIX.
C
C      ON OUTPUT
C
C        D CONTAINS THE EIGENVALUES IN ASCENDING ORDER.  IF AN
C          ERROR EXIT IS MADE, THE EIGENVALUES ARE CORRECT BUT
C          UNORDERED FOR INDICES 1,2,...,IERR-1.
C
C        E HAS BEEN DESTROYED.
C
C        Z CONTAINS ORTHONORMAL EIGENVECTORS OF THE SYMMETRIC
C          TRIDIAGONAL (OR FULL) MATRIX.  IF AN ERROR EXIT IS MADE,
C          Z CONTAINS THE EIGENVECTORS ASSOCIATED WITH THE STORED
C          EIGENVALUES.
C
C        IERR IS SET TO
C          ZERO       FOR NORMAL RETURN,
C          J          IF THE J-TH EIGENVALUE HAS NOT BEEN
C                     DETERMINED AFTER 30 ITERATIONS.
C
C     CALLS PYTHAG FOR  SQRT(A*A + B*B) .
C
C     QUESTIONS AND COMMENTS SHOULD BE DIRECTED TO BURTON S. GARBOW,
C     MATHEMATICS AND COMPUTER SCIENCE DIV, ARGONNE NATIONAL LABORATORY
C
C     THIS VERSION DATED AUGUST 1983.
C
C     ------------------------------------------------------------------
C
*_CS 23Jun91
* Modified for 4 by 4 matrices only;
*   The former variables L,L1,L2,II were replaced by constants
*   Several loops were unrolled
*   Some tests were removed, since the outcome was determined by the
*     iteration.
 
*_CS 17Jun91 patch: modified pythag routine
      real x, y
      PYTHAG(X,Y) = SQRT(X**2+Y**2)
 
      IERR = 0
C
      E(1) = E(2)
      E(2) = E(3)
      E(3) = E(4)
      E(4) = 0.0E0
C
      F = 0.0E0
      TST1 = 0.0E0
      E(4) = 0.0E0
C
*---First iteration of the DO 150 loop; L=1
      J = 0
      H = ABS(D(1)) + ABS(E(1))
      IF ( TST1.LT.H ) TST1 = H
C     .......... LOOK FOR SMALL SUB-DIAGONAL ELEMENT ..........
      TST2 = TST1 + ABS(E(1))
*_CS 08Aug91
* The following unused labels inhibit optimization, forcing the Salford
* compiler to store tst2 in memory, then bring it back to keep it
* single precision
      IF ( TST2.EQ.TST1 ) THEN
        M = 1
      ELSE
        TST2 = TST1 + ABS(E(2))
        IF ( TST2.EQ.TST1 ) THEN
          M = 2
        ELSE
          TST2 = TST1 + ABS(E(3))
          IF ( TST2.EQ.TST1 ) THEN
            M = 3
          ELSE
            M = 4
          ENDIF
        ENDIF
      ENDIF
      IF ( M.EQ.1 ) THEN
        D(1) = D(1) + F
      ELSE
 50     IF ( J.EQ.30 ) THEN
C     .......... SET ERROR -- NO CONVERGENCE TO AN
C                EIGENVALUE AFTER 30 ITERATIONS ..........
          IERR = 17
          GOTO 300
        ELSE
          J = J + 1
C     .......... FORM SHIFT ..........
          G = D(1)
          P = (D(2)-G)/(2.0E0*E(1))
          R = PYTHAG(P,1.0E0)
          D(1) = E(1)/(P+SIGN(R,P))
          D(2) = E(1)*(P+SIGN(R,P))
          DL1 = D(2)
          H = G - D(1)
C
          D(3) = D(3) - H
          D(4) = D(4) - H
C
          F = F + H
C     .......... QL TRANSFORMATION ..........
          P = D(M)
          C = 1.0E0
          C2 = C
          EL1 = E(2)
          S = 0.0E0
C     .......... FOR I=M-1 STEP -1 UNTIL L DO -- ..........
          DO 60 I = M - 1,1, - 1
            C3 = C2
            C2 = C
            S2 = S
            G = C*E(I)
            H = C*P
            R = PYTHAG(P,E(I))
            E(I+1) = S*R
            S = E(I)/R
            C = P/R
            P = C*D(I) - S*G
            D(I+1) = H + S*(C*G+S*D(I))
C     .......... FORM VECTOR ..........
            H = Z(1,I+1)
            Z(1,I+1) = S*Z(1,I) + C*H
            Z(1,I) = C*Z(1,I) - S*H
 
            H = Z(2,I+1)
            Z(2,I+1) = S*Z(2,I) + C*H
            Z(2,I) = C*Z(2,I) - S*H
 
            H = Z(3,I+1)
            Z(3,I+1) = S*Z(3,I) + C*H
            Z(3,I) = C*Z(3,I) - S*H
 
            H = Z(4,I+1)
            Z(4,I+1) = S*Z(4,I) + C*H
            Z(4,I) = C*Z(4,I) - S*H
C
 60       CONTINUE
C
          P = -S*S2*C3*EL1*E(1)/DL1
          E(1) = S*P
          D(1) = C*P
          TST2 = TST1 + ABS(E(1))
*_CS 08Aug91
* This unused label inhibits optimization, and forces the Salford
* compiler to store tst2 in memory, then bring it back to keep it
* single precision
          IF ( TST2.GT.TST1 ) GOTO 50
          D(1) = D(1) + F
        ENDIF
      ENDIF
 
*---L = 2
      J = 0
      H = ABS(D(2)) + ABS(E(2))
      IF ( TST1.LT.H ) TST1 = H
      TST2 = TST1 + ABS(E(2))
*_CS 08Aug91
* The following unused labels inhibit optimization, forcing the Salford
* compiler to store tst2 in memory, then bring it back to keep it
* single precision
      IF ( TST2.EQ.TST1 ) THEN
        M = 2
      ELSE
        TST2 = TST1 + ABS(E(3))
        IF ( TST2.EQ.TST1 ) THEN
          M = 3
        ELSE
          M = 4
        ENDIF
      ENDIF
      IF ( M.EQ.2 ) THEN
        D(2) = D(2) + F
      ELSE
 100    IF ( J.EQ.30 ) THEN
          IERR = 17
          GOTO 300
        ELSE
          J = J + 1
C     .......... FORM SHIFT ..........
          G = D(2)
          P = (D(3)-G)/(2.0E0*E(2))
          R = PYTHAG(P,1.0E0)
          D(2) = E(2)/(P+SIGN(R,P))
          D(3) = E(2)*(P+SIGN(R,P))
          DL1 = D(3)
          H = G - D(2)
          D(4) = D(4) - H
C
          F = F + H
C     .......... QL TRANSFORMATION ..........
          P = D(M)
          C = 1.0E0
          C2 = C
          EL1 = E(3)
          S = 0.0E0
C     .......... FOR I=M-1 STEP -1 UNTIL L DO -- ..........
          DO 120 I = M - 1,2, - 1
            C3 = C2
            C2 = C
            S2 = S
            G = C*E(I)
            H = C*P
            R = PYTHAG(P,E(I))
            E(I+1) = S*R
            S = E(I)/R
            C = P/R
            P = C*D(I) - S*G
            D(I+1) = H + S*(C*G+S*D(I))
C     .......... FORM VECTOR ..........
            H = Z(1,I+1)
            Z(1,I+1) = S*Z(1,I) + C*H
            Z(1,I) = C*Z(1,I) - S*H
 
            H = Z(2,I+1)
            Z(2,I+1) = S*Z(2,I) + C*H
            Z(2,I) = C*Z(2,I) - S*H
 
            H = Z(3,I+1)
            Z(3,I+1) = S*Z(3,I) + C*H
            Z(3,I) = C*Z(3,I) - S*H
 
            H = Z(4,I+1)
            Z(4,I+1) = S*Z(4,I) + C*H
            Z(4,I) = C*Z(4,I) - S*H
C
 120      CONTINUE
C
          P = -S*S2*C3*EL1*E(2)/DL1
          E(2) = S*P
          D(2) = C*P
          TST2 = TST1 + ABS(E(2))
*_CS 08Aug91
* This unused label inhibits optimization, and forces the Salford
* compiler to store tst2 in memory, then bring it back to keep it
* single precision
          IF ( TST2.GT.TST1 ) GOTO 100
          D(2) = D(2) + F
        ENDIF
      ENDIF
 
*---L = 3
      J = 0
      H = ABS(D(3)) + ABS(E(3))
      IF ( TST1.LT.H ) TST1 = H
C     .......... LOOK FOR SMALL SUB-DIAGONAL ELEMENT ..........
      TST2 = TST1 + ABS(E(3))
*_CS 08Aug91
* This unused label inhibits optimization, and forces the Salford
* compiler to store tst2 in memory, then bring it back to keep it
* single precision
      IF ( TST2.EQ.TST1 ) THEN
        M = 3
        GOTO 200
      ENDIF
      M = 4
C
 200  IF ( M.EQ.3 ) THEN
        D(3) = D(3) + F
      ELSE
 250    IF ( J.EQ.30 ) THEN
          IERR = 17
          GOTO 300
        ELSE
          J = J + 1
C     .......... FORM SHIFT ..........
          G = D(3)
          P = (D(4)-G)/(2.0E0*E(3))
          R = PYTHAG(P,1.0E0)
          D(3) = E(3)/(P+SIGN(R,P))
          D(4) = E(3)*(P+SIGN(R,P))
          DL1 = D(4)
          H = G - D(3)
C
          F = F + H
C     .......... QL TRANSFORMATION ..........
          P = D(M)
          C = 1.0E0
          C2 = C
          EL1 = E(4)
          S = 0.0E0
C     .......... FOR I=M-1 STEP -1 UNTIL L DO -- ..........
          IF ( M.EQ.4 ) THEN
            C3 = C2
            C2 = C
            S2 = S
            G = C*E(3)
            H = C*P
            R = PYTHAG(P,E(3))
            E(4) = S*R
            S = E(3)/R
            C = P/R
            P = C*D(3) - S*G
            D(4) = H + S*(C*G+S*D(3))
C     .......... FORM VECTOR ..........
            H = Z(1,4)
            Z(1,4) = S*Z(1,3) + C*H
            Z(1,3) = C*Z(1,3) - S*H
 
            H = Z(2,4)
            Z(2,4) = S*Z(2,3) + C*H
            Z(2,3) = C*Z(2,3) - S*H
 
            H = Z(3,4)
            Z(3,4) = S*Z(3,3) + C*H
            Z(3,3) = C*Z(3,3) - S*H
 
            H = Z(4,4)
            Z(4,4) = S*Z(4,3) + C*H
            Z(4,3) = C*Z(4,3) - S*H
          ENDIF
C
          P = -S*S2*C3*EL1*E(3)/DL1
          E(3) = S*P
          D(3) = C*P
          TST2 = TST1 + ABS(E(3))
*_CS 08Aug91
* This unused label inhibits optimization, and forces the Salford
* compiler to store tst2 in memory, then bring it back to keep it
* single precision
          IF ( TST2.GT.TST1 ) GOTO 250
          D(3) = D(3) + F
        ENDIF
      ENDIF
*---L = 4
      J = 0
      H = ABS(D(4)) + ABS(E(4))
      IF ( TST1.LT.H ) TST1 = H
C     .......... LOOK FOR SMALL SUB-DIAGONAL ELEMENT ..........
 
      D(4) = D(4) + F
 300  RETURN
      END
