      SUBROUTINE XHTRIDI(NM,N,AR,AI,D,E,E2,TAU)
C
      INTEGER I,J,K,L,N,II,NM,JP1
      REAL AR(NM,N),AI(NM,N),D(N),E(N),E2(N),TAU(2,N)
      REAL F,G,H,FI,GI,HH,SI,SCALE,PYTHAG
C
C     THIS SUBROUTINE IS A TRANSLATION OF A COMPLEX ANALOGUE OF
C     THE ALGOL PROCEDURE TRED1, NUM. MATH. 11, 181-195(1968)
C     BY MARTIN, REINSCH, AND WILKINSON.
C     HANDBOOK FOR AUTO. COMP., VOL.II-LINEAR ALGEBRA, 212-226(1971).
C
C     THIS SUBROUTINE REDUCES A COMPLEX HERMITIAN MATRIX
C     TO A REAL SYMMETRIC TRIDIAGONAL MATRIX USING
C     UNITARY SIMILARITY TRANSFORMATIONS.
C
C     ON INPUT
C
C        NM MUST BE SET TO THE ROW DIMENSION OF TWO-DIMENSIONAL
C          ARRAY PARAMETERS AS DECLARED IN THE CALLING PROGRAM
C          DIMENSION STATEMENT.
C
C        N IS THE ORDER OF THE MATRIX.
C
C        AR AND AI CONTAIN THE REAL AND IMAGINARY PARTS,
C          RESPECTIVELY, OF THE COMPLEX HERMITIAN INPUT MATRIX.
C          ONLY THE LOWER TRIANGLE OF THE MATRIX NEED BE SUPPLIED.
C
C     ON OUTPUT
C
C        AR AND AI CONTAIN INFORMATION ABOUT THE UNITARY TRANS-
C          FORMATIONS USED IN THE REDUCTION IN THEIR FULL LOWER
C          TRIANGLES.  THEIR STRICT UPPER TRIANGLES AND THE
C          DIAGONAL OF AR ARE UNALTERED.
C
C        D CONTAINS THE DIAGONAL ELEMENTS OF THE THE TRIDIAGONAL MATRIX.
C
C        E CONTAINS THE SUBDIAGONAL ELEMENTS OF THE TRIDIAGONAL
C          MATRIX IN ITS LAST N-1 POSITIONS.  E(1) IS SET TO ZERO.
C
C        E2 CONTAINS THE SQUARES OF THE CORRESPONDING ELEMENTS OF E.
C          E2 MAY COINCIDE WITH E IF THE SQUARES ARE NOT NEEDED.
C
C        TAU CONTAINS FURTHER INFORMATION ABOUT THE TRANSFORMATIONS.
C
C     CALLS PYTHAG FOR  SQRT(A*A + B*B) .
C
C     QUESTIONS AND COMMENTS SHOULD BE DIRECTED TO BURTON S. GARBOW,
C     MATHEMATICS AND COMPUTER SCIENCE DIV, ARGONNE NATIONAL LABORATORY
C
C     THIS VERSION DATED AUGUST 1983.
C
C     ------------------------------------------------------------------
C
*_CS 17Jun91 patch: modified pythag routine
      real x, y
      PYTHAG(X,Y) = SQRT(X**2+Y**2)
 
      TAU(1,N) = 1.0E0
      TAU(2,N) = 0.0E0
C
      DO 100 I = 1,N
        D(I) = AR(I,I)
 100  CONTINUE
C     .......... FOR I=N STEP -1 UNTIL 1 DO -- ..........
      DO 500 II = 1,N
        I = N + 1 - II
        L = I - 1
        H = 0.0E0
        SCALE = 0.0E0
        IF ( L.GE.1 ) THEN
C     .......... SCALE ROW (ALGOL TOL THEN NOT NEEDED) ..........
          DO 120 K = 1,L
            SCALE = SCALE + ABS(AR(I,K)) + ABS(AI(I,K))
 120      CONTINUE
C
          IF ( SCALE.NE.0.0E0 ) THEN
C
            DO 130 K = 1,L
              AR(I,K) = AR(I,K)/SCALE
              AI(I,K) = AI(I,K)/SCALE
              H = H + AR(I,K)*AR(I,K) + AI(I,K)*AI(I,K)
 130        CONTINUE
C
            E2(I) = SCALE*SCALE*H
            G = SQRT(H)
            E(I) = SCALE*G
            F = PYTHAG(AR(I,L),AI(I,L))
C     .......... FORM NEXT DIAGONAL ELEMENT OF MATRIX T ..........
            IF ( F.EQ.0.0E0 ) THEN
              TAU(1,L) = -TAU(1,I)
              SI = TAU(2,I)
              AR(I,L) = G
              GOTO 150
            ELSE
              TAU(1,L) = (AI(I,L)*TAU(2,I)-AR(I,L)*TAU(1,I))/F
              SI = (AR(I,L)*TAU(2,I)+AI(I,L)*TAU(1,I))/F
              H = H + F*G
              G = 1.0E0 + G/F
              AR(I,L) = G*AR(I,L)
              AI(I,L) = G*AI(I,L)
              IF ( L.EQ.1 ) GOTO 300
              GOTO 150
            ENDIF
          ELSE
            TAU(1,L) = 1.0E0
            TAU(2,L) = 0.0E0
          ENDIF
        ENDIF
        E(I) = 0.0E0
        E2(I) = 0.0E0
        GOTO 400
 150    F = 0.0E0
C
        DO 200 J = 1,L
          G = 0.0E0
          GI = 0.0E0
C     .......... FORM ELEMENT OF A*U ..........
          DO 160 K = 1,J
            G = G + AR(J,K)*AR(I,K) + AI(J,K)*AI(I,K)
            GI = GI - AR(J,K)*AI(I,K) + AI(J,K)*AR(I,K)
 160      CONTINUE
C
          JP1 = J + 1
          IF ( L.GE.JP1 ) THEN
C
            DO 170 K = JP1,L
              G = G + AR(K,J)*AR(I,K) - AI(K,J)*AI(I,K)
              GI = GI - AR(K,J)*AI(I,K) - AI(K,J)*AR(I,K)
 170        CONTINUE
          ENDIF
C     .......... FORM ELEMENT OF P ..........
          E(J) = G/H
          TAU(2,J) = GI/H
          F = F + E(J)*AR(I,J) - TAU(2,J)*AI(I,J)
 200    CONTINUE
C
        HH = F/(H+H)
C     .......... FORM REDUCED A ..........
        DO 250 J = 1,L
          F = AR(I,J)
          G = E(J) - HH*F
          E(J) = G
          FI = -AI(I,J)
          GI = TAU(2,J) - HH*FI
          TAU(2,J) = -GI
C
          DO 220 K = 1,J
            AR(J,K) = AR(J,K) - F*E(K) - G*AR(I,K) + FI*TAU(2,K)
     &                + GI*AI(I,K)
            AI(J,K) = AI(J,K) - F*TAU(2,K) - G*AI(I,K) - FI*E(K)
     &                - GI*AR(I,K)
 220      CONTINUE
 250    CONTINUE
C
 300    DO 350 K = 1,L
          AR(I,K) = SCALE*AR(I,K)
          AI(I,K) = SCALE*AI(I,K)
 350    CONTINUE
C
        TAU(2,L) = -SI
 400    HH = D(I)
        D(I) = AR(I,I)
        AR(I,I) = HH
        AI(I,I) = SCALE*SQRT(H)
 500  CONTINUE
C
      RETURN
      END
